/*
 * ConfigurationGenerator.cpp
 *
 *  Created on: Oct 5, 2008
 *      Author: mariusz
 */

#include "ConfigurationGenerator.h"

namespace dma
{
	namespace core
	{

		ConfigurationGenerator::ConfigurationGenerator(HbMapPtr hbMin) :
			strategies_(new Strategies()), hbMin_(hbMin), peakRestrictionfilter_(new PeakRestrictionFilter(strategies_, hbMin_))
		{

		}

		ConfigurationGenerator::ConfigurationGenerator() :
			strategies_(new Strategies()), hbMin_(new HbMap()), peakRestrictionfilter_(new PeakRestrictionFilter(strategies_, hbMin_))
		{

		}

		ConfigurationGenerator::~ConfigurationGenerator()
		{
			delete peakRestrictionfilter_;
		}

		MarkerConfigurationPtrVectorPtr ConfigurationGenerator::generate(MarkerPtr marker,
				unsigned int minPersonsNumber,
				unsigned int maxPersonsNumber,
				MixProportionPtrMapPtr mpMap,
				double mpMaxDev
				)
		{

			unsigned int personNumber;


			// Setting up currently analyzed marker name
			marker_ = marker;

			// Creating output vector
			shared_ptr<std::vector<MarkerConfigurationPtr> > output(new std::vector<MarkerConfigurationPtr>());

			// Iterator for output vector
			std::vector<MarkerConfigurationPtr>::iterator it1;

			// Checking whether minimum person number is not greater than maximum person number given
			if (maxPersonsNumber < minPersonsNumber)
			{
				maxPersonsNumber = minPersonsNumber;
			}

			// Getting set of alleles found on marker
			unsigned int i;
			unsigned int size1 = marker->getAllelesNumber();
			shared_ptr<std::vector<AlleleBasePtr> > allelesOnMarker(new std::vector<AlleleBasePtr>());
			for (i = 0; i < size1; i++)
			{
				allelesOnMarker->push_back(marker->getAllele(i));
			}

			// Generating pairwise combinations
			shared_ptr<std::vector<AlleleBasePtrPair> > pairwiseCombinations;
			pairwiseCombinations = generatePairwiseCombinations(allelesOnMarker);

			// Iterating through possible number of persons involved in mixture
			for (personNumber = minPersonsNumber; personNumber <= maxPersonsNumber; personNumber++)
			{
				// Generating genotype configurations
				shared_ptr<std::vector<MarkerConfigurationPtr> > generated;
				generated = generateConfigurations(marker->getName(), pairwiseCombinations, personNumber);

				// Adding configurations to output configurations set
				std::vector<MarkerConfigurationPtr>::iterator it2;
				for (it2 = generated->begin(); it2 < generated->end(); it2++)
				{
					output->push_back(*it2);
				}
			}

			// Checking whether filtering impossible combinations is necessary
			if (strategies_->getImpossibleConfigurationStrategy() == Strategies::IGS_EXCLUDE_IMPOSSIBLE_GENOTYPES)
			{
				output = filterConfigurationsByAllelePresence(allelesOnMarker, output);
			}

			// Calculating peak area/height according to mix proportion
			for (it1 = output->begin(); it1 < output->end(); ++it1)
			{

				// Getting mixture proportion for given number of contributors
				MixProportionPtrMap::const_iterator mpIterator = mpMap->find( (*it1)->getLociNumber() );
				if ( mpIterator != mpMap->end() ) {

					// Performing deconvolution
					peakAllotter.performDeconvolution((*it1), marker, (*mpIterator).second );
				} else {

					// Mixture proportion has not been found for given number of contributors
					std::string errorMsg = "Mixture proportion is not available for mixture with " +
							lexical_cast<std::string>( (*it1)->getLociNumber() ) + " contributors";
					throw std::runtime_error(errorMsg);
				}
			}

			// Filtering combinations basing on heterozygotic balance
			output = peakRestrictionfilter_->filterByHb(marker->getName(), output);

			// Filtering combinations basing on mixture proportion
			output = peakRestrictionfilter_->filterByMp(output, mpMap, mpMaxDev);

			// Checking whether filtering equivalent combinations is necessary
			if (strategies_->getEquivalentsStrategy() == Strategies::ES_EXCLUDE_EQUIVALENTS)
			{

				output = filterEquivalentConfigurations(output);
			}

			return output;
		}

		MarkerConfigurationPtrVectorPtr ConfigurationGenerator::generateConfigurations(std::string markerName, shared_ptr<
				std::vector<AlleleBasePtrPair> > pairsCombinations, unsigned int personNumber)
		{

			unsigned int i, j, k;
			unsigned int s1;

			// Getting combinations number - pairwise combinations number
			unsigned int pcn = pairsCombinations->size();

			// Current Marker Configuration Set
			shared_ptr<std::vector<MarkerConfigurationPtr> > cmcs(new std::vector<MarkerConfigurationPtr>());

			// Working Marker Configuration Set
			shared_ptr<std::vector<MarkerConfigurationPtr> > wmcs(new std::vector<MarkerConfigurationPtr>());

			// Marker Configuration Set iterator
			std::vector<MarkerConfigurationPtr>::iterator mcIterator;

			try
			{
				// Generating combinations
				for (i = 0; i < personNumber; i++)
				{

					// Reseting working marker set
					wmcs->clear();

					if (cmcs->size() == 0)
					{

						// First iteration - market set empty

						// Iterating through all pairwise combinations
						for (j = 0; j < pcn; j++)
						{

							// Creating locus genotype for given allele pair
							LocusPtr locus(new Locus(pairsCombinations->at(j).first->cloneToPtr(),
									pairsCombinations->at(j).second->cloneToPtr()));

							// Adding locus genotype to marker
							MarkerConfigurationPtr mc(new MarkerConfiguration(markerName));
							mc->addLocus(locus);

							// Checking whether configuration has not yet been generated
							if (checkForConfigurationDuplicates(wmcs, mc) == true)
							{

								// If configuration already existed move to next combination
								continue;
							}

							// Adding configuration
							wmcs->push_back(mc);
						}
					}
					else
					{

						// Consecutive ( non-first ) iteration - marker set not empty

						// Iterating thought existing loci combinations
						for (mcIterator = cmcs->begin(); mcIterator < cmcs->end(); mcIterator++)
						{

							// Iterating through all pairwise combinations
							for (j = 0; j < pcn; j++)
							{

								// Creating locus genotype for given allele pair
								LocusPtr locus(new Locus(pairsCombinations->at(j).first->cloneToPtr(),
										pairsCombinations->at(j).second->cloneToPtr()));

								// "Cloning" marker genotype
								MarkerConfigurationPtr mc(new MarkerConfiguration(markerName));
								s1 = (*mcIterator)->getLociNumber();
								for (k = 0; k < s1; k++)
								{
									LocusPtr clonedLocus((*mcIterator)->getLocus(k)->cloneToPtr());
									mc->addLocus(clonedLocus);
								}

								// Adding new locus genotype to marker
								mc->addLocus(locus);

								// Checking whether configuration has not yet been generated
								if (checkForConfigurationDuplicates(wmcs, mc) == true)
								{

									// If configuration already existed move to next combination
									continue;
								}

								// Adding configuration
								wmcs->push_back(mc);

							}
						}
					}

					// Erasing current marker set
					cmcs->clear();

					// Copying working configuration to new configuration
					for (mcIterator = wmcs->begin(); mcIterator < wmcs->end(); mcIterator++)
					{
						cmcs->push_back(*mcIterator);
					}
				}
			} catch (AlleleTypeConflictException e)
			{

				throw AlleleTypeConflictException(e.getOldAlleleType(),
						e.getNewAlleleType(),
						e.getMarkerName(),
						"ConfigurationGenerator::generateConfigurations");
			}

			return cmcs;
		}

		MarkerConfigurationPtrVectorPtr ConfigurationGenerator::filterEquivalentConfigurations(shared_ptr<std::vector<
				MarkerConfigurationPtr> > configurations)
		{

			// Output configurations
			shared_ptr<std::vector<MarkerConfigurationPtr> > output(new std::vector<MarkerConfigurationPtr>());

			// Iterating through all configurations
			std::vector<MarkerConfigurationPtr>::iterator it1;
			for (it1 = configurations->begin(); it1 != configurations->end(); it1++)
			{

				// Checking whether equivalent to given configurations exists
				if (checkForConfigurationEquivalents(output, (*it1)) == false)
				{

					// Equivalent configurations have NOT been found
					output->push_back(*it1);
				}
				else
				{

					// Equivalent configurations have been found
					if (strategies_->getExcludedConfigurationsStrategy() == Strategies::ECS_ASSIGN_STATUS)
					{

						// Changing status if it was valid
						if ((*it1)->isValid() == true)
						{
							MCStatusExcEquivalencePtr es(new MCStatusExcEquivalence());
							(*it1)->setStatus(es);
						}

						output->push_back(*it1);
					}
				}
			}

			return output;
		}

		MarkerConfigurationPtrVectorPtr ConfigurationGenerator::filterConfigurationsByAllelePresence(shared_ptr<std::vector<
				AlleleBasePtr> > allelesOnMarker, shared_ptr<std::vector<MarkerConfigurationPtr> > configurations)
		{

			bool foundAllele;
			bool foundAllAlleles;
			shared_ptr<std::vector<AlleleBasePtr> > allelesInGenotype;
			std::vector<AlleleBasePtr>::iterator it1;
			std::vector<AlleleBasePtr>::iterator it2;

			shared_ptr<std::vector<MarkerConfigurationPtr> > output(new std::vector<MarkerConfigurationPtr>());

			// Iterating through marker genotypes
			std::vector<MarkerConfigurationPtr>::iterator it3;
			for (it3 = configurations->begin(); it3 < configurations->end(); it3++)
			{

				// Getting set of unique alleles present on marker
				allelesInGenotype = (*it3)->getUniqueAlleles();

				// Iterating through all alleles on sample marker
				foundAllAlleles = true;
				for (it1 = allelesOnMarker->begin(); it1 < allelesOnMarker->end(); it1++)
				{

					// Checking whether allele is present in marker genotype
					foundAllele = false;
					for (it2 = allelesInGenotype->begin(); it2 < allelesInGenotype->end(); it2++)
					{
						if ((*it1)->getAlleleValue() == (*it2)->getAlleleValue())
						{
							foundAllele = true;
							break;
						}
					}

					if (foundAllele == false)
					{
						foundAllAlleles = false;
						break;
					}
				}

				if (strategies_->getExcludedConfigurationsStrategy() == Strategies::ECS_EXCLUDE_FROM_COMBINATIONS)
				{

					// If configuration is not valid it is not added to output configurations set
					if (foundAllAlleles == true)
					{
						output->push_back((*it3));
					}
				}
				else
				{

					// If configuration is not valid non-valid status is assigned
					if (foundAllAlleles == false)
					{
						MCStatusExcAlleleMissingPtr es(new MCStatusExcAlleleMissing(*it1));
						(*it3)->setStatus(es);
					}

					output->push_back((*it3));
				}

			}

			return output;
		}

		AlleleBasePtrPairVectorPtr ConfigurationGenerator::generatePairwiseCombinations(AlleleBasePtrVectorPtr alleles)
		{

			unsigned int i, j, size1;
			AlleleBasePtrPair pair1;

			// Getting number of alleles in set
			size1 = alleles->size();

			// Creating output vector
			AlleleBasePtrPairVectorPtr output(new AlleleBasePtrPairVector());

			// Generating combinations
			for (i = 0; i < size1; i++)
			{

				for (j = 0; j < size1; j++)
				{

					pair1.first = alleles->at(i)->cloneToPtr();
					pair1.first->setFrequency(0.0);
					pair1.first->setPeakHeight(0.0);
					pair1.first->setPeakArea(0.0);
					pair1.second = alleles->at(j)->cloneToPtr();
					pair1.second->setFrequency(0.0);
					pair1.second->setPeakHeight(0.0);
					pair1.second->setPeakArea(0.0);
					output->push_back(pair1);
				}
			}

			return output;
		}

		bool ConfigurationGenerator::checkForConfigurationDuplicates(MarkerConfigurationPtrVectorPtr existingConfigurations,
				MarkerConfigurationPtr newConfiguration)
		{

			unsigned int i, size;
			bool flagFound = false;

			size = existingConfigurations->size();
			for (i = 0; i < size; i++)
			{
				if (existingConfigurations->at(i)->isEqual(newConfiguration) == true)
				{
					flagFound = true;
					break;
				}
			}

			return flagFound;
		}

		bool ConfigurationGenerator::checkForConfigurationEquivalents(MarkerConfigurationPtrVectorPtr existingConfigurations,
				MarkerConfigurationPtr newConfiguration)
		{

			unsigned int i, size;
			bool flagFound = false;

			size = existingConfigurations->size();
			for (i = 0; i < size; i++)
			{

				if (existingConfigurations->at(i)->isEquivalent(newConfiguration) == true)
				{
					// Checking whether second configuration is also a valid one
					if (existingConfigurations->at(i)->isValid() == true)
					{
						flagFound = true;
						break;
					}
				}
			}

			return flagFound;
		}

		StrategiesPtr ConfigurationGenerator::getStrategies()
		{

			return strategies_;
		}
	}
}
