/*
 * ConfigurationGenerator.h
 *
 *  Created on: Oct 5, 2008
 *      Author: mariusz
 */

#ifndef GENOTYPEGENERATOR_H_
#define GENOTYPEGENERATOR_H_

#include <vector>
#include <utility>
#include <string>
#include <sstream>
#include <boost/shared_ptr.hpp>
#include "../models/AlleleBase.h"
#include "../models/CharacterAllele.h"
#include "../models/NumericAllele.h"
#include "../models/Marker.h"
#include "../models/MarkerConfiguration.h"
#include "../models/statuses/MCStatusExcEquivalence.h"
#include "../models/statuses/MCStatusExcDuplication.h"
#include "../models/statuses/MCStatusExcAlleleMissing.h"
#include "PeakAllotter.h"
#include "Strategies.h"
#include "PeakRestrictionFilter.h"

using namespace dma;

namespace dma
{
	namespace core
	{

		class ConfigurationGenerator;
		typedef shared_ptr<ConfigurationGenerator> ConfigurationGeneratorPtr;

		/**
		 * Class responsible for generating all possible genotypes
		 * combinations given set alleles present on marker from sample
		 */
		class ConfigurationGenerator
		{
		protected:

			/**
			 * Currently analyzed marker
			 */
			MarkerPtr marker_;

			/**
			 * Strategies used for generation
			 */
			StrategiesPtr strategies_;

			/**
			 * Heterozygotic minimas for markers
			 */
			HbMapPtr hbMin_;

			/**
			 * Peak allotter u
			 */
			PeakAllotter peakAllotter;

			/**
			 * Class used for filtering combinations basing on peak area and height
			 */
			PeakRestrictionFilter* peakRestrictionfilter_;

		public:

			ConfigurationGenerator(HbMapPtr hbMin);

			/**
			 * Configuration generator without defined heterozygotic minimas.
			 *
			 * @return
			 */
			ConfigurationGenerator();

			/**
			 * Method generates set of locus genotype configurations for different persons numbers
			 *
			 * @param marker Marker analyzer
			 * @param minPersonsNumber Minimum number of persons, which may have contributed to mixture
			 * @param maxPersonsNumber Maximum number of persons, which may have contributed to mixture
			 * @return
			 */
			shared_ptr<std::vector<MarkerConfigurationPtr> > generate(MarkerPtr marker,
					unsigned int minPersonsNumber,
					unsigned int maxPersonsNumber,
					MixProportionPtrMapPtr mpMap,
					double mpMaxDev
					);

			virtual ~ConfigurationGenerator();

			/**
			 * Retrieve currently used strategies
			 * @return
			 */
			StrategiesPtr getStrategies();

		protected:

			/**
			 * Method generates all possible combinations of allele pairs given number of person in mixture
			 *
			 * @param markerName
			 * @param pairsCombinations
			 * @param personNumber
			 * @return
			 */
			MarkerConfigurationPtrVectorPtr generateConfigurations(std::string markerName, shared_ptr<std::vector<
					AlleleBasePtrPair> > pairsCombinations, unsigned int personNumber);

			/**
			 * Method filters out configurations which are equivalent
			 * @param configurations
			 * @return
			 */
			MarkerConfigurationPtrVectorPtr
			filterEquivalentConfigurations(MarkerConfigurationPtrVectorPtr configurations);

			/**
			 * Method filters genotype configurations which are impossible due to of alleles from sample marker
			 *
			 * @param allelesOnMarker
			 * @param configurations
			 * @return
			 */
			MarkerConfigurationPtrVectorPtr
			filterConfigurationsByAllelePresence(AlleleBasePtrVectorPtr allelesOnMarker,
					MarkerConfigurationPtrVectorPtr configurations);

			/**
			 * Method generates of all possible allele pairs from given allele set
			 *
			 * @param alleles
			 * @return
			 */
			AlleleBasePtrPairVectorPtr generatePairwiseCombinations(AlleleBasePtrVectorPtr alleles);

			/**
			 * Method checks whether equivalent configuration is not present in already generated markers.
			 * Two configuration are said to be duplicated if the have the same loci with the same order
			 *
			 * @param existingConfigurations Already generated configurations
			 * @param newConfiguration New configuration
			 * @return
			 */
			bool checkForConfigurationDuplicates(shared_ptr<std::vector<MarkerConfigurationPtr> > existingConfigurations,
					MarkerConfigurationPtr newConfiguration);

			/**
			 * Method checks whether equivalent configuration is not present in already generated markers.
			 * Two configuration are said to be equivalent if the have the same loci with different order
			 *
			 * @param existingConfigurations Already generated configurations
			 * @param newConfiguration New configuration
			 * @return
			 */
			bool checkForConfigurationEquivalents(shared_ptr<std::vector<MarkerConfigurationPtr> > existingConfigurations,
					MarkerConfigurationPtr newConfiguration);


		};
	}
}

#endif /* GENOTYPEGENERATOR_H_ */
