/*
 * LrCalculator.cpp
 *
 *  Created on: Dec 11, 2008
 *      Author: mariusz
 */

#include "LrCalculator.h"

namespace dma
{
	namespace core
	{

		LrCalculator::LrCalculator(MarkerPtrVectorPtr populationData) :
			populationData_(populationData), profileFilter_(new ProfilesFilter())
		{
			profileFilter_->getStrategies()->setExcludedConfigurationsStrategy(Strategies::ECS_EXCLUDE_FROM_COMBINATIONS);
		}

		LrCalculator::~LrCalculator()
		{

		}

		void LrCalculator::setPopulationData(MarkerPtrVectorPtr populationData)
		{

			// Assigning through reset new population data
			populationData_ = populationData;
		}

		LikelihoodRatioPtr LrCalculator::calculateLr(SampleConfigurationPtr sampleConfiguration,
				HypothesisPtr hp,
				HypothesisPtr hd)
		{

			// Getting names of markers present in sample
			shared_ptr<std::vector<std::string> > markerNames = sampleConfiguration->getMarkerNames();
			std::vector<std::string>::iterator markerNamesIt;

			LikelihoodRatioPtr lr(new LikelihoodRatio());

			for (markerNamesIt = markerNames->begin(); markerNamesIt != markerNames->end(); markerNamesIt++)
			{

				// Fetching marker configurations
				MarkerConfigurationPtrVectorPtr markerConfigurations =
						sampleConfiguration->getMarkerConfigurations(*markerNamesIt);

				// Considering only numerical markers
				if (markerConfigurations->size() > 0)
				{

					if (markerConfigurations->at(0)->getType() == AlleleBase::NUMERIC)
					{

						// Calculating LR
						AlgExpPtr lrForMarker = calculateLrForMarker(markerConfigurations, *markerNamesIt, hp, hd);

						// Adding likelihood for marker to overall likelihood ratio
						lr->addLRForMarker(*markerNamesIt, lrForMarker);
					}
				}

			}

			return lr;
		}

		AlgExpPtr LrCalculator::calculateLrForMarker(MarkerConfigurationPtrVectorPtr markerConfigurations,
				std::string markerName,
				HypothesisPtr hp,
				HypothesisPtr hd)
		{

			AlgExpPtr lr(new AlgValue("0", 0.0));

			//		std::cout << "LrCalculator::calculateLrForMarker(): Marker name: " << markerName << std::endl;

			// Loop control variables
			unsigned int i;
			unsigned int size;

			// Local variables
			ProfilePtr profile;

			// Cloning marker configurations for usage in prosecutor's and defence's hypothesis
			MarkerConfigurationPtrVectorPtr mcHp = MarkerConfiguration::cloneMarkerConfigurations(markerConfigurations);
			MarkerConfigurationPtrVectorPtr mcHd = MarkerConfiguration::cloneMarkerConfigurations(markerConfigurations);

			//		MarkerConfigurationPrinter printer;
			//		std::cout << std::endl;
			//		std::cout << "HP marker configurations before filtering:" << std::endl;
			//		printer.print(mcHp);
			//		std::cout << "HD marker configurations before filtering:" << std::endl;
			//		printer.print(mcHd);
			//		std::cout << std::endl;

			///
			/// Filtering configurations basing on known profiles
			///

			// Filtering prosecutor's hypothesis
			size = hp->getNumberOfContributors();
			for (i = 0; i < size; i++)
			{
				if (hp->getContributor(i)->hasProfile() == true)
				{

					profile = hp->getContributor(i)->getProfile();
					profileFilter_ ->filterMarkerConfigurationsByKnownProfile(mcHp, markerName, profile);
				}
			}

			// Filtering defences's hypothesis
			size = hd->getNumberOfContributors();
			for (i = 0; i < size; i++)
			{
				if (hd->getContributor(i)->hasProfile() == true)
				{

					profile = hd->getContributor(i)->getProfile();
					profileFilter_ ->filterMarkerConfigurationsByKnownProfile(mcHd, markerName, profile);
				}
			}

			//		std::cout << "HP marker configurations after filtering:" << std::endl;
			//		printer.print(mcHp);
			//		std::cout << "HD marker configurations after filtering:" << std::endl;
			//		printer.print(mcHd);
			//		std::cout << std::endl;

			///
			/// Subtracting loci of known persons from marker configurations
			///

			MarkerConfigurationPtrVector::iterator itHp;
			MarkerConfigurationPtrVector::iterator itHd;
			LocusPtr locusInProfile;

			size = hp->getNumberOfContributors();
			for (itHp = mcHp->begin(); itHp != mcHp->end(); itHp++)
			{

				// Iterating through persons in hypothesis
				for (i = 0; i < size; i++)
				{

					if (hp->getContributor(i)->hasProfile() == true)
					{

						try
						{
							locusInProfile = hp->getContributor(i)->getProfile()->getMarker(markerName);
							(*itHp)->subtractLocus(locusInProfile);
						} catch (MarkerNotFoundException& e)
						{

							throw MarkerNotFoundException(e.getMarkerName(), "LrCalculator::calculateLrForMarker()");
						}
					}
				}
			}

			size = hd->getNumberOfContributors();
			for (itHd = mcHd->begin(); itHd != mcHd->end(); itHd++)
			{
				// Iterating through persons in hypothesis
				for (i = 0; i < size; i++)
				{

					if (hd->getContributor(i)->hasProfile() == true)
					{

						try
						{
							locusInProfile = hd->getContributor(i)->getProfile()->getMarker(markerName);
							(*itHd)->subtractLocus(locusInProfile);
						} catch (MarkerNotFoundException& e)
						{

							throw MarkerNotFoundException(e.getMarkerName(), "LrCalculator::calculateLrForMarker()");
						}
					}
				}
			}

			//		std::cout << "HP marker configurations after subtraction:" << std::endl;
			//		printer.print(mcHp);
			//		std::cout << "HD marker configurations after subtraction:" << std::endl;
			//		printer.print(mcHd);
			//		std::cout << std::endl;

			for (itHp = mcHp->begin(); itHp != mcHp->end(); itHp++)
			{
				if ((*itHp)->isValid() == false)
				{
					mcHp->erase(itHp);
					itHp--;
				}
			}
			for (itHd = mcHd->begin(); itHd != mcHd->end(); itHd++)
			{
				if ((*itHd)->isValid() == false)
				{
					mcHd->erase(itHd);
					itHd--;
				}
			}

			//		std::cout << "HP marker configurations after removal of invalid configurations:" << std::endl;
			//		printer.print(mcHp);
			//		std::cout << "HD marker configurations after removal of invalid configurations:" << std::endl;
			//		printer.print(mcHd);
			//		std::cout << std::endl;

			///
			/// Building up probabilities for hypothesis
			///
			AlgExpPtr lrHp = buildAEForMarkerConfigurations(mcHp);
			AlgExpPtr lrHd = buildAEForMarkerConfigurations(mcHd);
			lr.reset(new AlgOperDiv(lrHp, lrHd));

			return lr;
		}

		AlleleBasePtr LrCalculator::findAlleleInPopulationData(AlleleBasePtr searchedAllele) const
		{

			unsigned int i;
			unsigned int size = populationData_->size();

			for (i = 0; i < size; i++)
			{

				// Finding appropriate marker
				MarkerPtr marker = populationData_->at(i);
				if (marker->getName().compare(searchedAllele->getMarkerName()) == 0)
				{

					// Searching for allele
					try
					{

						return marker->getAlleleByValue(searchedAllele->getAlleleValue());
					} catch (AlleleNotFoundException e)
					{

						throw AlleleNotFoundException(searchedAllele->getMarkerName(),
								searchedAllele->getAlleleValue(),
								"LrCalculator::findAlleleInPopulationData");
					}

				}
			}

			throw MarkerNotFoundException(searchedAllele->getMarkerName(), "LrCalculator::findAlleleInPopulationData()");
		}

		AlgExpPtr LrCalculator::buildAEForMarkerConfigurations(MarkerConfigurationPtrVectorPtr mc)
		{

			unsigned int i;

			// Initially we have empty expression
			AlgExpPtr lr(new AlgValue("1", 1));

			unsigned int sizeMc = mc->size();
			if (sizeMc == 1)
			{

				lr = buildAEForMarkerConfiguration(mc->at(0));
			}
			else if (sizeMc == 2)
			{

				AlgExpPtr ex1 = buildAEForMarkerConfiguration(mc->at(0));
				AlgExpPtr ex2 = buildAEForMarkerConfiguration(mc->at(1));
				AlgOperAddPtr exAdd(new AlgOperAdd(ex1, ex2));
				lr = exAdd;
			}
			else if (sizeMc > 2)
			{

				AlgExpPtr ex1 = buildAEForMarkerConfiguration(mc->at(0));
				AlgExpPtr ex2 = buildAEForMarkerConfiguration(mc->at(1));
				AlgOperAddPtr exAdd(new AlgOperAdd(ex1, ex2));
				for (i = 2; i < sizeMc; i++)
				{
					AlgExpPtr ex3 = buildAEForMarkerConfiguration(mc->at(2));
					exAdd->addExp(ex3);
				}
				lr = exAdd;
			}

			return lr;
		}

		AlgExpPtr LrCalculator::buildAEForMarkerConfiguration(MarkerConfigurationPtr markerConfiguration)
		{

			unsigned int i;
			unsigned int size = markerConfiguration->getLociNumber();

			if (size == 0)
			{

				// No loci is present in marker configuration
				AlgValuePtr rv(new AlgValue("1", 1.0));
				return rv;
			}
			else if (size == 1)
			{

				// Single loci is present in marker configuration. LR is LR for locus
				return buildAEForLocus(markerConfiguration->getLocus(0));
			}
			else
			{

				// Multiple number of loci is present in marker configuration;
				double fac = boost::math::factorial<double>(size);
				AlgExpPtr ex1(new AlgValue(lexical_cast<std::string> (fac), fac));
				AlgExpPtr ex2 = buildAEForLocus(markerConfiguration->getLocus(0));
				AlgOperMultiPtr rv(new AlgOperMulti(ex1, ex2));

				for (i = 1; i < size; i++)
				{
					AlgExpPtr ex = buildAEForLocus(markerConfiguration->getLocus(i));
					rv->addExp(ex);
				}

				return rv;
			}
		}

		AlgExpPtr LrCalculator::buildAEForLocus(LocusPtr locus)
		{

			if (locus->getZygosity() == Locus::HOMOZYGOTE)
			{

				AlgValuePtr ex1 = findAlleleInPopulationData(locus->getAllele(0))->getFrequencyAsAlgValue();
				AlgValuePtr ex2 = findAlleleInPopulationData(locus->getAllele(1))->getFrequencyAsAlgValue();
				AlgOperMultiPtr rv(new AlgOperMulti(ex1, ex2));
				return rv;
			}
			else if (locus->getZygosity() == Locus::HETEROZYGOTE)
			{

				AlgValuePtr ex1(new AlgValue("2", 2.0));
				AlgValuePtr ex2 = findAlleleInPopulationData(locus->getAllele(0))->getFrequencyAsAlgValue();
				AlgValuePtr ex3 = findAlleleInPopulationData(locus->getAllele(1))->getFrequencyAsAlgValue();
				AlgOperMultiPtr rv(new AlgOperMulti(ex1, ex2));
				rv->addExp(ex3);
				return rv;
			}
			else
			{

				AlgValuePtr rv(new AlgValue("1", 1.0));
				return rv;
			}
		}
	}
}
