/*
 * LrCalculator.h
 *
 *  Created on: Dec 11, 2008
 *      Author: mariusz
 */

#ifndef LRCALCULATOR_H_
#define LRCALCULATOR_H_

#include <map>
#include "ProfilesFilter.h"
#include "../models/algebra/AlgExp.h"
#include "../models/algebra/AlgValue.h"
#include "../models/algebra/AlgOperAdd.h"
#include "../models/algebra/AlgOperMulti.h"
#include "../models/algebra/AlgOperDiv.h"
#include "../models/algebra/AlgOperSub.h"
#include "../models/LikelihoodRatio.h"
#include "../models/Marker.h"
#include "../models/SampleConfiguration.h"
#include "../models/Hypothesis.h"
#include "../TypeDefinitions.h"
#include <boost/shared_ptr.hpp>
#include <boost/math/special_functions/factorials.hpp>
#include <vector>

namespace dma
{
	namespace core
	{

		class LrCalculator;
		typedef shared_ptr<LrCalculator> LrCalculatorPtr;

		/**
		 * Class used for calculating LR ratio for given mixture sample.
s		 */
		class LrCalculator
		{
		private:

			/**
			 * Population data containing allele frequencies
			 */
			MarkerPtrVectorPtr populationData_;

			/**
			 * Filter used for exclusion of configurations basing on known profiles
			 */
			ProfilesFilterPtr profileFilter_;

		public:

			LrCalculator(MarkerPtrVectorPtr populationData);

			virtual ~LrCalculator();

			/**
			 * Method sets population data which will be used during calculation - allele frequency in population
			 *
			 * @param populationData
			 */
			void setPopulationData(MarkerPtrVectorPtr populationData);

			/**
			 * Method calculates likelihood ratio for possible configurations under given the prosecutor's hypothesis and the defense one
			 *
			 * @param sampleConfiguration
			 * @param hp Prosecutor's hypothesis
			 * @param hd Defence's hypothesis
			 * @return
			 */
			LikelihoodRatioPtr calculateLr(SampleConfigurationPtr sampleConfiguration, HypothesisPtr hp, HypothesisPtr hd);

			/**
			 * Method calculates likelihood ratio for given configurations set basing on defense and prosecutor's hypothesis
			 *
			 * @param markerConfigurations Set of different configurations for given marker
			 * @param markerName Name of marker under consideration
			 * @param hp Prosecutor's hypothesis
			 * @param hd Defence's hypothesis
			 * @return
			 */
			AlgExpPtr calculateLrForMarker(MarkerConfigurationPtrVectorPtr markerConfigurations,
					std::string markerName,
					HypothesisPtr hp,
					HypothesisPtr hd);


			/**
			 * Method searches population data for given allele. If allele is not found @class AlleleNotFoundException is generated
			 *
			 * @return
			 * @exception AlleleNotFoundException
			 */
			AlleleBasePtr findAlleleInPopulationData(AlleleBasePtr searchedAllele) const;

			/**
			 * Given set of marker configurations method generates algebraic expression representing likelihood ratio
			 *
			 * @param markerConfigurations
			 * @return
			 */
			AlgExpPtr buildAEForMarkerConfigurations(MarkerConfigurationPtrVectorPtr mc);

			/**
			 * Method generates algebraic expression representing probability of finding given marker configuration in population
			 *
			 * @param markerConfiguration
			 * @return
			 */
			AlgExpPtr buildAEForMarkerConfiguration(MarkerConfigurationPtr markerConfiguration);

			/**
			 * Method generates algebraic expression representing probability of finding given locus genotype in population
			 * @param locus
			 * @return
			 */
			AlgExpPtr buildAEForLocus(LocusPtr locus);

		};
	}
}

#endif /* LRCALCULATOR_H_ */
