#ifndef MIXANALYZER_H_
#define MIXANALYZER_H_

#include <map>
#include <vector>
#include "boost/shared_ptr.hpp"
#include "../TypeDefinitions.h"
#include "PersonNumberAnalyzer.h"
#include "ConfigurationGenerator.h"
#include "ProfilesFilter.h"
#include "../models/SampleConfiguration.h"
#include "../models/MixtureProportion.h"

namespace dma
{
	namespace core
	{


		class MixAnalyzer;
		typedef boost::shared_ptr<MixAnalyzer> MixAnalyzerPtr;

		class MixAnalyzer
		{

		public:

			/**
			 * Enumeration indicating basing on which approach allele configurations will be generated
			 *
			 *	- UNRESTICTED - Peak areas are NOT taken into account
			 *	- RESTRICTED - Peak areas are taken into account
			 */
			enum CONFIGURATION_GENERATION_METHOD
			{
				UNRESTICTED, RESTRICTED
			};

		private:

			/**
			 * Class implementing algorithm for calculating minimum number of persons which contributed to mixture
			 */
			PersonNumberAnalyzerPtr personNumberAnalyzer_;

			/**
			 *	Class implementing generator_ for pairwise allele combinations for marker
			 */
			ConfigurationGeneratorPtr configurationGenerator_;

			/**
			 * Class implementing filtering generated configurations by usage known profiles
			 */
			ProfilesFilterPtr profilesFilter_;

			/**
			 * Minimum values of HB - heterozygotic balance - for each of the markers from set
			 */
			HbMapPtr hbMin_;

			/**
			 * Mix proportions for given number of persons involved in mixture
			 */
			MixProportionPtrMapPtr mixProportions_;

			/**
			 * Maximum deviation of mix proportion across loci
			 */
			double maxMixProportionDeviation_;

		public:

			/**
			 *
			 * @param hbmin Heterozygotic balance minimums for markers
			 *
			 * @return
			 */
			MixAnalyzer(HbMapPtr hbmin);

			virtual ~MixAnalyzer();

			/**
			 * Method calculates minimum number of persons which must be involved to explain mixture
			 *
			 * @param sample
			 * @return
			 */
			unsigned int getNumberOfPersonsInvolved(SamplePtr sample);

			/**
			 * Method returns all possible configurations which can be generated for given mixture basing on given mixture proportion
			 *
			 * @param sample Sample being considered
			 * @param mixProportions Mixture proportions for each of
			 * @param maxMixProportionDeviation_ Maximum mixture proportion derivation
			 * @param maxPersonsNumber Maximum persons number which could be involed in mixture
			 * @param configurationGenerationMethod Variable defining which approach will be used - unrestricted or restricted
			 * @return
			 */
			SampleConfigurationPtr performDeconvolution(SamplePtr sample,
					MixProportionPtrMapPtr mixProportions,
					double maxMixProportionDeviation,
					unsigned int maxPersonsNumber,
					CONFIGURATION_GENERATION_METHOD configurationGenerationMethod);

			/**
			 * Method filters generated configurations basing on alleles from known profiles.
			 * For each configuration alleles for given marker found in profile must be also present in configuration.
			 * Only valid configurations are considered.
			 *
			 * @param unfiltred Unifiltred configurations
			 * @param knownProfile Profile known to the case
			 * @return
			 */
			void filterSampleByKnownProfile(SampleConfigurationPtr unfiltred, ProfilePtr knownProfile);

			/**
			 * Method returns heterozygote balance minimum for given marker
			 *
			 * @param markerName
			 * @return
			 * @exception MarkerNotFoundException
			 */
			double getHBmin(std::string markerName);

			/**
			 * Method sets value of heterozygote balance for given marker.
			 * If marker HB is not yet present new entry is created
			 * @param markerName
			 * @param hbmin
			 */
			void setHBmin(std::string markerName, double hbmin);

		};
	}
}

#endif /*MIXANALYZER_H_*/
