/*
 * PeakRestrictionFilter.cpp
 *
 *  Created on: Nov 12, 2008
 *      Author: mariusz
 */

#include "PeakRestrictionFilter.h"

namespace dma
{
	namespace core
	{

		PeakRestrictionFilter::PeakRestrictionFilter(StrategiesPtr strategies, HbMapPtr hbMin) :
			strategies_(strategies), peakProperty_(AlleleBase::PEAK_PROPERTY_AREA), hbMin_(hbMin)
		{

		}

		PeakRestrictionFilter::PeakRestrictionFilter(StrategiesPtr strategies) :
			strategies_(strategies), peakProperty_(AlleleBase::PEAK_PROPERTY_AREA), hbMin_(new HbMap())
		{

		}

		PeakRestrictionFilter::~PeakRestrictionFilter()
		{

		}

		MarkerConfigurationPtrVectorPtr PeakRestrictionFilter::filterByHb(std::string markerName,
				MarkerConfigurationPtrVectorPtr configurations)
		{

			unsigned int i, size;
			double hb;
			bool hbOnAllLociValid = false;

			// Creating output container
			MarkerConfigurationPtrVectorPtr output(new std::vector<MarkerConfigurationPtr>());

			// Getting heterozygotic balance for given marker
			double hbMin = this->getHBMin(markerName);

			// Performing check of heterozygotic balance
			std::vector<MarkerConfigurationPtr>::iterator it1;
			for (it1 = configurations->begin(); it1 < configurations->end(); it1++)
			{

				// Checking whether configuration is not invalid
				if ((*it1)->isValid() == true)
				{

					hbOnAllLociValid = true;

					// Checking whether heterozygotic balance is met for all loci in configuration
					size = (*it1)->getLociNumber();
					for (i = 0; i < size; i++)
					{

						// Calculating HB
						hb = (*it1)->getLocus(i)->calculateHb(peakProperty_);

						// Checking whether HB is greater than minimum for given marker
						if (hb < hbMin)
						{
							hbOnAllLociValid = false;
							break;
						}
					}

					if (hbOnAllLociValid == true)
					{

						// Configuration valid - adding to output container
						output->push_back((*it1));
					}
					else
					{

						// Heterozygotic balance is not met
						if (strategies_->getExcludedConfigurationsStrategy() == Strategies::ECS_ASSIGN_STATUS)
						{

							MCStatusExcHeterozygoticImbalancePtr
									es(new MCStatusExcHeterozygoticImbalance((*it1)->getLocus(i), hb));
							(*it1)->setStatus(es);
							output->push_back((*it1));
						}
					}
				}
				else
				{

					// Configuration is not valid - check for heterozygotic balance omitted
					output->push_back(*it1);
				}
			}

			return output;
		}

		MarkerConfigurationPtrVectorPtr PeakRestrictionFilter::filterByMp(MarkerConfigurationPtrVectorPtr configurations,
				MixProportionPtrMapPtr mpMap,
				double mpMaxDev)
		{

			unsigned int i, size;
			double actualMp;
			double expectedMp;
			bool mpOnAllLociValid = false;

			// Creating output container
			MarkerConfigurationPtrVectorPtr output(new MarkerConfigurationPtrVector());

			// Performing check of mixture proportion
			std::vector<MarkerConfigurationPtr>::iterator it1;
			for (it1 = configurations->begin(); it1 < configurations->end(); it1++)
			{

				// Checking whether configuration is not invalid
				if ((*it1)->isValid() == true)
				{

					mpOnAllLociValid = true;

					// Fetching number of loci within given genotype
					size = (*it1)->getLociNumber();

					// Fetching mixture proportion for given number of genotypes ( persons )
					MixProportionPtrMap::const_iterator mpIt = mpMap->find(size);
					MixtureProportionPtr mp;
					if (mpIt != mpMap->end())
					{

						mp = (*mpIt).second;
					}
					else
					{

						// Mixture proportion is not defined for given person number - generating exception
						std::string errorMessage = "Mixture proportion cannot be found for configuration with " + lexical_cast<
								std::string> (size) + " persons";
						throw std::runtime_error(errorMessage);
					}

					// Calculating sum of all alleles peak properties ( area or height )
					double totalRfu = 0;
					for (i = 0; i < size; i++)
					{
						if (peakProperty_ == AlleleBase::PEAK_PROPERTY_AREA)
						{

							totalRfu = totalRfu + (*it1)->getLocus(i)->getTotalAlleleArea();

						}
						else if (peakProperty_ == AlleleBase::PEAK_PROPERTY_HEIGHT)
						{

							totalRfu = totalRfu + (*it1)->getLocus(i)->getTotalAlleleHeight();
						}
					}

					// Checking whether mixture variation is not greater than expected one
					for (i = 0; i < size; i++)
					{

						// Getting expected mixture proportion
						expectedMp = mp->getContributorShare(i);

						// Calculating HB
						if (peakProperty_ == AlleleBase::PEAK_PROPERTY_AREA)
						{

							actualMp = (*it1)->getLocus(i)->getTotalAlleleArea() / totalRfu;
						}
						else if (peakProperty_ == AlleleBase::PEAK_PROPERTY_HEIGHT)
						{

							actualMp = (*it1)->getLocus(i)->getTotalAlleleHeight() / totalRfu;
						}

						// Checking whether actual mixture proportion is in range
						if ((actualMp < (expectedMp - mpMaxDev)) || (actualMp > (expectedMp + mpMaxDev)))
						{
							mpOnAllLociValid = false;
							break;
						}
					}

					// Decrementing counter if reached boundary
					if (i == size)
						i--;

					if (mpOnAllLociValid == true)
					{

						// Configuration valid - adding to output container
						output->push_back((*it1));
					}
					else
					{

						// Heterozygotic balance is not met
						if (strategies_->getExcludedConfigurationsStrategy() == Strategies::ECS_ASSIGN_STATUS)
						{

							MCStatusExcMixtureProportionPtr es(new MCStatusExcMixtureProportion((*it1)->getLocus(i),
									expectedMp,
									actualMp));
							(*it1)->setStatus(es);
							output->push_back((*it1));
						}
					}
				}
				else
				{

					// Configuration is not valid - check for heterozygotic balance omitted
					output->push_back(*it1);
				}
			}

			return output;
		}

		double PeakRestrictionFilter::getHBMin(std::string markerName)
		{

			std::map<std::string, double>::iterator it1;
			for (it1 = hbMin_->begin(); it1 != hbMin_->end(); it1++)
			{
				if ((*it1).first.compare(markerName) == 0)
				{
					return (*it1).second;
				}
			}

			// If marker with given name has not been found exception is throwned
			throw MarkerNotFoundException(markerName, "PeakRestrictionFilter::getHBmin");
		}

		StrategiesPtr PeakRestrictionFilter::getStrategies()
		{

			return strategies_;
		}
	}
}
