/*
 * ProfilesFilter.h
 *
 *  Created on: Dec 11, 2008
 *      Author: mariusz
 */

#ifndef PROFILESFILTER_H_
#define PROFILESFILTER_H_

#include <boost/shared_ptr.hpp>
#include "../models/SampleConfiguration.h"
#include "../models/MarkerConfiguration.h"
#include "../models/Profile.h"
#include "../models/statuses/MCStatusExcKnownProfile.h"
#include "Strategies.h"

namespace dma
{
	namespace core
	{

		class ProfilesFilter;
		typedef shared_ptr<ProfilesFilter> ProfilesFilterPtr;

		/**
		 * Class uses information regarding known profiles to filter out from generated combinations the ones for which
		 * locus from known profile is not present in loci within generated configurations. Only valid configurations are considered.
		 *
		 * Example:
		 * Marker configuration 1: 12 34 56
		 * Marker configuration 2: 12 56 78
		 * Locus from known profile: 34
		 *
		 * Marker configuration 1 will pass
		 * Marker configuration 2 will be excluded or invalidated
		 */
		class ProfilesFilter
		{
		private:

			/**
			 * Generation strategy
			 */
			StrategiesPtr strategies_;

		public:

			ProfilesFilter();

			virtual ~ProfilesFilter();

			/**
			 * Get strategies used with filter
			 * @return
			 */
			StrategiesPtr getStrategies() const;

			/**
			 * Method filters generated sample configuration basing on alleles from known profiles.
			 * For each configuration alleles for given marker found in profile must be also present in configuration.
			 * Only valid configurations are considered.
			 *
			 * @param unfiltred Unfiltred sample configuration
			 * @param knownProfile Profile known to the case
			 */
			void filterSampleByKnownProfile(SampleConfigurationPtr unfiltred, ProfilePtr knownProfile);

			/**
			 * Method filters generated marker configurations basing on alleles from known profiles.
			 * For each configuration alleles for given marker found in profile must be also present in configuration.
			 * Only valid configurations are considered.
			 *
			 * @remarks If no marker data is given in known profile configurations are not affected and now exception is throwned
			 * @param unfiltred Unfiltred set of marker configurations
			 * @param markerName Name of analyzed marker
			 * @param knownProfile Profile known to the case
			 * @return
			 */
			void filterMarkerConfigurationsByKnownProfile(MarkerConfigurationPtrVectorPtr unfiltred,
					std::string markerName,
					ProfilePtr knownProfile);

		};
	}
}

#endif /* PROFILESFILTER_H_ */
