/*
 * Strategies.h
 *
 *  Created on: Nov 17, 2008
 *      Author: mariusz
 */

#ifndef STRATEGIES_H_
#define STRATEGIES_H_

#include <boost/shared_ptr.hpp>

using namespace boost;

namespace dma
{
	namespace core
	{

		class Strategies;
		typedef shared_ptr<Strategies> StrategiesPtr;

		/**
		 * Class represents strategies used for generation of genotypes
		 */
		class Strategies
		{
		public:

			/**
			 * Strategy used for handling equivalent configurations i.e.:
			 *
			 * Configuration 1: AB CD
			 * Configuration 2: CD AB
			 */
			enum EQUIVALENTS_STRATEGY
			{
				ES_ALLOW_EQUIVALENTS, //!< ES_ALLOW_EQUIVALENTS
				ES_EXCLUDE_EQUIVALENTS
			//!< ES_EXCLUDE_EQUIVALENTS
			};

			/**
			 * Strategy used for handling genotypes in which not all alleles from mixture genotype are used
			 */
			enum IMPOSSIBLE_GENOTYPE_STRATEGY
			{
				IGS_ALLOW_IMPOSSIBLE_GENOTYPES, //!< IGS_ALLOW_IMPOSSIBLE_GENOTYPES
				IGS_EXCLUDE_IMPOSSIBLE_GENOTYPES
			//!< IGS_EXCLUDE_IMPOSSIBLE_GENOTYPES
			};

			/**
			 * Strategy used for dealing with excluded combinations
			 *
			 * 	- ECS_EXCLUDE_FROM_COMBINATIONS - Configurations excluded are deleted from combinations set and not considered anymore
			 * 	- ECS_ASSIGN_STATUS - Configurations excluded are not deleted, but their status is changed
			 */
			enum EXCLUDED_CONFIGURATIONS_STRATEGY
			{
				ECS_EXCLUDE_FROM_COMBINATIONS, //!< ECS_EXCLUDE_FROM_COMBINATIONS
				ECS_ASSIGN_STATUS
			//!< ECS_ASSIGN_STATUS
			};
		private:

			/**
			 * Strategy for handling excluded combinations
			 */
			EXCLUDED_CONFIGURATIONS_STRATEGY excludedConfigurationsStrategy_;

			/**
			 * Duplicates strategy to be used
			 */
			EQUIVALENTS_STRATEGY equivalentsStrategy_;

			/**
			 * Strategy regarding genotype configurations which are impossible due allele configuration on marker
			 */
			IMPOSSIBLE_GENOTYPE_STRATEGY impossibleConfigurationsStrategy_;

		public:

			Strategies(Strategies::EXCLUDED_CONFIGURATIONS_STRATEGY excludedConfigurationsStrategy = ECS_ASSIGN_STATUS,
					Strategies::EQUIVALENTS_STRATEGY equivalentsStrategy = ES_EXCLUDE_EQUIVALENTS,
					Strategies::IMPOSSIBLE_GENOTYPE_STRATEGY impossibleConfigurationsStrategy = IGS_EXCLUDE_IMPOSSIBLE_GENOTYPES);

			virtual ~Strategies();

			/**
			 * Set new duplicates handling strategy
			 *
			 * @param duplicatesStrategy
			 */
			void setEquivalentsStrategy(EQUIVALENTS_STRATEGY equivalentsStrategy);

			/**
			 * Get duplicates handling strategy
			 * @param duplicatesStrategy
			 */
			EQUIVALENTS_STRATEGY getEquivalentsStrategy() const;

			/**
			 * Set new impossible genotype strategy
			 *
			 * @param impossibleGenotypeStrategy
			 */
			void setImpossibleConfigurationStrategy(IMPOSSIBLE_GENOTYPE_STRATEGY impossibleConfigurationsStrategy);

			/**
			 * Get impossible genotype strategy
			 *
			 * @param impossibleGenotypeStrategy
			 */
			IMPOSSIBLE_GENOTYPE_STRATEGY getImpossibleConfigurationStrategy() const;

			/**
			 * Set new excluded configurations strategy
			 * @param excludedConfigurationsStrategy
			 */
			void setExcludedConfigurationsStrategy(EXCLUDED_CONFIGURATIONS_STRATEGY excludedConfigurationsStrategy);

			/**
			 * Get excluded configurations strategy
			 * @return
			 */
			EXCLUDED_CONFIGURATIONS_STRATEGY getExcludedConfigurationsStrategy() const;
		};
	}
}

#endif /* STRATEGIES_H_ */
