#ifndef ALLELEBASE_H_
#define ALLELEBASE_H_

#include <iostream>
#include <ostream>
#include <string>
#include <utility>
#include <vector>
#include <boost/lexical_cast.hpp>
#include <boost/shared_ptr.hpp>
#include "../utils/UtilFunc.h"
#include "algebra/AlgValue.h"

using boost::lexical_cast;
using boost::bad_lexical_cast;
using boost::shared_ptr;

namespace dma
{
	namespace core
	{

		class AlleleBase;
		typedef shared_ptr<AlleleBase> AlleleBasePtr;
		typedef std::vector< shared_ptr<AlleleBase> > AlleleBasePtrVector;
		typedef shared_ptr< std::vector< shared_ptr<AlleleBase> > > AlleleBasePtrVectorPtr;

		typedef std::pair<AlleleBasePtr, AlleleBasePtr> AlleleBasePtrPair;
		typedef std::vector< std::pair<AlleleBasePtr, AlleleBasePtr> > AlleleBasePtrPairVector;
		typedef shared_ptr< std::vector< std::pair<AlleleBasePtr, AlleleBasePtr> > > AlleleBasePtrPairVectorPtr;

		/**
		 * Class representing allele.
		 *
		 * Allele is characterized by it's:
		 * 	- value ( number of tandem repeats )
		 * 	- size ( number of base pairs )
		 * 	- peak height in RFU
		 * 	- peak area in RFU
		 */
		class AlleleBase
		{

		public:

			/**
			 * Allele types
			 * 	- NUMERIC: Allele for which value is represented by floating-point number - number of tandem repeats
			 * 	- CHARACTER: Allele for which value is represented i.e. by sex-character ( X and Y ) - AMEL marker
			 * 	- UNDEFINED
			 */
			enum ALLELE_TYPE
			{
				NUMERIC, CHARACTER, UNDEFINED
			};

			/**
			 *
			 */
			enum PEAK_PROPERTY
			{
				PEAK_PROPERTY_AREA, //!< PEAK_PROPERTY_AREA
				PEAK_PROPERTY_HEIGHT//!< PEAK_PROPERTY_HEIGHT
			};

			/**
			 *Type representation of a numeric allele as character value
			 */

			/**
			 * Numeric allele type represented as character value
			 */
			char static const ALLELE_TYPE_NUMERIC = 'N';

			/**
			 * Character allele type represented as character value
			 */
			char static const ALLELE_TYPE_CHARACTER = 'C';

			/**
			 * Undefined allele type represented as character value
			 */
			char static const ALLELE_TYPE_UNDEFINED = 'U';

		protected:

			/**
			 * Allele value - number of repeats or character casted to double value
			 */
			double alleleValue_;

			/**
			 * Allele size - number of base pairs
			 */
			double alleleSize_;

			/**
			 * Peak height - height of peak for given allele on EPG
			 */
			double peakHeight_;

			/**
			 * Peak area - area of peak for given allele on EPG
			 */
			double peakArea_;

			/**
			 * Allele type
			 */
			ALLELE_TYPE type_;

			/**
			 * Frequency of allele of given value in population
			 */
			double frequency_;

			/**
			 * Name of the marker on which allele is given
			 */
			std::string markerName_;

		protected:

			/**
			 * Constructor
			 *
			 * @param alleleValue Allele value - number of tandem repeats
			 * @param size Size of allele
			 * @param peakHeight Height of peak representing allele
			 * @param peakArea Area of peak representing allele
			 * @param type Type of allele - NUMERIC | CHARACTER | UNDEFINED
			 * @return
			 */
			AlleleBase(double alleleValue, double size, double peakHeight, double peakArea, ALLELE_TYPE type);

		public:

			virtual ~AlleleBase();

			/**
			 * Method clones given allele and returns it in a form of smart pointer
			 * @return
			 */
			virtual AlleleBasePtr cloneToPtr() const = 0;

			/**
			 *
			 * @return Method returns allele value
			 */
			double getAlleleValue() const;

			/**
			 * @return Method returns allele size
			 */
			double getAlleleSize() const;

			/**
			 * @return Method returns height of allele peak
			 */
			double getPeakHeight() const;

			/**
			 * @return Method returns area under peak
			 */
			double getPeakArea() const;

			/**
			 * @return Method sets height of allele peak
			 */
			void setPeakHeight(double peakHeight);

			/**
			 * @return Method serts area under peak
			 */
			void setPeakArea(double peakArea);

			/**
			 * @return Method returns frequency of occurrence of given allele in population
			 */
			double getFrequency() const;

			/**
			 *
			 * @return Method returns frequency of occurrence of given allele in population given as @class AlgValue
			 */
			AlgValuePtr getFrequencyAsAlgValue() const;

			/**
			 * Method sets frequency of occurrence of given allele in population
			 *
			 * @param frequency
			 */
			void setFrequency(double frequency);

			/**
			 *
			 * @return Name of marker on which allele is given
			 */
			std::string getMarkerName() const;

			/**
			 * Method sets marker name
			 *
			 * @param markerName
			 */
			void setMarkerName(std::string markerName);

			/**
			 * @return Get type of allele as enumeration value
			 */
			ALLELE_TYPE getType() const;

			/**
			 * @return Get allele type as character
			 */
			char getTypeAsChar() const;

			/**
			 * @return Get allele type as character
			 */
			static char getTypeAsChar(ALLELE_TYPE alleleType);

			/**
			 * @return Method returns allele value as string
			 */
			virtual std::string getAlleleValueAsString() const = 0;

			/**
			 * @return Method returns all data regarding allele as string
			 */
			virtual std::string toString() const = 0;

		};
	}
}
#endif /*ALLELBASE_H_*/
