#include "Locus.h"

namespace dma
{
	namespace core
	{

		Locus::Locus(AlleleBasePtr allele1, AlleleBasePtr allele2) :
			container_(new std::vector<AlleleBasePtr>()), markerName_("")
		{

			addAllele(allele1);
			addAllele(allele2);
		}

		Locus::Locus(Locus const& copy)
		{

			// Initializing container
			container_.reset(new std::vector<AlleleBasePtr>());

			// Copying alleles from copied locus
			addAllele(copy.getAllele(0));
			addAllele(copy.getAllele(1));

			// Setting marker name
			setMarkerName(copy.getMarkerName());
		}

		Locus& Locus::operator =(const Locus& rhs)
		{

			if (this != &rhs)
			{

				// Reseting container
				container_.reset(new std::vector<AlleleBasePtr>());

				// Copying alleles from copied locus
				addAllele(rhs.getAllele(0));
				addAllele(rhs.getAllele(1));

				// Setting marker name
				setMarkerName(rhs.getMarkerName());

			}

			return *this;
		}

		Locus::~Locus()
		{
		}

		Locus* Locus::cloneToPtr() const
		{

			// Pointer to cloned object
			Locus* newLocus = new Locus(getAllele(0)->cloneToPtr(), getAllele(1)->cloneToPtr());
			newLocus->setMarkerName(getMarkerName());
			return newLocus;
		}

		LocusPtr Locus::cloneToSmartPtr() const
		{

			LocusPtr rv(cloneToPtr());
			return rv;
		}

		unsigned int Locus::getAlleleNumber() const
		{

			return container_->size();
		}

		AlleleBase::ALLELE_TYPE Locus::getType() const
		{

			if (container_->size() > 0)
			{

				return getAllele(0)->getType();
			}
			else
			{

				return AlleleBase::UNDEFINED;
			}
		}

		Locus::ZYGOSITY Locus::getZygosity() const
		{

			if (container_->size() == 2)
			{

				if (container_->at(0)->getAlleleValue() == container_->at(1)->getAlleleValue())
				{

					// Values of both alleles are the same - homozygote
					return Locus::HOMOZYGOTE;

				}
				else
				{

					// Values of both alleles are different - heterozygote
					return Locus::HETEROZYGOTE;

				}
			}
			else
			{

				// Zygosity cannot be checked since genotype is not complete
				return Locus::UNDEFINED;
			}
		}

		unsigned int Locus::getZygosityAsNumber() const
		{

			if (getZygosity() == Locus::HETEROZYGOTE)
			{
				return 1;
			}
			else if (getZygosity() == Locus::HOMOZYGOTE)
			{
				return 2;
			}
			else
			{
				return 0;
			}
		}

		void Locus::addAllele(AlleleBasePtr allele)
		{

			if (container_->size() == 0)
			{

				// Setting name of the marker for added allele
				allele->setMarkerName(getMarkerName());

				// Adding allele and setting type and zygosity of locus
				container_->push_back(allele);
			}
			else if (container_->size() == 1)
			{

				// Checking whether alleles are of the same type
				if (getType() == allele->getType())
				{

					if (allele->getAlleleValue() == container_->at(0)->getAlleleValue())
					{

						// Adding allele to container
						container_->push_back(allele);

						// Setting name of the marker for added allele
						allele->setMarkerName(getMarkerName());

					}
					else if (allele->getAlleleValue() > container_->at(0)->getAlleleValue())
					{

						// Second allele has greater molecular weight ( value ) then first, inserting at front
						container_->push_back(allele);

						// Setting name of the marker for added allele
						allele->setMarkerName(getMarkerName());

					}
					else
					{

						// Second allele has smaller molecular weight ( value ) then first, inserting at front
						container_->insert(container_->begin(), allele);

						// Setting name of the marker for added allele
						allele->setMarkerName(getMarkerName());

					}
				}
				else
				{

					// Alleles are not of the same type
					throw AlleleTypeConflictException(getType(), allele->getType(), getMarkerName(), "");
				}
			}
			else
			{
				throw OverflowExcpetion(container_->size(), "Locus::addAllele()::container_");
			}
		}

		AlleleBasePtr Locus::getAllele(unsigned int index) const
		{

			if (index == 0 || index == 1)
			{

				return container_->at(index);
			}
			else
			{

				throw OutOfBoundsException(index, 2, "Locus::getAllele()::container_");
			}
		}

		bool Locus::isAllelePresent(double alleleValue) const
		{

			if (container_->size() == 2)
			{

				if (getAllele(0)->getAlleleValue() == alleleValue || getAllele(1)->getAlleleValue() == alleleValue)
				{

					return true;
				}
				else
				{

					return false;
				}
			}
			else
			{

				return false;
			}
		}

		double Locus::calculateHb(AlleleBase::PEAK_PROPERTY peakProperty) const
		{

			if (peakProperty == AlleleBase::PEAK_PROPERTY_AREA)
			{
				return calculateHbByPeakArea();
			}
			else
			{
				return calculateHbByPeakHeight();
			}
		}

		double Locus::calculateHbByPeakHeight() const
		{

			double hb;

			if (container_->size() == 2)
			{

				if (getAllele(0)->getPeakHeight() <= getAllele(1)->getPeakHeight())
				{
					hb = getAllele(0)->getPeakHeight() / getAllele(1)->getPeakHeight();
				}
				else
				{
					hb = getAllele(1)->getPeakHeight() / getAllele(0)->getPeakHeight();

				}
			}

			return hb;
		}

		double Locus::calculateHbByPeakArea() const
		{

			double hb;

			if (container_->size() == 2)
			{

				if (getAllele(0)->getPeakArea() <= getAllele(1)->getPeakArea())
				{
					hb = getAllele(0)->getPeakArea() / getAllele(1)->getPeakArea();
				}
				else
				{
					hb = getAllele(1)->getPeakArea() / getAllele(0)->getPeakArea();

				}
			}

			return hb;

		}

		unsigned int Locus::getHash() const
		{
			double hash = 0.0;
			double allele1 = 0.0;
			double allele2 = 0.0;
			double prime1 = 47;
			double prime2 = 67;

			if (container_->size() == 2)
			{

				allele1 = getAllele(0)->getAlleleValue();
				allele2 = getAllele(1)->getAlleleValue();
				hash = allele1 * prime1 + allele2 * prime2;
			}
			else
			{
				return 0;
			}

			return floor(hash);
		}

		std::string Locus::getMarkerName() const
		{

			return markerName_;
		}

		void Locus::setMarkerName(std::string markerName)
		{

			markerName_ = markerName;
			if (container_->size() == 2)
			{

				getAllele(0)->setMarkerName(markerName);
				getAllele(1)->setMarkerName(markerName);
			}
		}

		std::string Locus::toString() const
		{

			std::string buffer;

			buffer = getAllele(0)->getAlleleValueAsString();
			buffer = buffer + " ";
			buffer = buffer + getAllele(1)->getAlleleValueAsString();

			return buffer;
		}

		double Locus::getTotalAlleleArea() const
		{

			double sum = 0.0;

			unsigned int i;
			unsigned int size = container_->size();
			for (i = 0; i < size; i++)
			{
				sum = sum + container_->at(i)->getPeakArea();
			}

			return sum;
		}

		double Locus::getTotalAlleleHeight() const
		{

			double sum = 0.0;

			unsigned int i;
			unsigned int size = container_->size();
			for (i = 0; i < size; i++)
			{
				sum = sum + container_->at(i)->getPeakHeight();
			}

			return sum;
		}
	}
}
