#ifndef LOCUSGENOTYPE_H_
#define LOCUSGENOTYPE_H_

#include <stdexcept>
#include <vector>
#include <cmath>
#include <map>
#include <boost/shared_ptr.hpp>
#include "AlleleBase.h"
#include "NumericAllele.h"
#include "../exceptions/AlleleTypeConflictException.h"
#include "../exceptions/OutOfBoundsException.h"
#include "../exceptions/OverflowExcpetion.h"

namespace dma
{
	namespace core
	{

		class Locus;

		/**
		 * Smart pointer for class Locus;
		 */
		typedef shared_ptr<Locus> LocusPtr;

		/**
		 * Map of locus configurations for markers
		 */
		typedef std::map<std::string, LocusPtr> LocusPtrMap;

		/**
		 * Smart pointer map of locus configurations for markers
		 */
		typedef shared_ptr<LocusPtrMap> LocusPtrMapPtr;

		/**
		 * Class representing locus configuration ( allele combination ) which is present
		 * at locus for single person ( non-mixture sample )
		 *
		 * For empty locus ( no alleles present ) zygosity is undefined
		 */
		class Locus
		{

		public:

			/**
			 * Enumeration used for distinguishing locus zygosity
			 * 	- Homozygote	Both alleles are of the same value
			 * 	- Heterozygote	Alleles present on locus are of different value
			 * 	- Undefined		Zygosity is undefined, i.e. no alleles are present
			 */
			enum ZYGOSITY
			{
				HOMOZYGOTE, HETEROZYGOTE, UNDEFINED
			};

		private:

			/**
			 * Container for alleles in given locus
			 */
			shared_ptr< std::vector< AlleleBasePtr> > container_;

			/**
			 * Name of marker on which locus is present
			 */
			std::string markerName_;

		public:


			/**
			 * Create locus with given two alleles
			 *
			 * @param allele1
			 * @param allele2
			 * @return
			 */
			Locus(AlleleBasePtr allele1, AlleleBasePtr allele2);

			Locus(const Locus& copy);

			virtual ~Locus();

			Locus& operator =(const Locus& rhs);

			/**
			 * Create a self-copy ( clone ) and returns it as a pointer
			 */
			Locus* cloneToPtr() const;

			/**
			 * Create a self-copy ( clone ) and returns it as a smart pointer
			 * @return
			 */
			LocusPtr cloneToSmartPtr() const;

			/**
			 * @return Method returns locus type based on type of alleles stored
			 */
			AlleleBase::ALLELE_TYPE getType() const;

			/**
			 * @return Method returns zygosity at given locus
			 */
			ZYGOSITY getZygosity() const;

			/**
			 * Method returns number of alleles of the same value
			 * @return
			 */
			unsigned int getZygosityAsNumber() const;

			unsigned int getAlleleNumber() const;

			/**
			 * Add allele to a locus.
			 *
			 * @param allele Allele to-be-added to genotype
			 * @exception OverflowExcpetion Exception is generated when 3-th allele is added to genotype
			 * @exception AlleleTypeConflictException Exception is generated when second allele differ in type from first allele
			 */
			void addAllele(AlleleBasePtr allele);

			/**
			 *
			 *
			 * @param index Index of allele to-be-fetched ( first allele has index 0 )
			 * @return
			 * @exception OutOfBoundsException
			 */
			AlleleBasePtr getAllele(unsigned int index) const;

			/**
			 * Method checks whether allele with given value is within locus
			 *
			 * @param alleleValue
			 * @return
			 */
			bool isAllelePresent(double alleleValue) const;

			/**
			 * Calculate heterozygote balance
			 *
			 * @param hbCalculationMethod
			 * @return
			 */
			double calculateHb( AlleleBase::PEAK_PROPERTY peakProperty) const;

			/**
			 *
			 * Get sum of all allele areas
			 * @return
			 */
			double getTotalAlleleArea() const;

			/**
			 * Get sum of all allele heights
			 * @return
			 */
			double getTotalAlleleHeight() const;

			/**
			 * Method generate hash code for given allele configuration
			 *
			 * Hash value = allele1 * 47 + allele2  * 67
			 * @return int
			 */
			unsigned int getHash() const;

			/**
			 * Method returns name of marker on which locus is present
			 * @return
			 */
			std::string getMarkerName() const;

			/**
			 * Marker sets name of marker on which locus is present.
			 * Method also set marker name in for alleles found on locus
			 *
			 * @param markerName
			 */
			void setMarkerName(std::string markerName);

			/**
			 * Return information regarding loci as string
			 * @return
			 */
			std::string toString() const;

		private:

			/**
			 * Calculate heterozygotic balance basing on peak height
			 *
			 * @return
			 */
			double calculateHbByPeakHeight() const;

			/**
			 * Calculate homozygotic balance basing on peak area
			 *
			 * @return
			 */
			double calculateHbByPeakArea() const;
		};
	}
}

#endif /*LOCUSGENOTYPE_H_*/
