#include "Marker.h"

namespace dma
{
	namespace core
	{

		Marker::Marker(std::string name) :
			name_(name), alleles_(new std::vector<AlleleBasePtr>()), hbmin_(0)
		{

		}

		Marker::Marker(const Marker& copy) :
			name_(copy.name_), alleles_(new std::vector<AlleleBasePtr>()), hbmin_(copy.hbmin_)
		{

			// Copying alleles
			unsigned int i, size;
			size = copy.alleles_->size();
			for (i = 0; i < size; i++)
			{
				alleles_->push_back(copy.alleles_->at(i)->cloneToPtr() );
			}
		}

		Marker&Marker::operator=(const Marker& rhs)
		{

			if (this != &rhs)
			{

				// Copying class attributes
				name_ = rhs.name_;
				hbmin_ = rhs.hbmin_;

				// Reseting alleles container and copying alleles
				alleles_.reset(new std::vector<AlleleBasePtr>());
				unsigned int i, size;
				size = rhs.alleles_->size();
				for (i = 0; i < size; i++)
				{
					alleles_->push_back(rhs.alleles_->at(i)->cloneToPtr() );
				}
			}

			return *this;
		}

		Marker::~Marker()
		{

		}

		void Marker::addAllele(AlleleBasePtr allele)
		{

			// Checking whether type of alleles already present on marker is the same as added one
			if (alleles_->size() > 0 && getType() != allele->getType())
			{

				throw AlleleTypeConflictException(getType(), allele->getType(), getName(), "");
			}

			// Setting name of marker for allele
			allele->setMarkerName(getName());

			// Adding allele to container
			alleles_->push_back(allele);

		}

		unsigned int Marker::getAllelesNumber() const
		{
			return alleles_->size();
		}

		std::string Marker::getName() const
		{
			return name_;
		}

		AlleleBasePtr Marker::getAllele(unsigned int index) const
		{
			if (index < alleles_->size())
			{
				return alleles_->at(index);
			}
			else
			{
				throw OutOfBoundsException(index, alleles_->size(), "Marker::getAllele()::alleles_");
			}
		}

		AlleleBasePtr Marker::getAlleleByValue(double alleleValue) const
		{

			std::vector<AlleleBasePtr>::iterator it1;
			for (it1 = alleles_->begin(); it1 != alleles_->end(); it1++)
			{

				if ((*it1)->getAlleleValue() == alleleValue)
				{
					return (*it1);
				}
			}

			throw AlleleNotFoundException(getName(), alleleValue, "");
		}

		AlleleBase::ALLELE_TYPE Marker::getType() const
		{

			if (alleles_->size() > 0)
			{

				return alleles_->at(0)->getType();
			}
			else
			{

				return AlleleBase::UNDEFINED;
			}
		}

		double Marker::calculateHb(unsigned index1, unsigned index2, AlleleBase::PEAK_PROPERTY peakProperty)
		{

			// Checking whether indices are valid
			if (index1 >= alleles_->size())
			{
				throw OutOfBoundsException(index1, alleles_->size(), "Marker::calculateHb()::alleles_");
			}
			if (index2 >= alleles_->size())
			{
				throw OutOfBoundsException(index2, alleles_->size(), "Marker::calculateHb()::alleles_");
			}

			// Calculating heterozygote balance
			if ( peakProperty == AlleleBase::PEAK_PROPERTY_AREA ) {

				return calculateHbByPeakArea(index1, index2);
			} else if ( peakProperty == AlleleBase::PEAK_PROPERTY_HEIGHT ) {

				return calculateHbByPeakHeight(index1, index2);

			} else {

				return 0;
			}
		}

		double Marker::calculateHbByPeakArea(unsigned index1, unsigned index2)
		{

			double hb;

			if (alleles_->at(index1)->getPeakArea() <= alleles_->at(index2)->getPeakArea())
			{
				hb = alleles_->at(index1)->getPeakArea() / alleles_->at(index2)->getPeakArea();
			}
			else
			{
				hb = alleles_->at(index2)->getPeakArea() / alleles_->at(index1)->getPeakArea();
			}

			return hb;

		}

		double Marker::calculateHbByPeakHeight(unsigned index1, unsigned index2)
		{

			double hb;

			if (alleles_->at(index1)->getPeakHeight() <= alleles_->at(index2)->getPeakHeight())
			{
				hb = alleles_->at(index1)->getPeakHeight() / alleles_->at(index2)->getPeakHeight();
			}
			else
			{
				hb = alleles_->at(index2)->getPeakHeight() / alleles_->at(index1)->getPeakHeight();
			}

			return hb;
		}

		std::string Marker::toString() const
		{

			std::string buffer = "Marker " + getName() + ":\n";

			unsigned int i;
			unsigned int size = getAllelesNumber();
			for (i = 0; i < size; i++)
			{
				buffer = buffer + getAllele(i)->toString() + "\n";
			}

			return buffer;
		}

		bool Marker::isAllelePresent(double alleleValue) const
		{

			bool output = false;

			std::vector<AlleleBasePtr>::iterator it1;
			for (it1 = alleles_->begin(); it1 != alleles_->end(); it1++)
			{

				if ((*it1)->getAlleleValue() == alleleValue)
				{
					output = true;
					break;
				}
			}

			return output;
		}

	}
}
