#ifndef MARKER_H_
#define MARKER_H_

#include <string>
#include <vector>
#include <map>
#include "AlleleBase.h"
#include "../exceptions/OutOfBoundsException.h"
#include "../exceptions/AlleleNotFoundException.h"
#include "../exceptions/AlleleTypeConflictException.h"
#include <boost/shared_ptr.hpp>

using namespace boost;

namespace dma
{
	namespace core
	{

		class Marker;

		/**
		 * Smart pointer of @class Marker
		 */
		typedef shared_ptr<Marker> MarkerPtr;

		/**
		 * Vector of object of type @class Marker
		 */
		typedef std::vector<MarkerPtr> MarkerPtrVector;

		/**
		 * Smart pointer to vector with objects Marker class
		 */
		typedef shared_ptr<std::vector<MarkerPtr> > MarkerPtrVectorPtr;

		/**
		 * Map with object of Marker class identified by strings ( marker names )
		 */
		typedef std::map<std::string, MarkerPtr> MarkerPtrMap;

		/**
		 * Smart pointer to map with object of Marker class identified by strings ( marker names )
		 */
		typedef shared_ptr<std::map<std::string, MarkerPtr> > MarkerPtrMapPtr;

		/**
		 * Class represents single marker from mixed DNA sample
		 *
		 * Unlike locus marker can have more than 2 alleles present
		 */
		class Marker
		{
		private:

			/**
			 * Name if the marker. Used to distinguish markers from each other
			 */
			std::string name_;

			/**
			 * Set of alleles which are present on given marker
			 */
			shared_ptr<std::vector<AlleleBasePtr> > alleles_;

			/**
			 * Minimum value of heterozygote balance for given locus/marker
			 */
			double hbmin_;

		public:

			/**
			 * Constructor
			 *
			 * @param name Marker name i.e. AMEL
			 */
			Marker(std::string name);

			/**
			 * Copy constructor
			 *
			 * @param copy
			 * @return
			 */
			Marker(const Marker& copy);

			/**
			 * Assignment operator
			 *
			 * @param rhs
			 * @return
			 */
			Marker& operator=(const Marker& rhs);

			virtual ~Marker();

			/**
			 * Add allele to marker. Method may throw @class AlleleTypeConflictException if
			 * the type of added allele differ from type of alleles already present on marker
			 *
			 * @param allele Added allele
			 * @exception AlleleTypeConflictException
			 */
			void addAllele(AlleleBasePtr allele);

			/**
			 *
			 * @return Number of alleles which are present on given marker
			 */
			unsigned int getAllelesNumber() const;

			/**
			 * Retrieve allele by index. First allele has index 0. If attempt to fetch allele with non-existing index is done
			 * @class OutOfBoundsException is throwned.
			 *
			 * @param index
			 * @return
			 * @exception OutOfBoundsException
			 */
			AlleleBasePtr getAllele(unsigned int index) const;

			/**
			 * Retriever allele by given value. If allele with desired value does not exists
			 * on marker @class AlleleNotFoundException is throwned.
			 *
			 *
			 * @param alleleValue
			 * @return
			 * @exception AlleleNotFoundException
			 */
			AlleleBasePtr getAlleleByValue(double alleleValue) const;

			/**
			 *
			 *
			 * @return Get marker name
			 */
			std::string getName() const;

			/**
			 * Retrieves type of marker ( numerical or character ) based on alleles found on marker.
			 * If no alleles are present on marker the returned type is UNDEFINED
			 *
			 * @return
			 */
			AlleleBase::ALLELE_TYPE getType() const;

			/**
			 * Method calculates heterozygote balance between two given alleles.
			 * Result is <0,1>
			 */

			/**
			 * Method calculates heterozygotic balance between two alleles using selected peak property.
			 * Method may throw @class OutOfBoundsException if one of the alleles connot be retrieved.
			 *
			 * @param index1 Index of first allele to be considered. Note: Index of first allele on marker is 0
			 * @param index2 Index of second allele to be considered. Note: Index of first allele on marker is 0
			 * @param peakProperty Property of peak ( area, height ) on which calculation will be based
			 * @return Method returns value from interval <0,1>
			 * @exception OutOfBoundsException
			 */
			double calculateHb(unsigned index1, unsigned index2, AlleleBase::PEAK_PROPERTY peakProperty);

			/**
			 * Method checks whether allele with given value is on marker
			 *
			 * @param alleleValue
			 * @return
			 */
			bool isAllelePresent(double alleleValue) const;

			/**
			 * Method displays marker names and alleles which are present on marker.
			 *
			 * Output
			 * Marker EXAMPLE:
			 * Allel1
			 * Allel2
			 *get
			 * @return
			 */
			std::string toString() const;

		private:

			double calculateHbByPeakArea(unsigned index1, unsigned index2 );

			double calculateHbByPeakHeight(unsigned index1, unsigned index2 );

		};
	}
}

#endif /*MARKER_H_*/
