/*
 * MarkerConfiguration.cpp
 *
 *  Created on: Oct 5, 2008
 *      Author: mariusz
 */

#include "MarkerConfiguration.h"

namespace dma
{
	namespace core
	{

		MarkerConfiguration::MarkerConfiguration(std::string markerName) :
			markerName_(markerName), status_(new MCStatusValid()), loci_(new std::vector<LocusPtr>())
		{

		}

		MarkerConfiguration::MarkerConfiguration(const MarkerConfiguration& copy)
		{

			// Initializing container
			loci_.reset(new std::vector<LocusPtr>());

			// Copying vector data
			unsigned int i, size;
			size = copy.getLociNumber();
			for (i = 0; i < size; i++)
			{
				loci_->push_back(copy.getLocus(i));
			}

			// Setting basic attributes
			status_ = copy.getStatus();
			markerName_ = copy.getMarkerName();
		}

		MarkerConfiguration& MarkerConfiguration::operator=(const MarkerConfiguration& rhs)
		{

			if (this != &rhs)
			{

				// Reseting container
				loci_.reset(new std::vector<LocusPtr>());

				// Copying vector data
				unsigned int i, size;
				size = rhs.getLociNumber();
				for (i = 0; i < size; i++)
				{
					loci_->push_back(rhs.getLocus(i));
				}

				// Setting basic attributes
				status_ = rhs.getStatus();
				markerName_ = rhs.getMarkerName();

			}

			return *this;
		}

		MarkerConfiguration::~MarkerConfiguration()
		{

		}

		void MarkerConfiguration::addLocus(LocusPtr locus)
		{
			loci_->push_back(locus);
			locus->setMarkerName(markerName_);
		}

		MarkerConfigurationPtr MarkerConfiguration::cloneToSmartPtr() const
		{

			MarkerConfigurationPtr clone(new MarkerConfiguration(getMarkerName()));

			// Cloning locus data
			unsigned int i, size;
			size = getLociNumber();
			for (i = 0; i < size; i++)
			{
				clone->addLocus(getLocus(i)->cloneToSmartPtr());
			}

			// Setting basic attributes
			clone->setStatus(getStatus());

			return clone;
		}

		MarkerConfigurationPtrVectorPtr MarkerConfiguration::cloneMarkerConfigurations(MarkerConfigurationPtrVectorPtr input)
		{

			// Creating new output container
			MarkerConfigurationPtrVectorPtr output(new MarkerConfigurationPtrVector());

			// Copying data
			MarkerConfigurationPtrVector::const_iterator it1;
			for (it1 = input->begin(); it1 != input->end(); it1++)
			{
				output->push_back((*it1)->cloneToSmartPtr());
			}

			return output;
		}

		unsigned int MarkerConfiguration::getLociNumber() const
		{
			return loci_->size();
		}

		LocusPtr MarkerConfiguration::getLocus(unsigned int index) const
		{
			if (index <= loci_->size())
			{

				return loci_->at(index);
			}
			else
			{

				throw OutOfBoundsException(index, loci_->size(), "MarkerConfiguration::getLocus()::loci_");
			}
		}

		std::string MarkerConfiguration::getMarkerName() const
		{

			return markerName_;
		}

		MCStatusPtr MarkerConfiguration::getStatus() const
		{

			return status_;
		}

		AlleleBase::ALLELE_TYPE MarkerConfiguration::getType() const
		{

			if (loci_->size() > 0)
			{

				return loci_->at(0)->getType();
			}
			else
			{

				return AlleleBase::UNDEFINED;
			}
		}

		char MarkerConfiguration::getTypeAsChar() const
		{

			if (loci_->size() > 0)
			{

				return AlleleBase::getTypeAsChar(loci_->at(0)->getType());
			}
			else
			{

				return AlleleBase::getTypeAsChar(AlleleBase::UNDEFINED);
			}
		}

		shared_ptr<std::vector<AlleleBasePtr> > MarkerConfiguration::getUniqueAlleles() const
		{

			unsigned int i, j, s1, s2;
			bool found;
			shared_ptr<std::vector<AlleleBasePtr> > output(new std::vector<AlleleBasePtr>());

			// Iterating loci on marker
			std::vector<LocusPtr>::iterator it1;
			for (it1 = loci_->begin(); it1 < loci_->end(); it1++)
			{

				// Iterating through alleles on locus
				s1 = (*it1)->getAlleleNumber();
				for (i = 0; i < s1; i++)
				{
					found = false;

					// Checking whether allele does not already exist within the output set
					s2 = output->size();
					for (j = 0; j < s2; j++)
					{
						if ((*it1)->getAllele(i)->getAlleleValue() == output->at(j)->getAlleleValue())
						{
							found = true;
							break;
						}
					}

					if (found == false)
					{
						output->push_back((*it1)->getAllele(i));
					}
				}
			}

			return output;
		}

		bool MarkerConfiguration::isEqual(const MarkerConfigurationPtr second) const
		{

			unsigned int i;
			unsigned int size;

			// Checking whether types match
			if (getType() != second->getType())
			{
				return false;
			}

			// Checking whether sizes match
			if (getLociNumber() == second->getLociNumber())
			{

				size = getLociNumber();
				for (i = 0; i < size; i++)
				{

					// Checking whether hashes for both loci match
					if (getLocus(i)->getHash() != second->getLocus(i)->getHash())
					{
						return false;
					}
				}
			}
			else
			{

				return false;
			}

			return true;
		}

		bool MarkerConfiguration::isEquivalent(const MarkerConfigurationPtr second) const
		{
			// DEBUG DEBUG
			/*
			 MarkerConfigurationPrinter mgp;
			 MarkerConfigurationPtr first(new MarkerConfiguration());
			 unsigned int c3;
			 for (c3 = 0; c3 < getLociNumber(); c3++)
			 {
			 first->addLocus(getLocus(c3));
			 }
			 std::cout << "Finding equivalents for :" << std::endl;
			 mgp.print(first);
			 mgp.print(second);
			 */

			unsigned int c1, c2, size;
			bool found = true;

			// Checking whether both markers are of the same type
			if (getType() != second->getType())
			{

				// Markers differ in type
				return false;
			}

			// Comparing number of loci genotypes present on both markers
			if (getLociNumber() != second->getLociNumber())
			{

				// Markers differ in number of loci genotypes
				return false;
			}

			// Checking whether all loci from one marker are on the second one
			size = getLociNumber();
			for (c1 = 0; c1 < size; c1++)
			{
				found = false;

				// Searching for locus genotype on second marker
				for (c2 = 0; c2 < size; c2++)
				{
					if (this->getLocus(c1)->getHash() == second->getLocus(c2)->getHash())
					{
						found = true;
						break;
					}
				}

				// Locus genotype has not been found on second marker. Marker genotypes are not equivalent
				if (found == false)
				{
					return false;
				}
			}

			// Checking equivalence in reverse direction
			for (c1 = 0; c1 < size; c1++)
			{
				found = false;

				// Searching for locus genotype on second marker
				for (c2 = 0; c2 < size; c2++)
				{
					if (second->getLocus(c1)->getHash() == this->getLocus(c2)->getHash())
					{
						found = true;
						break;
					}
				}

				// Locus genotype has not been found on second marker. Marker genotypes are not equivalent
				if (found == false)
				{
					return false;
				}
			}

			return true;
		}

		bool MarkerConfiguration::isLocusPresent(LocusPtr locus) const
		{

			if (getType() != locus->getType())
			{
				return false;
			}

			// Searching for locus
			unsigned int i;
			unsigned int size = getLociNumber();
			for (i = 0; i < size; ++i)
			{

				if (getLocus(i)->getHash() == locus->getHash())
				{
					return true;
				}
			}

			return false;
		}

		bool MarkerConfiguration::isValid() const
		{

			return getStatus()->isValid();
		}

		void MarkerConfiguration::setStatus(MCStatusPtr status)
		{

			status_ = status;
		}

		void MarkerConfiguration::subtractLocus(LocusPtr locus)
		{

			if (getType() == locus->getType())
			{

				// Copying loci if they are the same as searched one
				std::vector<LocusPtr>::iterator it1;
				for (it1 = loci_->begin(); it1 != loci_->end(); it1++)
				{

					if ((*it1)->getHash() == locus->getHash())
					{

						loci_->erase(it1);
						break;
					}
				}
			}
		}

		std::string MarkerConfiguration::toString() const
		{

			std::string buffer = "\t\t";

			unsigned int i, size1, alleleNumber;
			size1 = getLociNumber();

			for (i = 0; i < size1; i++)
			{
				alleleNumber = getLocus(i)->getAlleleNumber();
				if (alleleNumber == 1)
				{
					buffer = buffer + getLocus(i)->getAllele(0)->getAlleleValueAsString() + "\t";
					buffer = buffer + lexical_cast<std::string> (getLocus(i)->getAllele(0)->getPeakArea());
					buffer = buffer + "\t";
					buffer = buffer + getLocus(i)->getAllele(0)->getAlleleValueAsString() + "\t";
					buffer = buffer + lexical_cast<std::string> (getLocus(i)->getAllele(0)->getPeakArea());
				}
				else if (alleleNumber == 2)
				{
					buffer = buffer + getLocus(i)->getAllele(0)->getAlleleValueAsString() + "\t";
					buffer = buffer + lexical_cast<std::string> (getLocus(i)->getAllele(0)->getPeakArea());
					buffer = buffer + "\t";
					buffer = buffer + getLocus(i)->getAllele(1)->getAlleleValueAsString() + "\t";
					buffer = buffer + lexical_cast<std::string> (getLocus(i)->getAllele(1)->getPeakArea());
				}
				else
				{
					buffer = buffer + "e e";
				}

				buffer = buffer + "\t\t\t";
			}

			buffer = buffer + "status: " + getStatus()->getName();

			return buffer;
		}

	}
}
