/*
 * MarkerConfiguration.h
 *
 *  Created on: Oct 5, 2008
 *      Author: mariusz
 */

#ifndef MARKERGENOTYPE_H_
#define MARKERGENOTYPE_H_

#include <vector>
#include <string>
#include <boost/shared_ptr.hpp>
#include "../models/Locus.h"
#include "../models/statuses/MCStatusValid.h"
#include "../exceptions/OutOfBoundsException.h"

namespace dma
{
	namespace core
	{

		// Containers
		class MarkerConfiguration;
		typedef shared_ptr<MarkerConfiguration> MarkerConfigurationPtr;
		typedef std::vector<MarkerConfigurationPtr> MarkerConfigurationPtrVector;
		typedef shared_ptr<MarkerConfigurationPtrVector> MarkerConfigurationPtrVectorPtr;

		/**
		 * Class representing set of genotypes of mixture contributors which have been found on STR marker
		 */
		class MarkerConfiguration
		{
		private:

			/**
			 * Name of the marker for which configuration has been generated
			 */
			std::string markerName_;

			/**
			 * Extended configuration status
			 */
			MCStatusPtr status_;

			/**
			 * Container for genotypes assigned to mixture contributors
			 */
			shared_ptr<std::vector<LocusPtr> > loci_;

		public:

			MarkerConfiguration(std::string markerName);

			MarkerConfiguration(const MarkerConfiguration& copy);

			MarkerConfiguration& operator=(const MarkerConfiguration& rhs);

			virtual ~MarkerConfiguration();

			/**
			 * Add locus ( contributor genotype ) to configuration
			 *
			 * @param locus Contributor's genotype
			 */
			void addLocus(LocusPtr locus);

			/**
			 * Create a self-copy ( clone ) and returns it as a smart pointer
			 *
			 * @return
			 */
			MarkerConfigurationPtr cloneToSmartPtr() const;

			/**
			 * Method clones given set of marker configurations
			 *
			 * @param input
			 * @return
			 */
			static MarkerConfigurationPtrVectorPtr cloneMarkerConfigurations(MarkerConfigurationPtrVectorPtr input);

			/**
			 * Get number of loci ( contributor genotypes ) present in configuration
			 *
			 * @return
			 */
			unsigned int getLociNumber() const;

			/**
			 * Retrieve locus ( contributor genotype ).
			 * Method may throw @class OutOfBoundsException if locus with given index is not present within configuration.
			 *
			 * @param index Locus number - first locus has index 0
			 * @return
			 * @exception OutOfBoundsException
			 */
			LocusPtr getLocus(unsigned int index) const;

			/**
			 * Retrieve name of marker to which configuration is related

			 * @return
			 */
			std::string getMarkerName() const;

			/**
			 * Get current configuration status
			 *
			 * @return
			 */
			MCStatusPtr getStatus() const;

			/**
			 * Method retrieves
			 *
			 * @return
			 */
			AlleleBase::ALLELE_TYPE getType() const;

			/**
			 * Method returns type of marker basing on loci
			 */
			char getTypeAsChar() const;

			/**
			 * Method returns set of unique alleles which present in genotype
			 *
			 * Example: For configuration 11	12		23
			 * method returns 1 2 3
			 */
			shared_ptr<std::vector<AlleleBasePtr> > getUniqueAlleles() const;

			/**
			 * Method checks whether given configuration is equal to second one.
			 *
			 * Configurations are are said to be equal if they have the same loci in the exact same order
			 *
			 * @param second Second configuration
			 * @return
			 */
			bool isEqual(const MarkerConfigurationPtr second) const;

			/**
			 * Method checks whether given configuration is equivalent to second one.
			 *
			 * Configurations are are said to be equivalent if they have the same loci regardless of their order
			 *
			 * @param second Second configuration
			 * @return
			 */
			bool isEquivalent(const MarkerConfigurationPtr second) const;

			/**
			 * Method checks whether given locus is part of marker configuration
			 *
			 * @param locus
			 * @return
			 */
			bool isLocusPresent(LocusPtr locus) const;

			/**
			 * Method returns information whether configuration is valid one
			 * @return
			 */
			bool isValid() const;

			/**
			 * Set new configuration status
			 * @param status
			 */
			void setStatus(MCStatusPtr status);

			/**
			 * Method subtracts from current marker configuration given locus ( if it exists - no error is reported if not found )
			 *
			 * @param locus
			 * @return
			 */
			void subtractLocus(LocusPtr locus);

			/**
			 * Get text representation of contained data
			 * @return
			 */
			std::string toString() const;

		};
	}
}
#endif /* MARKERGENOTYPE_H_ */
