/*
 * MixtureProportion.h
 *
 *  Created on: Nov 13, 2008
 *      Author: mariusz
 */

#ifndef MIXTUREPROPORTION_H_
#define MIXTUREPROPORTION_H_

#include <vector>
#include <map>
#include <stdexcept>

#include <boost/shared_ptr.hpp>
#include <boost/lexical_cast.hpp>
#include <boost/tokenizer.hpp>
#include "../exceptions/OutOfBoundsException.h"

using namespace boost;

namespace dma
{
	namespace core
	{

		class MixtureProportion;
		typedef shared_ptr<MixtureProportion> MixtureProportionPtr;
		typedef std::map<unsigned int, MixtureProportionPtr> MixProportionPtrMap;
		typedef shared_ptr<std::map<unsigned int, MixtureProportionPtr> > MixProportionPtrMapPtr;

		/**
		 * Class representing proportions of DNA contributors within a mixture.
		 * Initialized object of this class by default does not have any contributors.
		 *
		 * @remarks The absolute values used when adding contributor shared does not affect the output,
		 * in which all contributor shares sum up to 100%
		 */
		class MixtureProportion
		{
		private:

			/**
			 * Proportions
			 */
			shared_ptr<std::vector<double> > proportions_;

		public:

			MixtureProportion();

			virtual ~MixtureProportion();

			/**
			 * Method adds contributor with given share.
			 *
			 * @param contributorShare
			 */
			void addContributor(double contributorShare);

			/**
			 * Method returns share of contributor to whole mixture. If index is not valid @class OutOfBoundsException is throwned
			 *
			 * @param index First contributor has index 0
			 * @return
			 * @throw OverflowExcpetion
			 */
			double getContributorShare(unsigned int index) const;


			/**
			 * Method returns number of contributors
			 * @return
			 */
			unsigned int getContributorsNumber() const;

			/**
			 * Method returns proportion which is between set of contributors - summed up to 1.
			 * Proportion is only valid for given set of contributors and not valid for whole mixture.
			 *
			 * @example For mixture with given proportions:
			 * 	- Contributor 1: 0.25
			 * 	- Contributor 2: 0.25
			 * 	- Contributor 3: 0.50
			 * Share between contributors 1 and 2 is 0.5:0.5
			 *
			 * If index is not valid @class OutOfBoundsException is throwned
			 *
			 * @param indexes Fist contributor index is 0
			 * @return
			 */
			shared_ptr<std::vector<double> > getContributorsShares(const shared_ptr<std::vector<unsigned int> > indexes) const;

			/**
			 * Return data in text form
			 * @return
			 */
			std::string toString();


			/**
			 * Method responsible for parsing string containing mixture proportion ( i.e. 0.43:0.57 ).
			 * If parsing fails @class std::runtime_error is generated
			 *
			 * @param input
			 * @return
			 * @exception std::runtime_error
			 */
			static MixtureProportionPtr parse(std::string input) throw(std::runtime_error);
		};
	}
}

#endif /* MIXTUREPROPORTION_H_ */
