#ifndef SAMPLE_H_
#define SAMPLE_H_

#include <vector>
#include <string>
#include <map>
#include "Marker.h"
#include "../exceptions/MarkerNotFoundException.h"
#include <boost/shared_ptr.hpp>

using namespace boost;

namespace dma
{

	namespace core
	{

		class Sample;
		typedef shared_ptr<Sample> SamplePtr;
		typedef std::vector<shared_ptr<Sample> > SamplePtrVector;
		typedef shared_ptr< std::vector<shared_ptr<Sample> > > SamplePtrVectorPtr;

		/**
		 * Class representing sample - data obtained from DNA sample containing mixed DNA
		 */
		class Sample
		{

		private:

			/**
			 * Sample name
			 */
			std::string name_;

			/**
			 * Panel name - STP set ?
			 */
			std::string panel_;

			/**
			 * Markers found in sample
			 */
			shared_ptr<std::map<std::string, MarkerPtr> > markers_;

		public:

			Sample(std::string name, std::string panel);

			Sample(const Sample& copy);

			Sample& operator=(const Sample& rhs);

			virtual ~Sample();

			/**
			 * Get name of sample
			 *
			 * @return
			 */
			std::string getName() const;

			/**
			 * Get name of panel used for sample
			 * @return
			 */
			std::string getPanel() const;

			/**
			 * Get number of markers which are present in given sample
			 */
			unsigned int getMarkersNumber() const;

			/**
			 * Add marker to sample. In the case in which marker with give name exists already in sample it is replaced by new one.
			 *
			 * @param marker
			 */
			void addMarker(MarkerPtr marker);

			/**
			 * Get marker with given name. If marker cannot be found @class MarkerNotFoundException is generated
			 *
			 * @param markerName
			 * @return
			 * @exception MarkerNotFoundException
			 */
			MarkerPtr getMarker(std::string markerName) const;

			/**
			 * Method returns names of all markers which are present within sample
			 * @return
			 */
			shared_ptr<std::vector<std::string> > getMarkerNames() const;

			/**
			 * Method returns string representation of allele.
			 *
			 * Output:
			 * Sample EXAMPLE:
			 *
			 * --------------------------------------------------------------------
			 * Marker 1
			 *
			 * --------------------------------------------------------------------
			 * Marker 2
			 * @return
			 */
			std::string toString() const;
		};

	}
}

#endif /*SAMPLE_H_*/
