/*
 * SampleConfiguration.h
 *
 *  Created on: Nov 11, 2008
 *      Author: mariusz
 */

#ifndef SAMPLECONFIGURATION_H_
#define SAMPLECONFIGURATION_H_

#include <map>
#include <string>
#include <boost/shared_ptr.hpp>
#include "MarkerConfiguration.h"
#include "../exceptions/MarkerNotFoundException.h"

namespace dma
{

	namespace core
	{

		class SampleConfiguration;
		typedef shared_ptr<SampleConfiguration> SampleConfigurationPtr;

		/**
		 * Class representing sample for which for every marker all possible genotypes of contributors are given
		 */
		class SampleConfiguration
		{
		private:

			shared_ptr<std::map<std::string, MarkerConfigurationPtrVectorPtr> > markers_;

			std::string name_;

		public:

			SampleConfiguration(std::string name);

			SampleConfiguration(const SampleConfiguration& copy);

			SampleConfiguration& operator=(const SampleConfiguration& rhs);

			/**
			 * Create a self-copy ( clone ) and returns it as a smart pointer
			 */
			SampleConfigurationPtr cloneToSmartPtr() const;

			virtual ~SampleConfiguration();

			/**
			 * Method adds additional configuration to given marker. If configurations container for given marker could not found
			 * and could not been created @class MarkerNotFoundException is generated
			 *
			 * @param markerName
			 * @param markerConfiguration
			 * @exception MarkerNotFoundException
			 */
			void addMarkerConfiguration(std::string markerName, MarkerConfigurationPtr markerConfiguration);

			/**
			 * Method adds set of additional marker configurations to given marker. If marker has not existed in the set it is added.
			 * Method may throw @class MarkerNotFoundException is configurations container for given marker has been found and
			 * could not be created.
			 *
			 * @param markerName
			 * @param markerConfigurations
			 */
			void addMarkerConfigurations(std::string markerName, MarkerConfigurationPtrVectorPtr markerConfigurations);

			/**
			 * Method returns possible configurations for given marker
			 * @param name Marker name
			 * @return
			 * @exception MarkerNotFoundException
			 */
			MarkerConfigurationPtrVectorPtr getMarkerConfigurations(std::string markerName) const;

			/**
			 * Method returns all marker names which are in given sample
			 * @return
			 */
			shared_ptr<std::vector<std::string> > getMarkerNames() const;

			/**
			 * Method returns maximum number of loci which is present in configurations
			 *
			 * @param countValidOnly Flag indicating whether only valid configurations will be counted
			 * @return
			 */
			unsigned int getMaximumLociNumber(bool countValidOnly = false) const;

			/**
			 * Get sample namen
			 *
			 * @return
			 */
			std::string getName() const;

			/**
			 * Get sample name
			 *
			 * @return
			 */
			void setName(std::string name);

			/**
			 * Get text representation
			 * @return
			 */
			std::string toString() const;
		};

	}
}

#endif /* SAMPLECONFIGURATION_H_ */
