/*
 * DeconvolutionReportGenerator.cpp
 *
 *  Created on: Nov 18, 2008
 *      Author: mariusz
 */

#include "DeconvolutionReportGenerator.h"

namespace dma
{

	namespace external
	{

		DeconvolutionReportGenerator::DeconvolutionReportGenerator(std::string filename, FILE_TYPE fileType, char fieldsSeparator) :
			filename_(filename), fileType_(fileType), fs_(fieldsSeparator), eol_("\n")
		{
			stream_.open(filename.c_str());
		}

		DeconvolutionReportGenerator::~DeconvolutionReportGenerator()
		{
			stream_.close();
		}

		void DeconvolutionReportGenerator::generate(SampleConfigurationPtr outputSampleConfiguration)
		{

			std::string buffer;
			std::string markerName;

			// Marker configurations set for single marker
			shared_ptr<std::vector<MarkerConfigurationPtr> > markerConfigurations;

			// Getting maximum number of loci ( contributors )
			unsigned int maximumLociNumber = outputSampleConfiguration->getMaximumLociNumber();

			// Getting marker names and creating creating iterator for traversing them
			shared_ptr<std::vector<std::string> > markerNames = outputSampleConfiguration->getMarkerNames();
			std::vector<std::string>::const_iterator it1;

			// Creating iterator for traversing marker configurations
			std::vector<MarkerConfigurationPtr>::const_iterator it2;

			// Generating header
			shared_ptr<std::vector<std::string> > prefixedColumns(new std::vector<std::string>());
			prefixedColumns->push_back("Sample name");
			prefixedColumns->push_back("Marker name");
			prefixedColumns->push_back("Marker type");
			prefixedColumns->push_back("Configuration number");
			prefixedColumns->push_back("Contributors number");
			buffer = generateFileHeader(maximumLociNumber, prefixedColumns);

			// Writing header
			stream_ << buffer << eol_;

			// Iterating through markers
			for (it1 = markerNames->begin(); it1 != markerNames->end(); it1++)
			{

				markerName = *it1;

				// Getting marker configurations
				markerConfigurations = outputSampleConfiguration->getMarkerConfigurations( markerName );

				// Generating cell value constant for all configurations for marker - samples name and marker name
				prefixedColumns->clear();
				prefixedColumns->push_back(outputSampleConfiguration->getName());
				prefixedColumns->push_back(markerName);

				unsigned int configurationIndex = 0;
				for (it2 = markerConfigurations->begin(); it2 != markerConfigurations->end(); it2++)
				{

					// Generating data
					buffer = generateMarkerConfigurationData(*it2, configurationIndex++, (*it2)->getLociNumber(), prefixedColumns);

					// Writing data
					stream_ << buffer << eol_;
				}
			}
		}

		std::string DeconvolutionReportGenerator::generateFileHeader(unsigned int maximumLociNumber, shared_ptr<std::vector<
				std::string> > prefixedColumns)
		{

			// Creating header
			std::string buffer = "";

			// Appending constant columns
			std::vector<std::string>::iterator it1;
			for (it1 = prefixedColumns->begin(); it1 != prefixedColumns->end(); it1++)
			{

				buffer = buffer + *it1 + fs_;
			}

			// Writing column headers
			unsigned int i;
			for (i = 0; i < maximumLociNumber; i++)
			{
				buffer = buffer + "C" + lexical_cast<std::string> (i + 1) + " A1 Allele value";
				buffer = buffer + fs_;
				buffer = buffer + "C" + lexical_cast<std::string> (i + 1) + " A1 Peak height";
				buffer = buffer + fs_;
				buffer = buffer + "C" + lexical_cast<std::string> (i + 1) + " A1 Peak area";
				buffer = buffer + fs_;
				buffer = buffer + "C" + lexical_cast<std::string> (i + 1) + " A2 Allele value";
				buffer = buffer + fs_;
				buffer = buffer + "C" + lexical_cast<std::string> (i + 1) + " A2 Peak height";
				buffer = buffer + fs_;
				buffer = buffer + "C" + lexical_cast<std::string> (i + 1) + " A2 Peak area";
				buffer = buffer + fs_;
			}

			buffer = buffer + "Configuration status";
			if (fileType_ == FILE_TYPE_EXTENDED)
			{
				buffer = buffer + fs_ + "Detailed configuration status";
			}

			return buffer;
		}

		std::string DeconvolutionReportGenerator::generateMarkerConfigurationData(MarkerConfigurationPtr markerConfiguration,
				unsigned int configurationNumber,
				unsigned int contributorsNumber,
				shared_ptr<std::vector<std::string> > prefixedColumns)
		{

			std::string buffer = "";

			// Appending constant columns
			std::vector<std::string>::iterator it1;
			for (it1 = prefixedColumns->begin(); it1 != prefixedColumns->end(); it1++)
			{
				buffer = buffer + *it1 + fs_;
			}

			// Writing data
			buffer.append(1, markerConfiguration->getTypeAsChar());
			buffer = buffer + fs_;
			buffer = buffer + lexical_cast<std::string> (configurationNumber + 1) + fs_;
			buffer = buffer + lexical_cast<std::string> (contributorsNumber) + fs_;

			unsigned int i;
			unsigned int lociNumber = markerConfiguration->getLociNumber();
			for (i = 0; i < lociNumber; i++)
			{

				LocusPtr locus = markerConfiguration->getLocus(i);
				unsigned int allelesNumber = locus->getAlleleNumber();
				if (allelesNumber == 0)
				{
					buffer = buffer + fs_ + fs_ + fs_ + fs_ + fs_ + fs_;
				}
				else if (allelesNumber == 2)
				{
					buffer = buffer + locus->getAllele(0)->getAlleleValueAsString();
					buffer = buffer + fs_;
					buffer = buffer + lexical_cast<std::string> (locus->getAllele(0)->getPeakHeight());
					buffer = buffer + fs_;
					buffer = buffer + lexical_cast<std::string> (locus->getAllele(0)->getPeakArea());
					buffer = buffer + fs_;
					buffer = buffer + locus->getAllele(1)->getAlleleValueAsString();
					buffer = buffer + fs_;
					buffer = buffer + lexical_cast<std::string> (locus->getAllele(1)->getPeakHeight());
					buffer = buffer + fs_;
					buffer = buffer + lexical_cast<std::string> (locus->getAllele(1)->getPeakArea());
					buffer = buffer + fs_;
				}

			}

			buffer = buffer + markerConfiguration->getStatus()->getName();

			if (fileType_ == FILE_TYPE_EXTENDED)
			{
				buffer = buffer + fs_ + markerConfiguration->getStatus()->toString();
			}

			return buffer;
		}

	}
}
