/*
 * DeconvolutionReportGenerator.h
 *
 *  Created on: Nov 18, 2008
 *      Author: mariusz
 */

#ifndef DeconvolutionReportGenerator_H_
#define DeconvolutionReportGenerator_H_

#include <string>
#include <iostream>
#include <fstream>
#include <vector>
#include "models/Sample.h"
#include "models/SampleConfiguration.h"
#include "models/MarkerConfiguration.h"
#include "boost/shared_ptr.hpp"

using namespace dma::core;

namespace dma
{

	namespace external
	{

		class DeconvolutionReportGenerator;
		typedef boost::shared_ptr<DeconvolutionReportGenerator> DeconvolutionReportGeneratorPtr;

		/**
		 * Class responsible for generation of report following deconvolution process.
		 *
		 * The report column structure is presented below ( for configurations including 2 persons maximum ):
		 *
		 * 	- Sample name			- Name of input sample
		 * 	- Marker name			- Name of marker investigated
		 * 	- Marker type			- Type of marker ( N - numerical, C - character )
		 * 	- Configuration number	- Index ( 1 - started ) of configuration for marker
		 * 	- Contributors number	- Number of contributors which are present in configuration
		 * 	- C1 A1 Allele value	- First contributor first allele value
		 * 	- C1 A1 Peak height		- First contributor first allele peak height
		 * 	- C1 A1 Peak area		- First contributor first allele peak area
		 * 	- C1 A2 Allele value	- First contributor second allele value
		 * 	- C1 A2 Peak height		- First contributor second allele peak height
		 * 	- C1 A2 Peak area		- First contributor second allele peak area
		 * 	- C2 A1 Allele value	- Second contributor first allele value
		 * 	- C2 A1 Peak height		- Second contributor first allele peak height
		 * 	- C2 A1 Peak area		- Second contributor first allele peak area
		 * 	- C2 A2 Allele value	- Second contributor second allele value
		 * 	- C2 A2 Peak height		- Second contributor second allele peak height
		 * 	- C2 A2 Peak area		- Second contributor second allele peak area
		 * 	- Configuration status	- Configuration status code
		 *  Fields available only in extended version of report:
		 * 	- Detailed configuration status	- Detailed information regarding configuration status with information regarding the reason for exclusion
		 */
		class DeconvolutionReportGenerator
		{
		public:

			/**
			 * Default field separator
			 */
			static const char DEFAULT_FIELDS_SEPARATOR = ',';

			/**
			 * Type of file generated
			 */
			enum FILE_TYPE
			{

				FILE_TYPE_SIMPLE, //!< FILE_TYPE_SIMPLE	No detailed information regarding exclusion reason is given
				FILE_TYPE_EXTENDED
			//!< FILE_TYPE_EXTENDED Detailed information regarding exclusion is written
			};

		private:

			/**
			 * Name of the file to which data will be written
			 */
			std::string filename_;

			/**
			 * Type of file being generated
			 */
			FILE_TYPE fileType_;

			/**
			 * Fields separator
			 */
			char fs_;

			/**
			 * End Of Line character
			 */
			std::string eol_;

			/**
			 * File output stream
			 */
			std::ofstream stream_;

		public:

			/**
			 *
			 *
			 * @param filename Name of the file
			 * @param fieldsSeparator Fields separator - by default semicolon
			 * @return
			 */
			DeconvolutionReportGenerator(std::string filename, FILE_TYPE fileType = FILE_TYPE_EXTENDED, char fieldsSeparator =
					DEFAULT_FIELDS_SEPARATOR);

			virtual ~DeconvolutionReportGenerator();

			/**
			 * Generate results file
			 *
			 * @param outputSampleConfiguration
			 */
			void generate(SampleConfigurationPtr outputSampleConfiguration);

		protected:

			/**
			 * Generate file header with column names
			 *
			 * @param maximumLociNumber
			 * @param prefixedColumns
			 * @return
			 */
			std::string generateFileHeader(unsigned int maximumLociNumber, shared_ptr<std::vector<std::string> > prefixedColumns);

			/**
			 * Generate single data row
			 *
			 * @param markerConfiguration
			 * @param configurationNumber
			 * @param contributorsNumber
			 * @param prefixedColumns
			 * @return
			 */
			std::string generateMarkerConfigurationData(MarkerConfigurationPtr markerConfiguration,
					unsigned int configurationNumber,
					unsigned int contributorsNumber,
					shared_ptr<std::vector<std::string> > prefixedColumns);

		};
	}
}

#endif /* DeconvolutionReportGenerator_H_ */
