/*
 * ExternalDataParser.cpp
 *
 *  Created on: Jul 4, 2009
 *      Author: mariusz
 */

#include "ExternalDataParser.h"

namespace dma
{

	namespace external
	{

		ExternalDataParser::ExternalDataParser()
		{

		}

		ExternalDataParser::~ExternalDataParser()
		{

		}

		SamplePtr ExternalDataParser::parseMixtureDataFile(std::string filename, ExternalDataParser::MIXTURE_DATA_FILE_FORMAT mixtureDataFileFormat) throw (ParserException)
		{

			try
			{

				// Creating appropriate parser
				MixtureDataParserPtr mixtureDataParser;
				if ( mixtureDataFileFormat == COLUMNAR_FORMAT ) {

					mixtureDataParser.reset(new MixtureDataParserColumnarFormat(filename));
				} else {

					mixtureDataParser.reset(new MixtureDataParserLinearFormat(filename));
				}

				// Performing parsing
				shared_ptr<std::vector<SamplePtr> > samples = mixtureDataParser->parse();

				// Checking whether no more than single sample has been present in data file
				if (samples->size() == 1)
				{

					return samples->at(0);
				}
				else if (samples->size() == 0)
				{

					ParserException pe("No mixture sample data has been found in data file - file empty?");
					throw pe;
				}
				else
				{

					ParserException pe("More than one sample in present in data file");
					throw pe;
				}
			} catch (ParserException& pe)
			{

				// Appending information regarding file type and filename
				pe.setFileType(MixtureDataParserColumnarFormat::getFileType());
				pe.setFilename(filename);
				throw pe;

			} catch (std::runtime_error& re)
			{

				// Converting exception into parser one
				std::string errorMsg = "Runtime error: ";
				errorMsg.append(re.what());
				ParserException pe(errorMsg);
				pe.setFileType(MixtureDataParserColumnarFormat::getFileType());
				pe.setFilename(filename);
				throw pe;

			} catch (...)
			{

				// Creating exception
				ParserException pe("Unknown error has been reported while parsing data file");
				pe.setFileType(MixtureDataParserColumnarFormat::getFileType());
				pe.setFilename(filename);
				throw pe;
			}
		}

		MarkerPtrVectorPtr ExternalDataParser::parsePopulationDataFile(std::string filename) throw (ParserException)
		{

			try
			{

				// Creating parser and parsing
				PopulationDataParserPtr populationDataParser(new PopulationDataParser(filename));
				MarkerPtrVectorPtr populationData = populationDataParser->parse();

				// Checking whether population data for any marker has been parsed
				if (populationData->size() > 0)
				{

					return populationData;
				}
				else
				{

					ParserException pe("No population data has been found in data file - file empty?");
					throw pe;
				}
			} catch (ParserException& pe)
			{

				// Appending information regarding file type and filename
				pe.setFileType(PopulationDataParser::getFileType());
				pe.setFilename(filename);
				throw pe;

			} catch (std::runtime_error& re)
			{

				// Converting exception into parser one
				std::string errorMsg = "Runtime error: ";
				errorMsg.append(re.what());
				ParserException pe(errorMsg);
				pe.setFileType(PopulationDataParser::getFileType());
				pe.setFilename(filename);
				throw pe;

			} catch (...)
			{

				// Creating exception
				ParserException pe("Unknown error has been reported while parsing data file");
				pe.setFileType(PopulationDataParser::getFileType());
				pe.setFilename(filename);
				throw pe;
			}
		}

		HbMapPtr ExternalDataParser::parseHbDataFile(std::string filename) throw (ParserException)
		{

			try
			{

				HbDataParserPtr hbDataParser(new HbDataParser(filename));
				HbMapPtr hbMinData = hbDataParser->parse();

				if (hbMinData->size() > 0)
				{

					return hbMinData;
				}
				else
				{

					ParserException pe("No heterozygotic balance data has been found in data file - file empty?");
					throw pe;
				}
			} catch (ParserException& pe)
			{

				// Appending information regarding file type and filename
				pe.setFileType(HbDataParser::getFileType());
				pe.setFilename(filename);
				throw pe;

			} catch (std::runtime_error& re)
			{

				// Converting exception into parser one
				std::string errorMsg = "Runtime error: ";
				errorMsg.append(re.what());
				ParserException pe(errorMsg);
				pe.setFileType(HbDataParser::getFileType());
				pe.setFilename(filename);
				throw pe;

			} catch (...)
			{

				// Creating exception
				ParserException pe("Unknown error has been reported while parsing data file");
				pe.setFileType(HbDataParser::getFileType());
				pe.setFilename(filename);
				throw pe;
			}
		}

		SampleConfigurationPtr ExternalDataParser::parseOutputDataFile(std::string filename) throw (ParserException)
		{

			try
			{

				OutputDataParserPtr parser(new OutputDataParser(filename));
				SampleConfigurationPtr sc = parser->parse();

				if (sc->getMarkerNames()->size() > 0)
				{

					return sc;
				}
				else
				{

					ParserException pe("No configurations have been read from file - file empty?");
					throw pe;
				}
			} catch (ParserException& pe)
			{

				// Appending information regarding file type and filename
				pe.setFileType(OutputDataParser::getFileType());
				pe.setFilename(filename);
				throw pe;

			} catch (std::runtime_error& re)
			{

				// Converting exception into parser one
				std::string errorMsg = "Runtime error: ";
				errorMsg.append(re.what());
				ParserException pe(errorMsg);
				pe.setFileType(OutputDataParser::getFileType());
				pe.setFilename(filename);
				throw pe;

			} catch (...)
			{

				// Creating exception
				ParserException pe("Unknown error has been reported while parsing data file");
				pe.setFileType(OutputDataParser::getFileType());
				pe.setFilename(filename);
				throw pe;
			}
		}

	}

}
