/*
 * MixtureDataParser.cpp
 *
 *  Created on: Dec 12, 2009
 *      Author: mariusz
 */

#include "MixtureDataParser.h"

namespace dma
{

	namespace external
	{

		MixtureDataParser::MixtureDataParser(std::string filename, std::string separator) :
			DataParser(filename, MixtureDataParser::getFileType(), separator), samples_(new SamplePtrVector())
		{

		}

		MixtureDataParser::~MixtureDataParser()
		{

		}

		std::string MixtureDataParser::getFileType()
		{
			return "Mixture sample data file";
		}

		SamplePtrVectorPtr MixtureDataParser::getSamples() const
		{
			return samples_;
		}

		SamplePtrVectorPtr MixtureDataParser::parse() throw (ParserException)
		{

			int rowCounter = 0;
			bool truncLastCol = false;
			shared_ptr<std::vector<std::string> > row;

			while ((row = reader_->getLineAndExplode()) && row->size() > 0)
			{

				// Checking whether in header row extra columns are not appended
				if (rowCounter == 0 && row->size() > 0 && row->at(row->size() - 1).length() == 0)
				{
					truncLastCol = true;
				}

				// Truncating extra empty columns at the end
				if (row->size() > 0 && truncLastCol)
				{
					row->pop_back();
				}

				if (rowCounter == 0)
				{
					parseHeaderRow(row);
				}
				else
				{
					if (row->size() > 0)
					{
						parseDataRow(row, rowCounter);
					}
				}

				++rowCounter;
			}

			return this->getSamples();
		}

		AlleleBasePtr MixtureDataParser::parseAllelData(std::string alleleValue,
				std::string alleleSize,
				std::string peakHeight,
				std::string peakSize,
				unsigned int lineNumber) throw (ParserException)
		{

			char type = 'U';

			char cAllelValue = 0;
			double dAllelValue = 0.0;
			double dAlleleSize = 0.0;
			double dPeakHeight = 0.0;
			double dPeakSize = 0.0;

			// Converting doubles in format with ',' to format with '.'
			alleleValue = this->convertFloatingPoints(alleleValue);
			alleleSize = this->convertFloatingPoints(alleleSize);
			peakHeight = this->convertFloatingPoints(peakHeight);
			peakSize = this->convertFloatingPoints(peakSize);

			// Checking whether allele is present - allele value is given
			if (alleleValue.length() == 0)
			{

				NumericAllelePtr out(new NumericAllele(0.0));
				return out;
			}

			// Checking allele type and getting value
			if (Validator::isDouble(alleleValue))
			{

				type = AlleleBase::ALLELE_TYPE_NUMERIC;
				dAllelValue = lexical_cast<double> (alleleValue);
			}
			else if (alleleValue.length() == 1)
			{

				type = AlleleBase::ALLELE_TYPE_CHARACTER;
				cAllelValue = alleleValue.at(0);
			}
			else
			{

				// Generating error message and exception
				std::string errorMsg = "Allele value cannot be parsed: " + alleleValue;
				throw ParserException(errorMsg, fileType_, reader_->getFilename(), lineNumber + 1);
			}

			// Getting allele size
			try
			{
				dAlleleSize = lexical_cast<double> (alleleSize);
			} catch (bad_lexical_cast& e)
			{

				// Generating error message and exception
				std::string errorMsg = "Allele size cannot be parsed: " + alleleSize;
				throw ParserException(errorMsg, fileType_, reader_->getFilename(), lineNumber + 1);
			}

			// Getting peak height
			try
			{
				dPeakHeight = lexical_cast<double> (peakHeight);
			} catch (bad_lexical_cast& e)
			{

				// Generating error message and exception
				std::string errorMsg = "Allele peak height cannot be parsed: " + peakHeight;
				throw ParserException(errorMsg, fileType_, reader_->getFilename(), lineNumber + 1);

			}

			// Getting peak size
			try
			{
				dPeakSize = lexical_cast<double> (peakSize);
			} catch (bad_lexical_cast& e)
			{

				// Generating error message and exception
				std::string errorMsg = "Allele peak size cannot be parsed: " + peakSize;
				throw ParserException(errorMsg, fileType_, reader_->getFilename(), lineNumber + 1);
			}

			// Creating allele basing on allele type
			if (type == AlleleBase::ALLELE_TYPE_NUMERIC)
			{

				NumericAllelePtr allel(new NumericAllele(dAllelValue, dAlleleSize, dPeakHeight, dPeakSize));
				return allel;
			}
			else
			{

				CharacterAllelePtr allel(new CharacterAllele(cAllelValue, dPeakHeight, dPeakSize));
				return allel;
			}
		}

		void MixtureDataParser::addAllele(std::string sampleName,
				std::string panelName,
				std::string markerName,
				AlleleBasePtr allele,
				unsigned int lineNumber)
		{

			unsigned int i;
			SamplePtr destSample;
			SamplePtr currSample;
			MarkerPtr destMarker;
			MarkerPtr currMarker;

			// Checking whether sample does not already exists
			for (i = 0; i < samples_->size(); i++)
			{
				currSample = samples_->at(i);
				if (currSample->getName() == sampleName)
				{
					destSample = currSample;
				}
			}

			// If sample has not been found creating new one
			if (destSample == 0)
			{
				destSample.reset(new Sample(sampleName, panelName));
				samples_->push_back(destSample);
			}

			// Checking whether marker does not already exists within sample
			try
			{
				destMarker = destSample->getMarker(markerName);
			} catch (MarkerNotFoundException )
			{

				// Marker has not existed within sample. Creating new one
				destMarker.reset(new Marker(markerName));
				destSample->addMarker(destMarker);
			}

			// Adding allele to marker
			try
			{

				destMarker->addAllele(allele);
			} catch (AlleleTypeConflictException e)
			{

				// Generating error message and exception
				std::string errorMsg =
						"Allele type conflict ( mixed numerical and character alleles ) have been found for marker " + markerName;
				throw ParserException(errorMsg, fileType_, reader_->getFilename(), lineNumber + 1);
			}
		}

	}

}
