/*
 * MixtureDataParser.h
 *
 *  Created on: Dec 12, 2009
 *      Author: mariusz
 */

#ifndef MIXTUREDATAPARSER_H_
#define MIXTUREDATAPARSER_H_

#include <vector>
#include <string>
#include <iostream>
#include <boost/lexical_cast.hpp>
#include "boost/tuple/tuple.hpp"
#include <boost/shared_ptr.hpp>
#include "ParserException.h"
#include "DataParser.h"
#include "models/Sample.h"
#include "models/Marker.h"
#include "models/NumericAllele.h"
#include "models/CharacterAllele.h"
#include "../utils/Validator.h"

using namespace boost;
using namespace dma::core;

namespace dma
{

	namespace external
	{

		class MixtureDataParser;
		typedef shared_ptr<MixtureDataParser> MixtureDataParserPtr;

		/**
		 * Class used for parsing mixture data files
		 */
		class MixtureDataParser: public DataParser
		{
		protected:

			/**
			 * Container for data regarding samples found in data file
			 */
			SamplePtrVectorPtr samples_;



		public:

			/**
			 * Constructor
			 */
			MixtureDataParser(std::string filename, std::string separator = "");

			/**
			 * Destructor
			 */
			virtual ~MixtureDataParser();

			/**
			 * Get file type name
			 * @return
			 */
			static std::string getFileType();

			/**
			 * Method responsible for performing file parsing
			 *
			 * @exception ParserException
			 */
			SamplePtrVectorPtr parse() throw (ParserException);

			/**
			 * Method returns data, which has been read from mixture sample file
			 *
			 * @return
			 */
			SamplePtrVectorPtr getSamples() const;

		protected:

			/**
			 * Method parses header row
			 *
			 * @param row
			 * @exception ParserException
			 */
			virtual void parseHeaderRow(shared_ptr<std::vector<std::string> > row) = 0;

			/**
			 * Method parses single data row
			 *
			 * @param row
			 * @param lineNumber
			 * @exception ParserException
			 */
			virtual void
					parseDataRow(shared_ptr<std::vector<std::string> > row, unsigned int lineNumber) throw (ParserException) = 0;

			/**
			 * Method parses data regarding single allele.
			 * If allele data is empty allele with value 0.0 is returned
			 *
			 * @param allelValue
			 * @param size
			 * @param peakHeight
			 * @param peakSize
			 * @return
			 * @exception ParserException
			 */
			AlleleBasePtr parseAllelData(std::string allelValue,
					std::string size,
					std::string peakHeight,
					std::string peakSize,
					unsigned int lineNumber) throw (ParserException);

			/**
			 * Method appends allele to marker with given name, withing given panel and sample
			 *
			 * @param sampleName Name of samples
			 * @param panelName	Name of the panel used
			 * @param markerName Name of the marker
			 * @param allele Parsed allele
			 * @param lineNumber Line at which allele has been found
			 */
			void addAllele(std::string sampleName,
					std::string panelName,
					std::string markerName,
					AlleleBasePtr allele,
					unsigned int lineNumber);
		};

	}

}

#endif /* MIXTUREDATAPARSER_H_ */
