#ifndef MixtureDataParserColumnarFormat_H_
#define MixtureDataParserColumnarFormat_H_

#include <vector>
#include <string>
#include <iostream>
#include <boost/lexical_cast.hpp>
#include "boost/tuple/tuple.hpp"
#include <boost/shared_ptr.hpp>
#include "ParserException.h"
#include "MixtureDataParser.h"
#include "models/Sample.h"
#include "models/Marker.h"
#include "models/NumericAllele.h"
#include "models/CharacterAllele.h"
#include "../utils/Validator.h"

using namespace boost;
using namespace dma::core;

namespace dma
{

	namespace external
	{

		class MixtureDataParserColumnarFormat;
		typedef shared_ptr<MixtureDataParserColumnarFormat> MixtureDataParserColumnarFormatPtr;

		/**
		 * Class represents parsed used for processing mixture samples.
		 * All allele data for each of the markers is stored in single file.
		 */
		class MixtureDataParserColumnarFormat: public MixtureDataParser
		{

		private:

			/**
			 *	Index of column with sample name
			 */
			unsigned int sampleColumnIndex_;

			/**
			 *	Index of column with panel name
			 */
			unsigned int panelColumnIndex_;

			/**
			 *	Index of column with marker name
			 */
			unsigned int markerColumnIndex_;

			/**
			 *	Index of column at which data starts
			 */
			unsigned int firstDataColumnIndex_;

			/**
			 *	Default column number for sample name
			 */
			static const unsigned int SAMPLE_COLUMN_INDEX_D = 0;

			/**
			 *	Default column number for panel name
			 */
			static const unsigned int PANEL_COLUMN_INDEX_D = 1;

			/**
			 *	Default colum number for marker name
			 */
			static const unsigned int MARKER_COLUMN_INDEX_D = 2;

			/**
			 *	Default column number at which data starts
			 */
			static const unsigned int FIRST_DATA_COLUMN_INDEX_D = 3;

			/**
			 * Data column order - allele value
			 */
			static const unsigned int DATA_ORDER_ALLEL_VALUE = 0;

			/**
			 * Data column order - allele size
			 */
			static const unsigned int DATA_ORDER_ALLELE_SIZE = 1;

			/**
			 * Data column order - peak height
			 */
			static const unsigned int DATA_ORDER_PEAK_HEIGHT = 2;

			/**
			 * Data column order - peak size
			 */
			static const unsigned int DATA_ORDER_PEAK_SIZE = 3;

		public:

			/**
			 * Constructor
			 *
			 * @param filename Name of the file to-be-parsed
			 * @param separator Data fields separator. If not given separator will be determined
			 * @param sampleColumnIndex Index of column with sample name
			 * @param panelColumnIndex Index of column with panel name
			 * @param markerColumnIndex Index of column with marker name
			 * @param firstDataColumnIndex Index of column at which data starts
			 * @return
			 */
			MixtureDataParserColumnarFormat(std::string filename, std::string separator = "", unsigned int sampleColumnIndex =
					SAMPLE_COLUMN_INDEX_D, unsigned int panelColumnIndex = PANEL_COLUMN_INDEX_D, unsigned int markerColumnIndex =
					MARKER_COLUMN_INDEX_D, unsigned int firstDataColumnIndex = FIRST_DATA_COLUMN_INDEX_D);

			/**
			 * Destructor
			 * @return
			 */
			virtual ~MixtureDataParserColumnarFormat();

		private:

			/**
			 * Method parses header row
			 *
			 * @param row
			 * @exception ParserException
			 */
			void parseHeaderRow(shared_ptr<std::vector<std::string> > row);

			/**
			 * Method parses single data row
			 *
			 * @param row
			 * @param lineNumber
			 * @exception ParserException
			 */
			void parseDataRow(shared_ptr<std::vector<std::string> > row, unsigned int lineNumber) throw (ParserException);

		};
	}
}

#endif /*MixtureDataParserColumnarFormat_H_*/
