/*
 * OutputDataParser.cpp
 *
 *  Created on: Dec 13, 2008
 *      Author: mariusz
 */

#include "OutputDataParser.h"

namespace dma
{

	namespace external
	{

		OutputDataParser::OutputDataParser(std::string filename, std::string separator) :
			DataParser(filename, OutputDataParser::getFileType(), separator), sample_(new SampleConfiguration(""))
		{

		}

		OutputDataParser::~OutputDataParser()
		{

		}

		std::string OutputDataParser::getFileType()
		{

			return "Deconvolution report data file";
		}

		SampleConfigurationPtr OutputDataParser::parse()
		{

			int rowCounter = 0;
			shared_ptr<std::vector<std::string> > row;

			while ((row = reader_->getLineAndExplode()) && row->size() > 0)
			{

				// Checking whether empty row has not been read
				if (row->size() > 0)
				{
					// Checking whether first row contains valid data - exceptions is not throwned
					if (rowCounter == 0)
					{

						try
						{
							parseDataRow(row, rowCounter);
						} catch (...)
						{

						}
					}
					else
					{
						parseDataRow(row, rowCounter);
					}
				}

				++rowCounter;
			}

			return getSampleConfiguration();
		}

		SampleConfigurationPtr OutputDataParser::getSampleConfiguration() const
		{

			return sample_;
		}

		void OutputDataParser::parseDataRow(shared_ptr<std::vector<std::string> > row, unsigned int lineNumber)
		{

			if (row->size() >= 6)
			{

				LocusPtr locus;
				char markerType = '0';
				int contributorsNumber = -1;

				// Fetching header data
				std::string sSampleName = row->at(SAMPLE_NAME_COLUMN_INDEX_D);
				std::string sMarkerName = row->at(MARKER_NAME_COLUMN_INDEX_D);
				std::string sMarkerType = row->at(MARKER_TYPE_COLUMN_INDEX_D);
				std::string sContributorsNumber = row->at(CONTRIBUTORS_NUMBER_COLUMN_INDEX_D);

				// Fetching status depending on whether detailed configuration status has been given ( odd number of columns )
				std::string sStatus = "";
				if (row->size() % 2 == 0)
				{

					sStatus = row->at(row->size() - 1);
				}
				else
				{

					sStatus = row->at(row->size() - 2);
				}

				/*
				 * VALIDATING DATA
				 */

				// Validating sample name
				if (sSampleName.size() == 0)
				{

					std::string errorMsg = "Sample name not given";
					throw ParserException(errorMsg, fileType_, reader_->getFilename(), lineNumber + 1);
				}

				// Validating marker name
				if (sMarkerName.size() == 0)
				{

					std::string errorMsg = "Marker name not given";
					throw ParserException(errorMsg, fileType_, reader_->getFilename(), lineNumber + 1);
				}

				// Validating marker type
				if (sMarkerType.size() == 1)
				{

					markerType = sMarkerType.at(0);
				}
				else
				{

					std::string errorMsg = "Incorrect marker type designation ( expected 'N' or 'C' symbols )";
					throw ParserException(errorMsg, fileType_, reader_->getFilename(), lineNumber + 1);
				}

				if (markerType != AlleleBase::ALLELE_TYPE_NUMERIC && markerType != AlleleBase::ALLELE_TYPE_CHARACTER)
				{

					std::string errorMsg = "Incorrect marker type designation ( expected 'N' or 'C' symbols )";
					throw ParserException(errorMsg, fileType_, reader_->getFilename(), lineNumber + 1);
				}

				// Validating contributors number
				if (sContributorsNumber.size() > 0)
				{
					try
					{
						contributorsNumber = lexical_cast<double> (sContributorsNumber);

						if (contributorsNumber <= 0)
						{
							std::string errorMsg = "Contributor's number cannot be less than one";
							throw ParserException(errorMsg, fileType_, reader_->getFilename(), lineNumber + 1);
						}
					} catch (bad_lexical_cast& e)
					{

						std::string errorMsg = "Contributor's number is not a numerical value";
						throw ParserException(errorMsg, fileType_, reader_->getFilename(), lineNumber + 1);
					}
				}
				else
				{

					std::string errorMsg = "Contributor's number has not been given";
					throw ParserException(errorMsg, fileType_, reader_->getFilename(), lineNumber + 1);
				}

				// Validating status
				if (sStatus.size() == 0)
				{

					std::string errorMsg = "Configuration status is not not present or empty";
					throw ParserException(errorMsg, fileType_, reader_->getFilename(), lineNumber + 1);
				}

				// Checking whether sample name has been checked
				if (sample_->getName().size() == 0)
				{
					sample_->setName(sSampleName);
				}

				/*
				 * CREATING MARKER CONFIGURATION
				 */
				MarkerConfigurationPtr markerConfiguration(new MarkerConfiguration(sMarkerName));

				/*
				 * FETCHING ALLELIC DATA
				 */
				int i = 0;
				int startCol;
				for (i = 0; i < contributorsNumber; i++)
				{
					startCol = FIRST_DATA_COLUMN_INDEX_D + 6 * i;
					locus = parseLocusData(row, markerType, startCol, i + 1, lineNumber);
					markerConfiguration->addLocus(locus);
				}

				// FETCHING CONFIGURATION STATUS
				MCStatusPtr status = parseStatus(sStatus, lineNumber);
				markerConfiguration->setStatus(status);

				// Adding marker configuration to sample
				sample_->addMarkerConfiguration(sMarkerName, markerConfiguration);
			}
			else
			{

				std::string errorMsg = "Inappropriate columns number: " + lexical_cast<std::string> (row->size())
						+ ", expecting minimum 6";
				throw ParserException(errorMsg, fileType_, reader_->getFilename(), lineNumber + 1);
			}
		}

		LocusPtr OutputDataParser::parseLocusData(shared_ptr<std::vector<std::string> > row,
				char markerType,
				unsigned int startCol,
				unsigned int contributorNumber,
				unsigned int lineNumber)
		{

			// Creating output locus
			LocusPtr locus;

			// Getting allele data
			std::string sAllele1Value = convertFloatingPoints(row->at(startCol));
			std::string sAllele1PeakHeight = convertFloatingPoints(row->at(startCol + 1));
			std::string sAllele1PeakArea = convertFloatingPoints(row->at(startCol + 2));
			std::string sAllele2Value = convertFloatingPoints(row->at(startCol + 3));
			std::string sAllele2PeakHeight = convertFloatingPoints(row->at(startCol + 4));
			std::string sAllele2PeakArea = convertFloatingPoints(row->at(startCol + 5));

			// Checking whether second allele is present
			bool allele2Present = false;
			if (sAllele2Value.size() > 0)
			{
				allele2Present = true;
			}

			if (markerType == AlleleBase::ALLELE_TYPE_NUMERIC)
			{

				// NUMERIC ALLELE
				try
				{
					// Parsing data into numeric form and adding allele to locus
					double allele1Value = lexical_cast<double> (sAllele1Value);
					double allele1PeakHeight = lexical_cast<double> (sAllele1PeakHeight);
					double allele1PeakArea = lexical_cast<double> (sAllele1PeakArea);
					NumericAllelePtr nap1(new NumericAllele(allele1Value, 0, allele1PeakHeight, allele1PeakArea));

					double allele2Value = lexical_cast<double> (sAllele2Value);
					double allele2PeakHeight = lexical_cast<double> (sAllele2PeakHeight);
					double allele2PeakArea = lexical_cast<double> (sAllele2PeakArea);
					NumericAllelePtr nap2(new NumericAllele(allele2Value, 0, allele2PeakHeight, allele2PeakArea));

					// Creating locus
					try
					{

						locus.reset(new Locus(nap1, nap2));
					} catch (AlleleTypeConflictException e)
					{

						std::string errorMsg = "Allele type conflict has been found for contributor "
								+ lexical_cast<std::string> (contributorNumber);
						throw ParserException(errorMsg, fileType_, reader_->getFilename(), lineNumber + 1);
					}

				} catch (bad_lexical_cast& e)
				{

					std::string errorMsg = "Incorrect format of genetic data for contributor "
							+ lexical_cast<std::string> (contributorNumber);
					throw ParserException(errorMsg, fileType_, reader_->getFilename(), lineNumber + 1);
				} catch (std::exception& e)
				{

					std::string errorMsg = "Incorrect format of genetic data for contributor "
							+ lexical_cast<std::string> (contributorNumber);
					throw ParserException(errorMsg, fileType_, reader_->getFilename(), lineNumber + 1);
				}
			}
			else
			{

				// CHARACTER ALLELE
				try
				{
					// Parsing data into numeric form and adding allele to locus
					char allele1Value = sAllele1Value.at(0);
					double allele1PeakHeight = lexical_cast<double> (sAllele1PeakHeight);
					double allele1PeakArea = lexical_cast<double> (sAllele1PeakArea);
					CharacterAllelePtr cap1(new CharacterAllele(allele1Value, allele1PeakHeight, allele1PeakArea));

					char allele2Value = sAllele2Value.at(0);
					double allele2PeakHeight = lexical_cast<double> (sAllele2PeakHeight);
					double allele2PeakArea = lexical_cast<double> (sAllele2PeakArea);
					CharacterAllelePtr cap2(new CharacterAllele(allele2Value, allele2PeakHeight, allele2PeakArea));

					// Creating locus
					try
					{

						locus.reset(new Locus(cap1, cap2));
					} catch (AlleleTypeConflictException e)
					{

						std::string errorMsg = "Allele type conflict has been found for contributor "
								+ lexical_cast<std::string> (contributorNumber);
						throw ParserException(errorMsg, fileType_, reader_->getFilename(), lineNumber + 1);
					}

				} catch (bad_lexical_cast& e)
				{

					std::string errorMsg = "Incorrect format of genetic data for contributor "
							+ lexical_cast<std::string> (contributorNumber);
					throw ParserException(errorMsg, fileType_, reader_->getFilename(), lineNumber + 1);
				} catch (std::exception& e)
				{

					std::string errorMsg = "Incorrect format of genetic data for contributor "
							+ lexical_cast<std::string> (contributorNumber);
					throw ParserException(errorMsg, fileType_, reader_->getFilename(), lineNumber + 1);
				}

			}

			return locus;
		}

		MCStatusPtr OutputDataParser::parseStatus(std::string cellValue, unsigned int lineNumber) throw (ParserException)
		{

			MCStatusPtr output;

			if (cellValue == MCStatusExcAlleleMissing::getStatusName())
			{

				NumericAllelePtr na0(new NumericAllele(0));
				output.reset(new MCStatusExcAlleleMissing(na0));
			}
			else if (cellValue == MCStatusExcDuplication::getStatusName())
			{

				output.reset(new MCStatusExcDuplication());
			}
			else if (cellValue == MCStatusExcEquivalence::getStatusName())
			{

				output.reset(new MCStatusExcEquivalence());
			}
			else if (cellValue == MCStatusExcHeterozygoticImbalance::getStatusName())
			{

				NumericAllelePtr na0(new NumericAllele(0));
				LocusPtr locus_0_0(new Locus(na0, na0));
				output.reset(new MCStatusExcHeterozygoticImbalance(locus_0_0, 0));
			}
			else if (cellValue == MCStatusExcKnownProfile::getStatusName())
			{

				NumericAllelePtr na0(new NumericAllele(0));
				LocusPtr locus_0_0(new Locus(na0, na0));
				output.reset(new MCStatusExcKnownProfile(locus_0_0));
			}
			else if (cellValue == MCStatusExcMixtureProportion::getStatusName())
			{

				NumericAllelePtr na0(new NumericAllele(0));
				LocusPtr locus_0_0(new Locus(na0, na0));
				output.reset(new MCStatusExcMixtureProportion(locus_0_0, 0, 0));
			}
			else if (cellValue == MCStatusValid::getStatusName())
			{

				output.reset(new MCStatusValid());
			}
			else
			{

				// Generating error message and exception
				std::string errorMsg = "Undefined configuration status - '" + cellValue + "'";
				throw ParserException(errorMsg, fileType_, reader_->getFilename(), lineNumber + 1);
			}

			return output;
		}

	}
}
