/*
 * OutputDataParser.h
 *
 *  Created on: Dec 13, 2008
 *      Author: mariusz
 */

#ifndef OUTPUTDATAPARSER_H_
#define OUTPUTDATAPARSER_H_

#include <string>
#include <boost/shared_ptr.hpp>
#include <boost/lexical_cast.hpp>
#include "DataParser.h"
#include "ParserException.h"
#include "models/CharacterAllele.h"
#include "models/NumericAllele.h"
#include "models/SampleConfiguration.h"
#include "models/statuses/MCStatus.h"
#include "models/statuses/MCStatusExcAlleleMissing.h"
#include "models/statuses/MCStatusExcDuplication.h"
#include "models/statuses/MCStatusExcEquivalence.h"
#include "models/statuses/MCStatusExcHeterozygoticImbalance.h"
#include "models/statuses/MCStatusExcKnownProfile.h"
#include "models/statuses/MCStatusExcMixtureProportion.h"
#include "models/statuses/MCStatusValid.h"

using namespace boost;
using namespace dma::core;

namespace dma
{

	namespace external
	{

		class OutputDataParser;
		typedef shared_ptr<OutputDataParser> OutputDataParserPtr;

		/**
		 * Class used to parse deconvolution report
		 */
		class OutputDataParser: public DataParser
		{
		private:

			/**
			 * Read sample
			 */
			SampleConfigurationPtr sample_;

			/**
			 *	Default column number for sample name
			 */
			static const unsigned int SAMPLE_NAME_COLUMN_INDEX_D = 0;

			/**
			 *	Default column number for marker name
			 */
			static const unsigned int MARKER_NAME_COLUMN_INDEX_D = 1;

			/**
			 *	Default column number for marker name
			 */
			static const unsigned int MARKER_TYPE_COLUMN_INDEX_D = 2;

			/**
			 *	Default column number for marker name
			 */
			static const unsigned int CONTRIBUTORS_NUMBER_COLUMN_INDEX_D = 4;

			/**
			 *	Default column number at which data starts
			 */
			static const unsigned int FIRST_DATA_COLUMN_INDEX_D = 5;

		public:

			/**
			 * Constructor
			 * @param filename Filename to read
			 * @param separator Data fields separator. If not given separator will be determined
			 * @return
			 */
			OutputDataParser(std::string filename, std::string separator = "");

			virtual ~OutputDataParser();

			/**
			 * Get file type name
			 * @return
			 */
			static std::string getFileType();

			/**
			 * Parse datafile
			 * @return
			 */
			SampleConfigurationPtr parse();

			/**
			 * Get sample parsed configuration
			 * @return
			 */
			SampleConfigurationPtr getSampleConfiguration() const;

		private:

			/**
			 * Method responsible for parsing single data row.
			 * @param row
			 * @param lineNumber
			 */
			void parseDataRow(shared_ptr<std::vector<std::string> > row, unsigned int lineNumber);

			/**
			 * Method responsible for parsing data regarding allele found for given contributor
			 *
			 * @param row Row data extracted from CSV file
			 * @param markerType  Marker type given as character. Valid marker/allele types are given as enumerations in @class AlleleBase class
			 * @param startCol
			 * @param lineNumber
			 * @return
			 */
			LocusPtr parseLocusData(shared_ptr<std::vector<std::string> > row,
					char markerType,
					unsigned int startCol,
					unsigned int contributorNumber,
					unsigned int lineNumber);

			/**
			 * Method responsible for parsing status code and creating status representation object
			 *
			 * @param cellValue
			 * @return
			 */
			MCStatusPtr parseStatus(std::string cellValue, unsigned int lineNumber) throw (ParserException);

		};

	}
}

#endif /* OUTPUTDATAPARSER_H_ */
