#include "PopulationDataParser.h"

namespace dma
{
	namespace external
	{

		PopulationDataParser::PopulationDataParser(std::string filename, std::string separator) :
			DataParser(filename, PopulationDataParser::getFileType(), separator), markers_(new MarkerPtrVector())
		{

		}

		PopulationDataParser::~PopulationDataParser()
		{

		}

		std::string PopulationDataParser::getFileType()
		{

			return "Population date file";
		}

		MarkerPtrVectorPtr PopulationDataParser::getMarkers() const
		{
			return this->markers_;
		}

		MarkerPtrVectorPtr PopulationDataParser::parse() throw (ParserException)
		{
			int rowCounter = 0;
			bool truncLastCol = false;
			shared_ptr<std::vector<std::string> > row;

			while ((row = reader_->getLineAndExplode()) && row->size() > 0)
			{

				// Checking whether in header row extra columns are not appended
				if (rowCounter == 0 && row->size() > 0 && row->at(row->size() - 1).length() == 0)
				{
					truncLastCol = true;
				}

				// Truncating extra empty columns at the end
				if (row->size() > 0 && truncLastCol)
				{
					row->pop_back();
				}

				if (rowCounter == 0)
				{
					parseHeaderRow(row);
				}
				else
				{
					if (row->size() > 0)
					{
						parseDataRow(row, rowCounter);
					}
				}

				++rowCounter;
			}

			return this->getMarkers();
		}

		void PopulationDataParser::parseHeaderRow(shared_ptr<std::vector<std::string> > row)
		{
			unsigned int i;
			unsigned int size = row->size();
			std::string markerName;

			// Omitting first column
			for (i = 1; i < size; i++)
			{
				markerName = this->trimQuotation(row->at(i));

				// Adding marker name
				this->markerNames_.push_back(markerName);

				// Creating empty marker
				MarkerPtr marker(new Marker(markerName));
				this->markers_->push_back(marker);
			}
		}

		void PopulationDataParser::parseDataRow(shared_ptr<std::vector<std::string> > row, unsigned int lineNumber)
				throw (ParserException)
		{

			unsigned int i;
			unsigned int size = row->size();
			std::string allelValue;
			std::string frequency;
			double dAllelValue;
			double dFrequency;
			std::string markerName;
			std::string cell;

			// Checking whether any data is given in row
			if (row->size() == 0)
			{
				return;
			}

			// Getting allele value
			allelValue = this->trimQuotation(row->at(0));
			allelValue = this->convertFloatingPoints(allelValue);
			try
			{
				dAllelValue = lexical_cast<double> (allelValue);
			} catch (bad_lexical_cast& e)
			{

				// Generating error message and exception
				std::string errorMsg = "Could not parse allele value for marker " + markerName + ": " + allelValue;
				throw ParserException(errorMsg, fileType_, reader_->getFilename(), lineNumber + 1);
			}

			for (i = 1; i < size; i++)
			{

				cell = row->at(i);
				if (cell.length() > 0)
				{

					frequency = this->trimQuotation(row->at(i));
					frequency = this->convertFloatingPoints(frequency);
					try
					{

						dFrequency = lexical_cast<double> (frequency);
					} catch (bad_lexical_cast& e)
					{

						// Generating error message and exception
						std::string errorMsg = "Could not parse allele frequency for marker " + markerName + ": " + frequency;
						throw ParserException(errorMsg, fileType_, reader_->getFilename(), lineNumber + 1);
					}

					// Checking whether given marker with given index exists
					if (i <= (markerNames_.size()))
					{

						markerName = markerNames_.at(i - 1);
					}
					else
					{

						// Generating error message and exception
						std::string errorMsg = "Could not assign frequency to any marker ( data outside marker's column's )";
						throw ParserException(errorMsg, fileType_, reader_->getFilename(), lineNumber + 1);
					}

					// Creating allele representation and adding to appropriate marker
					AlleleBasePtr allel(new NumericAllele(dAllelValue, 0, 0, 0));
					allel->setFrequency(dFrequency);
					this->addAllel(markerName, allel, lineNumber);
				}
			}
		}

		void PopulationDataParser::addAllel(std::string markerName, AlleleBasePtr allele, unsigned int lineNumber)
		{

			unsigned int i;

			MarkerPtr destMarker;
			MarkerPtr currMarker;

			// Checking whether sample does not already exists
			for (i = 0; i < markers_->size(); i++)
			{

				currMarker = markers_->at(i);
				if (currMarker->getName() == markerName)
				{

					destMarker = currMarker;
					break;
				}
			}

			// If marker with given name does not exist creating new one
			if (destMarker.get() == 0)
			{

				destMarker.reset(new Marker(markerName));
				markers_->push_back(destMarker);
			}

			// Adding allele
			try
			{

				destMarker->addAllele(allele);
			} catch (AlleleTypeConflictException e)
			{

				// Generating error message and exception
				std::string errorMsg =
						"Allele type conflict ( mixed numerical and character alleles ) have been found for marker " + markerName;
				throw ParserException(errorMsg, fileType_, reader_->getFilename(), lineNumber + 1);
			}
		}
	}
}
