#ifndef POPULATIONDATAPARSER_H_
#define POPULATIONDATAPARSER_H_

#include <vector>
#include <string>
#include <iostream>
#include "ParserException.h"
#include "DataParser.h"
#include "models/Sample.h"
#include "models/Marker.h"
#include "models/NumericAllele.h"
#include "models/CharacterAllele.h"
#include "../utils/Validator.h"
#include <boost/lexical_cast.hpp>
#include "boost/tuple/tuple.hpp"
#include <boost/shared_ptr.hpp>

using namespace boost;
using namespace dma::core;

namespace dma
{

	namespace external
	{

		class PopulationDataParser;
		typedef shared_ptr<PopulationDataParser> PopulationDataParserPtr;

		/**
		 * Class represents parser used for accessing population data - allele frequencies in population
		 *
		 * Input file syntax assumption:
		 * Allele,	MarkerName1,	MarkerName2,	MarkerName3
		 * 1,		0.1,			0.2,
		 * 2,,,0.5
		 * 3,		0.9,			0.8
		 * 4,,,0.5
		 *
		 * @exception IOException
		 * @exception FileNotFoundException
		 */
		class PopulationDataParser: public DataParser
		{
		private:

			/**
			 * Container for markers found in datafile
			 */
			MarkerPtrVectorPtr markers_;

			/**
			 * Container with list of names of all markers which are present in datafile
			 */
			std::vector<std::string> markerNames_;

		public:

			/**
			 * Constructor
			 *
			 * @param filename Name of the file with data
			 * @param separator Data fields separator. If not given separator will be determined
			 * @return
			 * @exception IOException
			 * @exception FileNotFoundException
			 */
			PopulationDataParser(std::string filename, std::string separator = "");

			virtual ~PopulationDataParser();

			/**
			 * Get file type name
			 * @return
			 */
			static std::string getFileType();

			/**
			 * Parse datafile
			 *
			 * @exception ParserException
			 * @return
			 */
			MarkerPtrVectorPtr parse() throw (ParserException);

			/**
			 * Method returns data read from datafile
			 */
			MarkerPtrVectorPtr getMarkers() const;

		private:

			/**
			 * Method parses header row
			 *
			 * @param row
			 */
			void parseHeaderRow(shared_ptr<std::vector<std::string> > row);

			/**
			 * Method parses single data row
			 *
			 * @param row
			 * @param lineNumber
			 * @exception ParserException
			 */
			void parseDataRow(shared_ptr<std::vector<std::string> > row, unsigned int lineNumber) throw (ParserException);

			/**
			 * Method appends given allele to specified marker
			 *
			 * @param markerName Name of marker to which allele should be added
			 * @param allele Allele
			 * @param lineNumber Line number at which data has been read
			 */
			void addAllel(std::string markerName, AlleleBasePtr allele, unsigned int lineNumber);

		};

	}
}
#endif /*POPULATIONDATAPARSER_H_*/
