/*
 * ProfileDataParser.cpp
 *
 *  Created on: Dec 6, 2008
 *      Author: mariusz
 */

#include "ProfileDataParser.h"

namespace dma
{

	namespace external
	{

		ProfileDataParser::ProfileDataParser(std::string filename, std::string separator) :
			DataParser(filename, ProfileDataParser::getFileType(), separator), profiles_(new ProfilePtrMap())
		{

		}

		ProfileDataParser::~ProfileDataParser()
		{

		}

		std::string ProfileDataParser::getFileType()
		{

			return "Profile data file";
		}

		ProfilePtrMapPtr ProfileDataParser::parse() throw (ParserException)
		{

			int rowCounter = 0;
			shared_ptr<std::vector<std::string> > row;

			while ((row = reader_->getLineAndExplode()) && row->size() > 0)
			{

				// Checking whether empty row has not been encounted
				if (row->size() > 0)
				{
					// Checking whether first row contains valid data - exceptions is not throwned
					if (rowCounter == 0)
					{
						try
						{
							parseDataRow(row, rowCounter);
						} catch (ParserException)
						{
						}
					}
					else
					{
						parseDataRow(row, rowCounter);
					}
				}

				++rowCounter;
			}

			// Creating profile

			return this->getProfiles();

		}

		ProfilePtrMapPtr ProfileDataParser::getProfiles() const
		{

			return profiles_;
		}

		void ProfileDataParser::parseDataRow(shared_ptr<std::vector<std::string> > row, unsigned int lineNumber)
				throw (ParserException)
		{

			// Checking whether row is of appropriate size
			if (row->size() >= 4)
			{

				// Fetching data
				std::string profileName = row->at(0);
				std::string markerName = row->at(1);
				std::string allele1Str = convertFloatingPoints(row->at(2));
				std::string allele2Str = "";
				if (row->size() > 3)
				{

					allele2Str = convertFloatingPoints(row->at(3));
				}

				// Checking allele types and creating allele representations
				AlleleBase::ALLELE_TYPE alleleType;
				if (Validator::isDouble(allele1Str) == true)
				{
					alleleType = AlleleBase::NUMERIC;

				}
				else if (allele1Str.size() == 1)
				{

					alleleType = AlleleBase::CHARACTER;
				}

				// Creating alleles representations
				AlleleBasePtr allele1;
				AlleleBasePtr allele2;

				if (alleleType == AlleleBase::NUMERIC)
				{

					// Converting to double - allele 1
					try
					{
						double dAllele1 = lexical_cast<double> (allele1Str);
						allele1.reset(new NumericAllele(dAllele1));
					} catch (bad_lexical_cast& e)
					{

						// Generating error message and exception
						std::string errorMsg = "Unable to parse allele at line  '" + allele1Str + "'";
						throw ParserException(errorMsg, fileType_, reader_->getFilename(), lineNumber + 1);
					}

					// Converting to double - allele 2
					if (row->size() > 3)
					{
						try
						{
							double dAllele2 = lexical_cast<double> (allele2Str);
							allele2.reset(new NumericAllele(dAllele2));
						} catch (bad_lexical_cast& e)
						{

							// Generating error message and exception
							std::string errorMsg = "Unable to parse allele '" + allele2Str + "'";
							throw ParserException(errorMsg, fileType_, reader_->getFilename(), lineNumber + 1);

						}
					}
				}
				else if (alleleType == AlleleBase::CHARACTER)
				{

					allele1.reset(new CharacterAllele(allele1Str.at(0)));
					if (row->size() > 3)
					{
						allele2.reset(new CharacterAllele(allele2Str.at(0)));
					}
				}
				else
				{

					// Generating error message and exception
					std::string errorMsg = "Unable to parse allele '" + allele1Str + "'";
					throw ParserException(errorMsg, fileType_, reader_->getFilename(), lineNumber + 1);
				}

				// Creating alleles and locus
				try
				{
					LocusPtr locus(new Locus(allele1, allele2));

					// Adding locus to appropriate profile
					addMarker(profileName, markerName, locus);
				} catch (AlleleTypeConflictException e)
				{

					// Generating error message and exception
					std::string errorMsg =
							"Allele type conflict ( mixed numerical and character alleles ) have been found for marker "
									+ e.getMarkerName();
					throw ParserException(errorMsg, fileType_, reader_->getFilename(), lineNumber + 1);

				}
			}
			else
			{

				// Generating error message and exception
				std::string errorMsg = "Row with data has inappropriate number of columns ( expected 4 )";
				throw ParserException(errorMsg, fileType_, reader_->getFilename(), lineNumber + 1);
			}
		}

		void ProfileDataParser::addMarker(std::string profileName, std::string markerName, LocusPtr marker)
		{

			// Checking whether profile with given name exists
			ProfilePtrMap::iterator it1 = profiles_->find(profileName);

			if (it1 == profiles_->end())
			{

				// Creating new profile
				ProfilePtr newProfile(new Profile());
				profiles_->insert(std::make_pair(profileName, newProfile));
				it1 = profiles_->find(profileName);
			}

			// Adding locus to profile
			it1->second->addMarker(markerName, marker);
		}
	}
}
