/*
 * PythonDeconvolutionController.cpp
 *
 *  Created on: Jul 30, 2009
 *      Author: mariusz
 */

#include "PythonDeconvolutionController.h"

namespace dma
{

	namespace python
	{

		PythonDeconvolutionController::PythonDeconvolutionController(std::string mdFilename,
				ExternalDataParser::MIXTURE_DATA_FILE_FORMAT mixtureDataFileFormat,
				std::string hbdFilename,
				std::string reportFilename) :
			mdFilename_(mdFilename), mixtureDataFileFormat_(mixtureDataFileFormat), hbdFilename_(hbdFilename), reportFilename_(reportFilename),
					mixtureProportions_(new MixProportionPtrMap()), cmController(new DeconvolutionController())
		{

		}

		PythonDeconvolutionController::~PythonDeconvolutionController()
		{

		}

		void PythonDeconvolutionController::addMixtureProportion(std::string mpString) throw (std::runtime_error)
		{

			// Parsing mixture proportion
			MixtureProportionPtr mp = MixtureProportion::parse(mpString);

			// Adding mixture proportion to collection
			unsigned int cn = mp->getContributorsNumber();
			mixtureProportions_->insert( std::make_pair(cn, mp));
		}

		Table PythonDeconvolutionController::deconvulate()
		{

			// Performing operations on calculation unit
			SampleConfigurationPtr configurations = cmController->doPerform(mdFilename_, mixtureDataFileFormat_, hbdFilename_, mixtureProportions_, 0.4);

			// Generating report file
			DeconvolutionReportGeneratorPtr generator(new DeconvolutionReportGenerator(reportFilename_));
			generator->generate(configurations);

			// Generating results table
			Table resultTable = generateResultTable(configurations);

			return resultTable;
		}

		Table PythonDeconvolutionController::generateResultTable(SampleConfigurationPtr configurations)
		{

			// Generating result table
			Table table1;

			// Marker configurations set
			shared_ptr<std::vector<MarkerConfigurationPtr> > markerConfigurations;

			// Getting names of marker within sample
			shared_ptr<std::vector<std::string> > markerNames = configurations->getMarkerNames();
			std::vector<std::string>::const_iterator itMarkerNames;
			std::vector<MarkerConfigurationPtr>::iterator it2;

			// Getting maximum loci number
			unsigned int maximumLociNumber = configurations->getMaximumLociNumber();

			// Generating header row
			shared_ptr<std::vector<std::string> > prefixedColumns(new std::vector<std::string>());
			prefixedColumns->push_back("Marker name");
			prefixedColumns->push_back("Configuration number");
			TableRow headerRow = generateHeaderRow(maximumLociNumber, prefixedColumns);
			table1.addRow(headerRow);

			for (itMarkerNames = markerNames->begin(); itMarkerNames != markerNames->end(); itMarkerNames++)
			{

				// Getting marker configurations
				markerConfigurations = configurations->getMarkerConfigurations(*itMarkerNames);

				// Creating prefixed columns - marker name
				prefixedColumns->clear();
				prefixedColumns->push_back(*itMarkerNames);

				//
				unsigned int c1 = 0;
				for (it2 = markerConfigurations->begin(); it2 != markerConfigurations->end(); it2++)
				{

					if ((*it2)->isValid())
					{

						// Generating data row
						TableRow dataRow = generateDataRow(*it2, c1, prefixedColumns);
						table1.addRow(dataRow);
						c1++;
					}
				}
			}

			return table1;
		}

		TableRow PythonDeconvolutionController::generateHeaderRow(unsigned int maximumLociNumber, shared_ptr<std::vector<
				std::string> > prefixedColumns)
		{

			// Creating row
			TableRow row(TableCell::CELL_TYPE_HEADER);

			// Writing file header
			std::vector<std::string>::iterator it1;
			for (it1 = prefixedColumns->begin(); it1 != prefixedColumns->end(); it1++)
			{
				row.addCell(*it1);
			}

			// Writing column headers
			unsigned int i;
			for (i = 0; i < maximumLociNumber; i++)
			{
				row.addCell("C" + lexical_cast<std::string> (i + 1) + " A1 Allele value");
				row.addCell("C" + lexical_cast<std::string> (i + 1) + " A1 Peak height");
				row.addCell("C" + lexical_cast<std::string> (i + 1) + " A1 Peak area");
				row.addCell("C" + lexical_cast<std::string> (i + 1) + " A2 Allele value");
				row.addCell("C" + lexical_cast<std::string> (i + 1) + " A2 Peak height");
				row.addCell("C" + lexical_cast<std::string> (i + 1) + " A2 Peak area");
			}

			return row;
		}

		TableRow PythonDeconvolutionController::generateDataRow(MarkerConfigurationPtr markerConfiguration,
				unsigned int configurationNumber,
				shared_ptr<std::vector<std::string> > prefixedColumns)
		{

			// Creating row
			TableRow row(TableCell::CELL_TYPE_DATA);

			// Writing prefixed columns
			std::vector<std::string>::iterator it1;
			for (it1 = prefixedColumns->begin(); it1 != prefixedColumns->end(); it1++)
			{
				row.addCell(*it1);
			}

			// Writing data
			row.addCell(lexical_cast<std::string> (configurationNumber + 1));

			unsigned int i;
			unsigned int lociNumber = markerConfiguration->getLociNumber();
			for (i = 0; i < lociNumber; i++)
			{

				LocusPtr locus = markerConfiguration->getLocus(i);
				unsigned int allelesNumber = locus->getAlleleNumber();
				if (allelesNumber == 0)
				{
					row.addCell("");
					row.addCell("");
					row.addCell("");
					row.addCell("");
					row.addCell("");
					row.addCell("");
				}
				else if (allelesNumber == 2)
				{

					row.addCell(locus->getAllele(0)->getAlleleValueAsString());
					row.addCell(dma::utils::toString(locus->getAllele(0)->getPeakHeight()));
					row.addCell(dma::utils::toString(locus->getAllele(0)->getPeakArea()));
					row.addCell(locus->getAllele(1)->getAlleleValueAsString());
					row.addCell(dma::utils::toString(locus->getAllele(1)->getPeakHeight()));
					row.addCell(dma::utils::toString(locus->getAllele(1)->getPeakArea()));
				}

			}

			return row;
		}

	}

}
