/*
 * TestConfigurationGenerator.cpp
 *
 *  Created on: Oct 5, 2008
 *      Author: mariusz
 */

#include "TestConfigurationGenerator.h"

using namespace dma;

namespace dma
{

	namespace test
	{

		TestConfigurationGenerator::TestConfigurationGenerator() :
			ConfigurationGenerator(), na10(new NumericAllele(10.0)), na11(new NumericAllele(11.0)),
					na12(new NumericAllele(12.0)), na13(new NumericAllele(13.0)), na14(new NumericAllele(14.0)),
					na15(new NumericAllele(15.0))
		{

			hbMin_->insert(std::make_pair("testMarker", 0.6));

			// Mixture proportion
			MixtureProportionPtr mp_1_1(new MixtureProportion());
			mp_1_1->addContributor(0.5);
			mp_1_1->addContributor(0.5);

		}

		TestConfigurationGenerator::~TestConfigurationGenerator()
		{

		}

		void TestConfigurationGenerator::testGeneratePairwiseCombinations()
		{

			// Generating alleles upon which pairwise combinations will be created
			shared_ptr<std::vector<AlleleBasePtr> > alleles(new std::vector<AlleleBasePtr>());
			alleles->push_back(na10);
			alleles->push_back(na11);
			alleles->push_back(na12);

			// Generating expected pairs
			AlleleBasePtrPair pair_10_10 = std::make_pair(na10->cloneToPtr(), na10->cloneToPtr());
			AlleleBasePtrPair pair_10_11 = std::make_pair(na10->cloneToPtr(), na11->cloneToPtr());
			AlleleBasePtrPair pair_10_12 = std::make_pair(na10->cloneToPtr(), na12->cloneToPtr());
			AlleleBasePtrPair pair_11_10 = std::make_pair(na11->cloneToPtr(), na10->cloneToPtr());
			AlleleBasePtrPair pair_11_11 = std::make_pair(na11->cloneToPtr(), na11->cloneToPtr());
			AlleleBasePtrPair pair_11_12 = std::make_pair(na11->cloneToPtr(), na12->cloneToPtr());
			AlleleBasePtrPair pair_12_10 = std::make_pair(na12->cloneToPtr(), na10->cloneToPtr());
			AlleleBasePtrPair pair_12_11 = std::make_pair(na12->cloneToPtr(), na11->cloneToPtr());
			AlleleBasePtrPair pair_12_12 = std::make_pair(na12->cloneToPtr(), na12->cloneToPtr());

			// Generating pairwise combinations
			shared_ptr<std::vector<AlleleBasePtrPair> > generated = this->generatePairwiseCombinations(alleles);

			// Testing
			ASSERT_EQUAL( true, testGeneratePairwiseCombinations_ContainsPair(pair_10_10, generated) );
			ASSERT_EQUAL( true, testGeneratePairwiseCombinations_ContainsPair(pair_10_11, generated) );
			ASSERT_EQUAL( true, testGeneratePairwiseCombinations_ContainsPair(pair_10_12, generated) );
			ASSERT_EQUAL( true, testGeneratePairwiseCombinations_ContainsPair(pair_11_10, generated) );
			ASSERT_EQUAL( true, testGeneratePairwiseCombinations_ContainsPair(pair_11_11, generated) );
			ASSERT_EQUAL( true, testGeneratePairwiseCombinations_ContainsPair(pair_11_12, generated) );
			ASSERT_EQUAL( true, testGeneratePairwiseCombinations_ContainsPair(pair_12_10, generated) );
			ASSERT_EQUAL( true, testGeneratePairwiseCombinations_ContainsPair(pair_12_11, generated) );
			ASSERT_EQUAL( true, testGeneratePairwiseCombinations_ContainsPair(pair_12_12, generated) );
		}

		bool TestConfigurationGenerator::testGeneratePairwiseCombinations_ContainsPair(AlleleBasePtrPair pair, shared_ptr<
				std::vector<AlleleBasePtrPair> > collection)
		{

			bool result = false;
			std::vector<AlleleBasePtrPair>::const_iterator it1;
			for (it1 = collection->begin(); it1 != collection->end(); it1++)
			{

				if ((*it1).first->getAlleleValue() == pair.first->getAlleleValue() && (*it1).second->getAlleleValue()
						== pair.second->getAlleleValue())
				{

					result = true;
					break;
				}
			}

			return result;
		}

		void TestConfigurationGenerator::testGenerateConfigurations_2A_1P()
		{

			// Setting person number
			int personNumber = 1;

			// Generating alleles upon which pairwise combinations will be created
			AlleleBasePtrVectorPtr alleles(new AlleleBasePtrVector());
			alleles->push_back(na10);
			alleles->push_back(na11);

			shared_ptr<std::vector<AlleleBasePtrPair> > pairwiseCombinations = this->generatePairwiseCombinations(alleles);
			ASSERT_EQUAL( 4, pairwiseCombinations->size() );

			// Generating expected configurations
			LocusPtr locus_10_10(new Locus(na10, na10));
			LocusPtr locus_10_11(new Locus(na10, na11));
			LocusPtr locus_11_11(new Locus(na11, na11));
			MarkerConfigurationPtr mc_10_10(new MarkerConfiguration("MarkerA"));
			mc_10_10->addLocus(locus_10_10);
			MarkerConfigurationPtr mc_10_11(new MarkerConfiguration("MarkerA"));
			mc_10_11->addLocus(locus_10_11);
			MarkerConfigurationPtr mc_11_11(new MarkerConfiguration("MarkerA"));
			mc_11_11->addLocus(locus_11_11);

			// Creating generator and generating configurations
			MarkerConfigurationPtrVectorPtr generated = this->generateConfigurations("Marker1",
					pairwiseCombinations,
					personNumber);

			ASSERT_EQUAL( 3, generated->size() );
			ASSERT_EQUAL( true, testGenerateConfigurations_ConfigurationExists(mc_10_10, generated));
			ASSERT_EQUAL( true, testGenerateConfigurations_ConfigurationExists(mc_10_11, generated));
			ASSERT_EQUAL( true, testGenerateConfigurations_ConfigurationExists(mc_11_11, generated));

		}

		void TestConfigurationGenerator::testCheckForConfigurationDuplicates()
		{

			// Generating configuration collection
			LocusPtr locus_10_11(new Locus(na10, na11));
			LocusPtr locus_12_13(new Locus(na12, na13));
			LocusPtr locus_14_15(new Locus(na14, na15));
			MarkerConfigurationPtr mc_10_11__12_13(new MarkerConfiguration("MarkerA"));
			mc_10_11__12_13->addLocus(locus_10_11);
			mc_10_11__12_13->addLocus(locus_12_13);
			MarkerConfigurationPtr mc_12_13__14_15(new MarkerConfiguration("MarkerA"));
			mc_12_13__14_15->addLocus(locus_12_13);
			mc_12_13__14_15->addLocus(locus_14_15);
			MarkerConfigurationPtrVectorPtr collection(new MarkerConfigurationPtrVector());
			collection->push_back(mc_10_11__12_13);
			collection->push_back(mc_12_13__14_15);

			// Generating searched configuration
			MarkerConfigurationPtr mc_12_13__10_11(new MarkerConfiguration("MarkerA"));
			mc_12_13__10_11->addLocus(locus_12_13);
			mc_12_13__10_11->addLocus(locus_10_11);

			// Testing
			ASSERT_EQUAL( false, this->checkForConfigurationDuplicates( collection, mc_12_13__10_11 ) );
			ASSERT_EQUAL( true, this->checkForConfigurationDuplicates( collection, mc_10_11__12_13 ) );
		}

		void TestConfigurationGenerator::testCheckForConfigurationEquivalents()
		{

			// Generating configuration collection
			LocusPtr locus_10_11(new Locus(na10, na11));
			LocusPtr locus_12_13(new Locus(na12, na13));
			LocusPtr locus_14_15(new Locus(na14, na15));
			MarkerConfigurationPtr mc_10_11__12_13(new MarkerConfiguration("MarkerA"));
			mc_10_11__12_13->addLocus(locus_10_11);
			mc_10_11__12_13->addLocus(locus_12_13);
			MarkerConfigurationPtr mc_12_13__14_15(new MarkerConfiguration("MarkerA"));
			mc_12_13__14_15->addLocus(locus_12_13);
			mc_12_13__14_15->addLocus(locus_14_15);
			MarkerConfigurationPtrVectorPtr collection(new MarkerConfigurationPtrVector());
			collection->push_back(mc_10_11__12_13);
			collection->push_back(mc_12_13__14_15);

			// Generating searched configuration
			MarkerConfigurationPtr mc_12_13__10_11(new MarkerConfiguration("MarkerA"));
			mc_12_13__10_11->addLocus(locus_12_13);
			mc_12_13__10_11->addLocus(locus_10_11);

			// Testing
			ASSERT_EQUAL( true, this->checkForConfigurationEquivalents( collection, mc_10_11__12_13) );
			ASSERT_EQUAL( true, this->checkForConfigurationEquivalents( collection, mc_12_13__10_11) );

		}

		void TestConfigurationGenerator::testFilterEquivalentConfigurations()
		{

			// Generating configurations for filtering
			LocusPtr locus_10_10(new Locus(na10, na10));
			LocusPtr locus_10_11(new Locus(na10, na11));
			MarkerConfigurationPtr mc_10_10__10_10(new MarkerConfiguration("MarkerA"));
			mc_10_10__10_10->addLocus(locus_10_10);
			mc_10_10__10_10->addLocus(locus_10_10);
			MarkerConfigurationPtr mc_10_10__10_11(new MarkerConfiguration("MarkerB"));
			mc_10_10__10_11->addLocus(locus_10_10);
			mc_10_10__10_11->addLocus(locus_10_11);
			MarkerConfigurationPtr mc_10_11__10_10(new MarkerConfiguration("MarkerC"));
			mc_10_11__10_10->addLocus(locus_10_11);
			mc_10_11__10_10->addLocus(locus_10_10);
			MarkerConfigurationPtrVectorPtr configurations(new MarkerConfigurationPtrVector());
			configurations->push_back(mc_10_10__10_10);
			configurations->push_back(mc_10_10__10_11);
			configurations->push_back(mc_10_11__10_10);
			ASSERT_EQUAL( 3, configurations->size() );

			// Fetching strategies
			StrategiesPtr strategies = this->getStrategies();

			// Filtering equivalent configurations - assign status
			strategies->setExcludedConfigurationsStrategy(Strategies::ECS_ASSIGN_STATUS);
			configurations = this->filterEquivalentConfigurations(configurations);
			ASSERT_EQUAL( 3, configurations->size() );
			ASSERT_EQUAL( "MarkerA", configurations->at(0)->getMarkerName() );
			ASSERT_EQUAL( "MarkerB", configurations->at(1)->getMarkerName() );
			ASSERT_EQUAL( "MarkerC", configurations->at(2)->getMarkerName() );
			ASSERT_EQUAL( MCStatusValid::getStatusName(), configurations->at(0)->getStatus()->getName() );
			ASSERT_EQUAL( MCStatusValid::getStatusName(), configurations->at(1)->getStatus()->getName() );
			ASSERT_EQUAL( MCStatusExcEquivalence::getStatusName(), configurations->at(2)->getStatus()->getName() );

			// Filtering equivalent configurations - exclude
			strategies->setExcludedConfigurationsStrategy(Strategies::ECS_EXCLUDE_FROM_COMBINATIONS);
			configurations = this->filterEquivalentConfigurations(configurations);
			ASSERT_EQUAL( 2, configurations->size() );
			ASSERT_EQUAL( "MarkerA", configurations->at(0)->getMarkerName() );
			ASSERT_EQUAL( "MarkerB", configurations->at(1)->getMarkerName() );
		}

		void TestConfigurationGenerator::testFilterConfigurationsByAllelePresence()
		{

			// Generating alleles which have been found on marker
			AlleleBasePtrVectorPtr allelesOnMarker(new AlleleBasePtrVector());
			allelesOnMarker->push_back(na10);
			allelesOnMarker->push_back(na11);

			// Generating configurations for filtering
			LocusPtr locus_10_10(new Locus(na10, na10));
			LocusPtr locus_10_11(new Locus(na10, na11));
			MarkerConfigurationPtr mc_10_10__10_10(new MarkerConfiguration("MarkerA"));
			mc_10_10__10_10->addLocus(locus_10_10);
			mc_10_10__10_10->addLocus(locus_10_10);
			MarkerConfigurationPtr mc_10_10__10_11(new MarkerConfiguration("MarkerB"));
			mc_10_10__10_11->addLocus(locus_10_10);
			mc_10_10__10_11->addLocus(locus_10_11);
			MarkerConfigurationPtrVectorPtr configurations(new MarkerConfigurationPtrVector());
			configurations->push_back(mc_10_10__10_10);
			configurations->push_back(mc_10_10__10_11);

			ASSERT_EQUAL( 2, configurations->size() );

			// Fetching strategies
			StrategiesPtr strategies = this->getStrategies();

			// Filtering equivalent configurations - assign status
			strategies->setExcludedConfigurationsStrategy(Strategies::ECS_ASSIGN_STATUS);
			configurations = this->filterConfigurationsByAllelePresence(allelesOnMarker, configurations);
			ASSERT_EQUAL( 2, configurations->size() );
			ASSERT_EQUAL( "MarkerA", configurations->at(0)->getMarkerName() );
			ASSERT_EQUAL( "MarkerB", configurations->at(1)->getMarkerName() );
			ASSERT_EQUAL( MCStatusExcAlleleMissing::getStatusName(), configurations->at(0)->getStatus()->getName() );
			ASSERT_EQUAL( MCStatusValid::getStatusName(), configurations->at(1)->getStatus()->getName() );

			// Filtering equivalent configurations - exclude
			strategies->setExcludedConfigurationsStrategy(Strategies::ECS_EXCLUDE_FROM_COMBINATIONS);
			configurations = this->filterConfigurationsByAllelePresence(allelesOnMarker, configurations);
			ASSERT_EQUAL( 1, configurations->size() );
			ASSERT_EQUAL( "MarkerB", configurations->at(0)->getMarkerName() );
		}

		void TestConfigurationGenerator::testGenerateConfigurations_3A_2P()
		{

			// Setting person number
			int personNumber = 2;

			// Generating alleles upon which pairwise combinations will be created
			AlleleBasePtrVectorPtr alleles(new AlleleBasePtrVector());
			alleles->push_back(na10);
			alleles->push_back(na11);
			alleles->push_back(na12);

			shared_ptr<std::vector<AlleleBasePtrPair> > pairwiseCombinations = this->generatePairwiseCombinations(alleles);
			ASSERT_EQUAL( 9, pairwiseCombinations->size() );

			// Generating expected configurations
			LocusPtr locus_10_10(new Locus(na10, na10));
			LocusPtr locus_10_11(new Locus(na10, na11));
			LocusPtr locus_10_12(new Locus(na10, na12));
			LocusPtr locus_11_10(new Locus(na11, na10));
			LocusPtr locus_11_11(new Locus(na11, na11));
			LocusPtr locus_11_12(new Locus(na11, na12));
			LocusPtr locus_12_10(new Locus(na12, na10));
			LocusPtr locus_12_11(new Locus(na12, na11));
			LocusPtr locus_12_12(new Locus(na12, na12));
			MarkerConfigurationPtr mc_10_10__10_10(new MarkerConfiguration("MarkerA"));
			mc_10_10__10_10->addLocus(locus_10_10);
			mc_10_10__10_10->addLocus(locus_10_10);
			MarkerConfigurationPtr mc_10_10__10_11(new MarkerConfiguration("MarkerA"));
			mc_10_10__10_11->addLocus(locus_10_10);
			mc_10_10__10_11->addLocus(locus_10_11);
			MarkerConfigurationPtr mc_10_10__10_12(new MarkerConfiguration("MarkerA"));
			mc_10_10__10_12->addLocus(locus_10_10);
			mc_10_10__10_12->addLocus(locus_10_12);
			MarkerConfigurationPtr mc_10_10__11_11(new MarkerConfiguration("MarkerA"));
			mc_10_10__11_11->addLocus(locus_10_10);
			mc_10_10__11_11->addLocus(locus_11_11);
			MarkerConfigurationPtr mc_10_10__11_12(new MarkerConfiguration("MarkerA"));
			mc_10_10__11_12->addLocus(locus_10_10);
			mc_10_10__11_12->addLocus(locus_11_12);
			MarkerConfigurationPtr mc_10_10__12_12(new MarkerConfiguration("MarkerA"));
			mc_10_10__12_12->addLocus(locus_10_10);
			mc_10_10__12_12->addLocus(locus_12_12);
			MarkerConfigurationPtr mc_10_11__10_10(new MarkerConfiguration("MarkerA"));
			mc_10_11__10_10->addLocus(locus_10_11);
			mc_10_11__10_10->addLocus(locus_10_10);
			MarkerConfigurationPtr mc_10_11__10_11(new MarkerConfiguration("MarkerA"));
			mc_10_11__10_11->addLocus(locus_10_11);
			mc_10_11__10_11->addLocus(locus_10_11);
			MarkerConfigurationPtr mc_10_11__10_12(new MarkerConfiguration("MarkerA"));
			mc_10_11__10_12->addLocus(locus_10_11);
			mc_10_11__10_12->addLocus(locus_10_12);
			MarkerConfigurationPtr mc_10_11__11_11(new MarkerConfiguration("MarkerA"));
			mc_10_11__11_11->addLocus(locus_10_11);
			mc_10_11__11_11->addLocus(locus_11_11);
			MarkerConfigurationPtr mc_10_11__11_12(new MarkerConfiguration("MarkerA"));
			mc_10_11__11_12->addLocus(locus_10_11);
			mc_10_11__11_12->addLocus(locus_11_12);
			MarkerConfigurationPtr mc_10_11__12_12(new MarkerConfiguration("MarkerA"));
			mc_10_11__12_12->addLocus(locus_10_11);
			mc_10_11__12_12->addLocus(locus_12_12);
			MarkerConfigurationPtr mc_10_12__10_10(new MarkerConfiguration("MarkerA"));
			mc_10_12__10_10->addLocus(locus_10_12);
			mc_10_12__10_10->addLocus(locus_10_10);
			MarkerConfigurationPtr mc_10_12__10_11(new MarkerConfiguration("MarkerA"));
			mc_10_12__10_11->addLocus(locus_10_12);
			mc_10_12__10_11->addLocus(locus_10_11);
			MarkerConfigurationPtr mc_10_12__10_12(new MarkerConfiguration("MarkerA"));
			mc_10_12__10_12->addLocus(locus_10_12);
			mc_10_12__10_12->addLocus(locus_10_12);
			MarkerConfigurationPtr mc_10_12__11_11(new MarkerConfiguration("MarkerA"));
			mc_10_12__11_11->addLocus(locus_10_12);
			mc_10_12__11_11->addLocus(locus_11_11);
			MarkerConfigurationPtr mc_10_12__11_12(new MarkerConfiguration("MarkerA"));
			mc_10_12__11_12->addLocus(locus_10_12);
			mc_10_12__11_12->addLocus(locus_11_12);
			MarkerConfigurationPtr mc_10_12__12_12(new MarkerConfiguration("MarkerA"));
			mc_10_12__12_12->addLocus(locus_10_12);
			mc_10_12__12_12->addLocus(locus_12_12);
			MarkerConfigurationPtr mc_11_11__10_10(new MarkerConfiguration("MarkerA"));
			mc_11_11__10_10->addLocus(locus_11_11);
			mc_11_11__10_10->addLocus(locus_10_10);
			MarkerConfigurationPtr mc_11_11__10_11(new MarkerConfiguration("MarkerA"));
			mc_11_11__10_11->addLocus(locus_11_11);
			mc_11_11__10_11->addLocus(locus_10_11);
			MarkerConfigurationPtr mc_11_11__10_12(new MarkerConfiguration("MarkerA"));
			mc_11_11__10_12->addLocus(locus_11_11);
			mc_11_11__10_12->addLocus(locus_10_12);
			MarkerConfigurationPtr mc_11_11__11_11(new MarkerConfiguration("MarkerA"));
			mc_11_11__11_11->addLocus(locus_11_11);
			mc_11_11__11_11->addLocus(locus_11_11);
			MarkerConfigurationPtr mc_11_11__11_12(new MarkerConfiguration("MarkerA"));
			mc_11_11__11_12->addLocus(locus_11_11);
			mc_11_11__11_12->addLocus(locus_11_12);
			MarkerConfigurationPtr mc_11_11__12_12(new MarkerConfiguration("MarkerA"));
			mc_11_11__12_12->addLocus(locus_11_11);
			mc_11_11__12_12->addLocus(locus_12_12);
			MarkerConfigurationPtr mc_11_12__10_10(new MarkerConfiguration("MarkerA"));
			mc_11_12__10_10->addLocus(locus_11_12);
			mc_11_12__10_10->addLocus(locus_10_10);
			MarkerConfigurationPtr mc_11_12__10_11(new MarkerConfiguration("MarkerA"));
			mc_11_12__10_11->addLocus(locus_11_12);
			mc_11_12__10_11->addLocus(locus_10_11);
			MarkerConfigurationPtr mc_11_12__10_12(new MarkerConfiguration("MarkerA"));
			mc_11_12__10_12->addLocus(locus_11_12);
			mc_11_12__10_12->addLocus(locus_10_12);
			MarkerConfigurationPtr mc_11_12__11_11(new MarkerConfiguration("MarkerA"));
			mc_11_12__11_11->addLocus(locus_11_12);
			mc_11_12__11_11->addLocus(locus_11_11);
			MarkerConfigurationPtr mc_11_12__11_12(new MarkerConfiguration("MarkerA"));
			mc_11_12__11_12->addLocus(locus_11_12);
			mc_11_12__11_12->addLocus(locus_11_12);
			MarkerConfigurationPtr mc_11_12__12_12(new MarkerConfiguration("MarkerA"));
			mc_11_12__12_12->addLocus(locus_11_12);
			mc_11_12__12_12->addLocus(locus_12_12);
			MarkerConfigurationPtr mc_12_12__10_10(new MarkerConfiguration("MarkerA"));
			mc_12_12__10_10->addLocus(locus_12_12);
			mc_12_12__10_10->addLocus(locus_10_10);
			MarkerConfigurationPtr mc_12_12__10_11(new MarkerConfiguration("MarkerA"));
			mc_12_12__10_11->addLocus(locus_12_12);
			mc_12_12__10_11->addLocus(locus_10_11);
			MarkerConfigurationPtr mc_12_12__10_12(new MarkerConfiguration("MarkerA"));
			mc_12_12__10_12->addLocus(locus_12_12);
			mc_12_12__10_12->addLocus(locus_10_12);
			MarkerConfigurationPtr mc_12_12__11_11(new MarkerConfiguration("MarkerA"));
			mc_12_12__11_11->addLocus(locus_12_12);
			mc_12_12__11_11->addLocus(locus_11_11);
			MarkerConfigurationPtr mc_12_12__11_12(new MarkerConfiguration("MarkerA"));
			mc_12_12__11_12->addLocus(locus_12_12);
			mc_12_12__11_12->addLocus(locus_11_12);
			MarkerConfigurationPtr mc_12_12__12_12(new MarkerConfiguration("MarkerA"));
			mc_12_12__12_12->addLocus(locus_12_12);
			mc_12_12__12_12->addLocus(locus_12_12);

			// Creating generator and generating configurations
			MarkerConfigurationPtrVectorPtr generated = this->generateConfigurations("Marker1",
					pairwiseCombinations,
					personNumber);

			ASSERT_EQUAL( 36, generated->size() );
			ASSERT_EQUAL( true, testGenerateConfigurations_ConfigurationExists(mc_10_10__10_10, generated));
			ASSERT_EQUAL( true, testGenerateConfigurations_ConfigurationExists(mc_10_10__10_11, generated));
			ASSERT_EQUAL( true, testGenerateConfigurations_ConfigurationExists(mc_10_10__10_12, generated));
			ASSERT_EQUAL( true, testGenerateConfigurations_ConfigurationExists(mc_10_10__11_11, generated));
			ASSERT_EQUAL( true, testGenerateConfigurations_ConfigurationExists(mc_10_10__11_12, generated));
			ASSERT_EQUAL( true, testGenerateConfigurations_ConfigurationExists(mc_10_10__12_12, generated));
			ASSERT_EQUAL( true, testGenerateConfigurations_ConfigurationExists(mc_10_11__10_10, generated));
			ASSERT_EQUAL( true, testGenerateConfigurations_ConfigurationExists(mc_10_11__10_11, generated));
			ASSERT_EQUAL( true, testGenerateConfigurations_ConfigurationExists(mc_10_11__10_12, generated));
			ASSERT_EQUAL( true, testGenerateConfigurations_ConfigurationExists(mc_10_11__11_11, generated));
			ASSERT_EQUAL( true, testGenerateConfigurations_ConfigurationExists(mc_10_11__11_12, generated));
			ASSERT_EQUAL( true, testGenerateConfigurations_ConfigurationExists(mc_10_11__12_12, generated));
			ASSERT_EQUAL( true, testGenerateConfigurations_ConfigurationExists(mc_10_12__10_10, generated));
			ASSERT_EQUAL( true, testGenerateConfigurations_ConfigurationExists(mc_10_12__10_11, generated));
			ASSERT_EQUAL( true, testGenerateConfigurations_ConfigurationExists(mc_10_12__10_12, generated));
			ASSERT_EQUAL( true, testGenerateConfigurations_ConfigurationExists(mc_10_12__11_11, generated));
			ASSERT_EQUAL( true, testGenerateConfigurations_ConfigurationExists(mc_10_12__11_12, generated));
			ASSERT_EQUAL( true, testGenerateConfigurations_ConfigurationExists(mc_10_12__12_12, generated));
			ASSERT_EQUAL( true, testGenerateConfigurations_ConfigurationExists(mc_11_11__10_10, generated));
			ASSERT_EQUAL( true, testGenerateConfigurations_ConfigurationExists(mc_11_11__10_11, generated));
			ASSERT_EQUAL( true, testGenerateConfigurations_ConfigurationExists(mc_11_11__10_12, generated));
			ASSERT_EQUAL( true, testGenerateConfigurations_ConfigurationExists(mc_11_11__11_11, generated));
			ASSERT_EQUAL( true, testGenerateConfigurations_ConfigurationExists(mc_11_11__11_12, generated));
			ASSERT_EQUAL( true, testGenerateConfigurations_ConfigurationExists(mc_11_11__12_12, generated));
			ASSERT_EQUAL( true, testGenerateConfigurations_ConfigurationExists(mc_11_12__10_10, generated));
			ASSERT_EQUAL( true, testGenerateConfigurations_ConfigurationExists(mc_11_12__10_11, generated));
			ASSERT_EQUAL( true, testGenerateConfigurations_ConfigurationExists(mc_11_12__10_12, generated));
			ASSERT_EQUAL( true, testGenerateConfigurations_ConfigurationExists(mc_11_12__11_11, generated));
			ASSERT_EQUAL( true, testGenerateConfigurations_ConfigurationExists(mc_11_12__11_12, generated));
			ASSERT_EQUAL( true, testGenerateConfigurations_ConfigurationExists(mc_11_12__12_12, generated));
			ASSERT_EQUAL( true, testGenerateConfigurations_ConfigurationExists(mc_12_12__10_10, generated));
			ASSERT_EQUAL( true, testGenerateConfigurations_ConfigurationExists(mc_12_12__10_11, generated));
			ASSERT_EQUAL( true, testGenerateConfigurations_ConfigurationExists(mc_12_12__10_12, generated));
			ASSERT_EQUAL( true, testGenerateConfigurations_ConfigurationExists(mc_12_12__11_11, generated));
			ASSERT_EQUAL( true, testGenerateConfigurations_ConfigurationExists(mc_12_12__11_12, generated));
			ASSERT_EQUAL( true, testGenerateConfigurations_ConfigurationExists(mc_12_12__12_12, generated));
		}

		bool TestConfigurationGenerator::testGenerateConfigurations_ConfigurationExists(MarkerConfigurationPtr configuration,
				MarkerConfigurationPtrVectorPtr collection)
		{

			bool result = false;
			int i;

			MarkerConfigurationPtrVector::const_iterator it1;
			for (it1 = collection->begin(); it1 != collection->end(); it1++)
			{

				if ((*it1)->isEqual(configuration) == true)
				{

					result = true;
					break;

				}
			}

			return result;
		}

		void TestConfigurationGenerator::testConfigurationNumber()
		{

			// Generating alleles upon which pairwise combinations will be created
			AlleleBasePtrVectorPtr alleles_1(new AlleleBasePtrVector());
			alleles_1->push_back( na10->cloneToPtr() );
			AlleleBasePtrVectorPtr alleles_2(new AlleleBasePtrVector());
			alleles_2->push_back( na10->cloneToPtr() );
			alleles_2->push_back( na11->cloneToPtr() );
			AlleleBasePtrVectorPtr alleles_3(new AlleleBasePtrVector());
			alleles_3->push_back( na10->cloneToPtr() );
			alleles_3->push_back( na11->cloneToPtr() );
			alleles_3->push_back( na12->cloneToPtr() );
			AlleleBasePtrVectorPtr alleles_4(new AlleleBasePtrVector());
			alleles_4->push_back( na10->cloneToPtr() );
			alleles_4->push_back( na11->cloneToPtr() );
			alleles_4->push_back( na12->cloneToPtr() );
			alleles_4->push_back( na13->cloneToPtr() );
			AlleleBasePtrVectorPtr alleles_5(new AlleleBasePtrVector());
			alleles_5->push_back( na10->cloneToPtr() );
			alleles_5->push_back( na11->cloneToPtr() );
			alleles_5->push_back( na12->cloneToPtr() );
			alleles_5->push_back( na13->cloneToPtr() );
			alleles_5->push_back( na14->cloneToPtr() );
			AlleleBasePtrVectorPtr alleles_6(new AlleleBasePtrVector());
			alleles_6->push_back( na10->cloneToPtr() );
			alleles_6->push_back( na11->cloneToPtr() );
			alleles_6->push_back( na12->cloneToPtr() );
			alleles_6->push_back( na13->cloneToPtr() );
			alleles_6->push_back( na14->cloneToPtr() );
			alleles_6->push_back( na15->cloneToPtr() );

			/*
			shared_ptr<std::vector<AlleleBasePtrPair> > pairwise_1 = this->generatePairwiseCombinations(alleles_1);
			shared_ptr<std::vector<AlleleBasePtrPair> > pairwise_2 = this->generatePairwiseCombinations(alleles_2);
			shared_ptr<std::vector<AlleleBasePtrPair> > pairwise_3 = this->generatePairwiseCombinations(alleles_3);
			shared_ptr<std::vector<AlleleBasePtrPair> > pairwise_4 = this->generatePairwiseCombinations(alleles_4);
			shared_ptr<std::vector<AlleleBasePtrPair> > pairwise_5 = this->generatePairwiseCombinations(alleles_5);
			shared_ptr<std::vector<AlleleBasePtrPair> > pairwise_6 = this->generatePairwiseCombinations(alleles_6);

			MarkerConfigurationPtrVectorPtr configurations_1_1 = this->generateConfigurations("MarkerA", pairwise_1, 1);
			MarkerConfigurationPtrVectorPtr configurations_2_1 = this->generateConfigurations("MarkerA", pairwise_2, 1);
			MarkerConfigurationPtrVectorPtr configurations_1_2 = this->generateConfigurations("MarkerA", pairwise_1, 2);
			MarkerConfigurationPtrVectorPtr configurations_2_2 = this->generateConfigurations("MarkerA", pairwise_2, 2);
			MarkerConfigurationPtrVectorPtr configurations_3_2 = this->generateConfigurations("MarkerA", pairwise_3, 2);
			MarkerConfigurationPtrVectorPtr configurations_4_2 = this->generateConfigurations("MarkerA", pairwise_4, 2);
			MarkerConfigurationPtrVectorPtr configurations_3_3 = this->generateConfigurations("MarkerA", pairwise_3, 3);
			MarkerConfigurationPtrVectorPtr configurations_4_3 = this->generateConfigurations("MarkerA", pairwise_4, 3);
			MarkerConfigurationPtrVectorPtr configurations_5_3 = this->generateConfigurations("MarkerA", pairwise_5, 3);
			MarkerConfigurationPtrVectorPtr configurations_6_3 = this->generateConfigurations("MarkerA", pairwise_6, 3);

			std::cout << "Number of alleles - 1, Number of persons - 1, Number of configurations - " << configurations_1_1->size() << std::endl;
			std::cout << "Number of alleles - 2, Number of persons - 1, Number of configurations - " << configurations_2_1->size() << std::endl;
			std::cout << "Number of alleles - 1, Number of persons - 2, Number of configurations - " << configurations_1_2->size() << std::endl;
			std::cout << "Number of alleles - 2, Number of persons - 2, Number of configurations - " << configurations_2_2->size() << std::endl;
			std::cout << "Number of alleles - 3, Number of persons - 2, Number of configurations - " << configurations_3_2->size() << std::endl;
			std::cout << "Number of alleles - 4, Number of persons - 2, Number of configurations - " << configurations_4_2->size() << std::endl;
			std::cout << "Number of alleles - 3, Number of persons - 3, Number of configurations - " << configurations_3_3->size() << std::endl;
			std::cout << "Number of alleles - 4, Number of persons - 3, Number of configurations - " << configurations_4_3->size() << std::endl;
			std::cout << "Number of alleles - 5, Number of persons - 3, Number of configurations - " << configurations_5_3->size() << std::endl;
			std::cout << "Number of alleles - 6, Number of persons - 3, Number of configurations - " << configurations_6_3->size() << std::endl;
			*/



		}
	}
}
