import time
import simplejson as json
import os
import sys

from dma.frontend.parsers import KnownProfileParser
from dma.frontend.models import *

import dmapython

#
# Function responsible for handling uploaded mixture deconvolution data files
# 
# @param files: Array containing uploaded files
# @param flagUseStandardHbData: Flag indicating whether to use standard HB data
# @return: Method return object of type MDDataFileUploadResult
def handleUploadedMDDataFiles(files, flagUseStandardHbData):
    
    # Creating result object
    result = MDDataFileUploadResult()
    result.saveResult = True
    
    # Generating timestamp which will be used for file naming
    strTime = time.strftime('%Y-%m-%d_%H%M%S')
    
    # Getting uploaded file
    fileMdf = files['fileMdf']    
    # Generating path
    pathToSaveMdf = settings.IMPORT_DIR_MDF + os.sep + strTime + '_' + fileMdf.name
    # Writing mixture data file
    destination = open(pathToSaveMdf, 'wb+')
    for chunk in fileMdf.chunks():
        destination.write(chunk)
    destination.close()
    # Setting data in results object
    result.pathToSavedMdf = pathToSaveMdf
    
    if flagUseStandardHbData == 'True':
        
        result.pathToSavedHbdf = settings.STANDARD_DATA_DIR + '/standard_hbmin.csv'; 
    else:
        # Getting uploaded file
        fileHbdf = files['fileHbdf']
        # Generating path
        pathToSaveHbdf = settings.IMPORT_DIR_HBDF + os.sep + strTime + '_' + fileHbdf.name
        # Writing heterozygotic balance data file
        destination = open(pathToSaveHbdf, 'wb+')
        for chunk in fileHbdf.chunks():
            destination.write(chunk)
        destination.close()
        # Setting data in results object
        result.pathToSavedHbdf = pathToSaveHbdf
    
    return result;

#
# Method responsible for parsing mixture proportions submitted by form
# Method throws RuntimeError exception
#
# @param requestPost: POST variables
# @return: Method returns array containing mixture proportions in the form 'A:B:C'
#
def parseMixtureProportions(requestPost):

    # Output array
    output = []

    maxPersonNumber = int( requestPost.get('maxPersonNumber'), 0 );
    
    # Iterating different configurations
    for i in range(2,maxPersonNumber+1):
        mixturePropotion = '';
        for j in range(1,i+1):
            
            # Getting mixture proportion for given contributor
            mp = requestPost.get('mp_' + str(i) + '_' + str(j), None );
            if mp != None:
                
                if ( len(mp) > 0 ):
                    
                    # Converting number format from poland to international
                    mp = mp.replace(',','.');
                    
                    # Attempting to parse floating point value
                    try:
                        
                        mpFloat = float(mp);
                        mixturePropotion += mp;
                        if ( j < i ):
                            mixturePropotion += ':';
                        
                    except Exception:
                        raise RuntimeError, 'Mixture proportion is in invalid format for contributor ' + str(j) + ' for configuration with ' + str(i) + ' contributors';
                else:
                    raise RuntimeError, 'Mixture proportion is not given for contributor ' + str(j) + ' for configuration with ' + str(i) + ' contributors';
            else:
                raise RuntimeError, 'Mixture proportion is not given for contributor ' + str(j) + ' for configuration with ' + str(i) + ' contributors';
        
        output.append(mixturePropotion);
            
    return output;

#
# Method for handling mixture deconvolution request send by form
#
# @param requestPost: POST variables
# @param requestFiles: FILES variables
# @return: Method returns object of type MixtureDeconvolutionResult
#
def handleMixtureDeconvolutionRequest(requestPost, requestFiles):

    # Creating result object
    result = MixtureDeconvolutionResult();
    result.flagIsError = False;
    
    # Checking whether to use standard HB data file
    useStandardHb = requestPost.get('useStandardHb');
    
    # Handling uploaded files
    uploadResult = handleUploadedMDDataFiles(requestFiles, useStandardHb);
    result.uploadResult = uploadResult;
    
    try:
        
        if ( uploadResult.saveResult == True ):
       
            # Fetching mixture proportions strings
            mixtureProportions = parseMixtureProportions(requestPost);
       
            # Generating report path
            strTime = time.strftime('%Y-%m-%d_%H%M%S');
            reportFilename = strTime + '.csv';
            pathToSaveReport = settings.REPORT_DIR + os.sep + reportFilename;
       
            # Converting to enumeration
            mdfTypeInt = requestPost.get('mdfType')
            if( mdfTypeInt == '0' ):
                mdff = dmapython.MixtureDataFileFormat.ColumnarFormat
            else:
                mdff = dmapython.MixtureDataFileFormat.LinearFormat
       
            # Performing deconvolution by library
            deconvolutionController = dmapython.PythonDeconvolutionController( str(uploadResult.pathToSavedMdf), mdff, str(uploadResult.pathToSavedHbdf), pathToSaveReport);
            
            # Adding mixture proportions
            for mixtureProportion in mixtureProportions:
                deconvolutionController.addMixtureProportion( str(mixtureProportion) );
            
            resultTable = deconvolutionController.deconvulate();
            
            # Convert results table to table which can be displayed by template system
            result.resultsTableHtml = [];
            rowsNumber = resultTable.getRowsNumber();
            if rowsNumber > 0:
                
                # Displaying row with contributor numbers
                tableRow = [];
                contributorNumber = ( resultTable.getRow(0).getCellsNumber() - 2 ) / 6;
                cell = TableCell();
                cell.type = 'D';
                cell.colspan = 2;
                cell.contents = '';
                tableRow.append(cell);
                for i in range(0, contributorNumber):
                    cell = TableCell();
                    cell.type = 'H';
                    cell.colspan = 6;
                    cell.contents = 'Contributor #' + str(i+1);
                    tableRow.append(cell);
                result.resultsTableHtml.append(tableRow);
                
                # Displaying originally reported rows
                for i in range(0, rowsNumber):
                    tableRow = [];
                    cellsNumber = resultTable.getRow(i).getCellsNumber();
                    
                    # Inserting original data reported
                    for j in range(0, cellsNumber):
                        
                        cell = TableCell();
                        cell.type = resultTable.getRow(i).getCell(j).getCellType()
                        cell.contents = resultTable.getRow(i).getCell(j).getContents();
                        tableRow.append( cell );
                    result.resultsTableHtml.append(tableRow)
            
                
            # Setting original result table to result object
            result.resultsTable = resultTable;
            
            # Setting path to report file
            result.reportFilename = reportFilename;
            
        else:
            
            # ERRORS HAS BEEN REPORTED WHILE SAVING DATA FILES
            result.flagIsError = True;
            result.errorMessage = "Couldn't save data files";
            
    except RuntimeError, e:
        
        result.flagIsError = True;
        result.errorMessage = "DnaMixtureAnalysis library has generated following error: " + e.message;
    
    return result;
