import time
import simplejson as json
import os

from dma.frontend.parsers import KnownProfileParser
from dma.frontend.models import *;

import dmapython

#
# Function responsible for handling uploaded result file from step 1 and population data
# 
# @param files: Array containing uploaded files
# @return: Method return object of type LrDataFileUploadResult
def handleUploadedLrDataFiles(requestPost, requestFiles):
    
    # Creating result object
    result = LrDataFileUploadResult()
    result.saveResult = True
    
    # Generating timestamp which will be used for file naming
    strTime = time.strftime('%Y-%m-%d_%H%M%S')
    
    # Checking whether report file has been uploaded
    reportGiven = requestPost.get('reportGiven');
    if reportGiven == "True":
        
        # Report generated in step 1 will be used
        result.pathToSavedOdf = settings.REPORT_DIR + os.sep + requestPost.get('reportFilename');
    else:
        
        # Getting uploaded file
        fileOdf = requestFiles['fileOdf'];
        
        # Generating path
        pathToSaveOdf = settings.IMPORT_DIR_ODF + os.sep + strTime + '_' + fileOdf.name;
    
        # Writing mixture data file
        destination = open(pathToSaveOdf, 'wb+')
        for chunk in fileOdf.chunks():
            destination.write(chunk)
        destination.close()
        # Setting data in results object
        result.pathToSavedOdf = pathToSaveOdf
        
    # Checking whether non-standard population data file have been uploaded
    if reportGiven == "True":
        
        result.pathToSavedPdf = settings.STANDARD_DATA_DIR + os.sep + 'allele_freq_pl.csv';
    else:
        
        # Getting uploaded file
        filePdf = requestFiles['filePdf'];
        
        # Generating path
        pathToSavePdf = settings.IMPORT_DIR_PDF + os.sep + strTime + '_' + filePdf.name;
    
        # Writing mixture data file
        destination = open(pathToSavePdf, 'wb+')
        for chunk in filePdf.chunks():
            destination.write(chunk)
        destination.close()
        # Setting data in results object
        result.pathToSavedPdf = pathToSavePdf;
    
    return result;

#
# Function responsible for handling and parsing of uploaded known profiles data file
#
# @param requestPost: HTTP request POST elements
# @param requestFiles: Uploaded files 
# @return: Function returns response object of class KpDataFileUploadResult
def handleUploadedKpDataFile(requestPost, requestFiles):
        

    # Creating response object
    response = KpDataFileUploadResult();
    
    # Fetching posted data
    response.contributorNumber = requestPost.get('kpuf[contributorNumber]', -1);
    response.hypothesis = requestPost.get('kpuf[hypothesis]','');
    
    # Fetching FILE data
    uploadedFile = requestFiles.get('kpuf[file]');
    
    # Performing file parsing and validation
    try: 
                
        parser = KnownProfileParser();
        data = parser.parse(uploadedFile);
        
        # If not exception has been reported valid response code is setted
        response.responseCode = response.RESPONSE_CODE_VALID;
        response.responseBody = data;
    except RuntimeError, e:
        
        # Exception has been reported while parsing data - error response code
        response.responseCode = response.RESPONSE_CODE_ERROR;
        response.responseBody = e.message;
        
    return response;

#
# Method parses POST data to create hypothesis with contributor and their profiles
#
# @param requestPost: Post data
# @return: Method returns dictionary with entries 'hp' and 'hd'
def parseLrCalculationPostData( requestPost ):
    
    # Prosecutor's hypothesis - list of contributors
    hp = {};
    
    # Defences's hypothesis - list of contributors
    hd = {};
    
    # Output container
    output = { 'hp' : hp, 'hd' : hd };
    
    # Proper length of data variables
    lenContributorName = len('contributor_name_hp_0');
    lenProfile = len('profile_hp_0');

    # Getting POST data keys
    postDataKeys = requestPost.keys();
      
     # FINDING CONTRIBUTOR NAMES
    for key in  postDataKeys:
        
        if key.startswith('contributor_name_') == True and len(key) == lenContributorName:
            
            ht = key[17:19];
            cn = key[20:21];
            
            # Getting contributor name
            name = requestPost.get(key);
            
            # Creating contributor
            contributor = Contributor(cn, name);
            
            # Adding contributor
            if ht == 'hp':
                hp['contributor_' + str(cn) ] = contributor;
            elif ht == 'hd':
                hd['contributor_' + str(cn) ] = contributor;
    
    # FINDING CONTRIBUTOR PROFILES
    for key in  postDataKeys:
        
        if key.startswith('profile_') == True and len(key) == lenProfile:
            
            ht = key[8:10];
            cn = key[11:12];
            
            # Getting profile in JSON format
            profileJSON = requestPost.get(key); 
            profile = json.loads( profileJSON );
            
            
            # Adding profile to appropriate contributor
            if ht == 'hp':
                
                if hp.has_key('contributor_'+str(cn)):
                    
                    contributor = hp.get('contributor_'+str(cn));
                    contributor.profile = profile;
            elif ht == 'hd':
                
                if hd.has_key('contributor_'+str(cn)):
                    contributor = hd.get('contributor_'+str(cn));
                    contributor.profile = profile;
    
    # Sorting lists
    return output;


#
# Method for handling LR calculation request send by form
#
# @param requestPost: POST variables
# @param requestFiles: FILES variables
# @return: Method returns object of type LrCalculationResult
#
def handleLrCalculationRequest(requestPost, requestFiles):
    
    # Creating result object
    result = LrCalculationResult();
    
    # Handling uploaded data
    uploadResult = handleUploadedLrDataFiles(requestPost, requestFiles);
    
    # Parsing hypothesis data
    hypothesis = parseLrCalculationPostData(requestPost);

    # Creating person and hypothesis representations for usage DMA library
    hp = dmapython.HypothesisPython('HP', 'P');
    hd = dmapython.HypothesisPython('HD', 'D');
    
    for key in hypothesis['hp']:
        
        # Fetching contributor object
        contributor = hypothesis['hp'].get(key);
        # Creating person representation which can be added to hypothesis
        person = dmapython.PersonPython( str(contributor.name), contributor.getProfileAsJson() );
        # Adding to hypothesis
        hp.addContributor(person);
                
    for key in hypothesis['hd']:
        
        # Fetching contributor object
        contributor = hypothesis['hd'].get(key);
        # Creating person representation which can be added to hypothesis
        person = dmapython.PersonPython( str(contributor.name), contributor.getProfileAsJson() );
        # Adding to hypothesis
        hd.addContributor(person);
   
    
    # Performing LR calculation by usage of library
    pdFilename = '/Applications/MAMP/htdocs/workspace_python/dma/src/dma/media/datarep/allele_freq_pl.csv';
    
    try:

        lrCalculationController = dmapython.PythonLrCalculationController( str(uploadResult.pathToSavedOdf), str(uploadResult.pathToSavedPdf), hp, hd);
        lr = lrCalculationController.calculateLr();
        markerNamesInJson = lr.getMarkerNames();
        markerNames = json.loads(markerNamesInJson);
        
        # Building up result object
        for markerName in markerNames:
            
            markerName = str(markerName);
            lrFormula = lr.getLrFormula(markerName);
            lrValue = lr.getLrValue(markerName);
            lrObj = LikelihoodRatio();
            lrObj.formula = lrFormula;
            lrObj.value = lrValue;
            result.lrs[markerName] = lrObj;
            
        result.totalLrValue = lr.getTotalLrValue();
        
    except RuntimeError, e:
        
        result.flagIsError = True;
        result.errorMessage = e.message.replace("\n", "ASDAS");
    
    return result;